/****************************************/
/*  CompareFunc.c body          	*/
/*					*/
/*  Description : comp functions for    */
/*                gtk_clist using       */
/*		  attached  FileData    */
/*					*/
/*  Written by : Daniel Lacroix	        */
/*  GFile is under the GPL2 licence     */
/****************************************/

#include "GFile.h"

/*****************************************************/
/* if string1 come first in the dictonary order then */
/* it return true else false ( or contrary ).        */
/* the maximum string length is 2048                 */
static gboolean cmp_string(gchar *string1, gchar *string2)
{ guint16 compt;
  for(compt=0;compt<2048 && string1[compt] && string2[compt];compt++)
  {
    if(string1[compt]>string2[compt]) return(TRUE);
    else if(string2[compt]>string1[compt]) return(FALSE);
  }
  if(string1[compt]) return(TRUE); else return(FALSE);
}
/*****************************************************/

/***************************************/
/* function that made all sort by name */
static gint all_sort_by_name(GtkCList *clist,gconstpointer ptr1,
  gconstpointer ptr2,gboolean dir_first,gboolean invert)
{ FileData *vFileData1, *vFileData2;

  vFileData1 = ((GtkCListRow *)ptr1)->data;
  vFileData2 = ((GtkCListRow *)ptr2)->data;

  /* "." and ".." MUST ALWAYS be on the top */    
  if(!strcmp(vFileData1->name,"."))  return(0);
  if(!strcmp(vFileData2->name,"."))  return(1);
  if(!strcmp(vFileData1->name,"..")) return(0);
  if(!strcmp(vFileData2->name,"..")) return(1);

  if(dir_first == TRUE)   
  {
    if((vFileData1->mode & FILE_DATA_IS_DIRECTORY) &&
       (vFileData2->mode & FILE_DATA_IS_DIRECTORY))
    {
      if(invert == TRUE)
        return(!(gint)cmp_string(vFileData1->name,vFileData2->name));
      else
        return((gint)cmp_string(vFileData1->name,vFileData2->name));
    }  
    else
    if(vFileData1->mode & FILE_DATA_IS_DIRECTORY)
      return(0);
    else
    if(vFileData2->mode & FILE_DATA_IS_DIRECTORY)
      return(1);
  }
  if(invert == TRUE)
    return(!(gint)cmp_string(vFileData1->name,vFileData2->name));
  else
    return((gint)cmp_string(vFileData1->name,vFileData2->name));
}			
/***************************************/

/*****************************************/
/* function that made all sort by rights */
static gint all_sort_by_rights(GtkCList *clist,gconstpointer ptr1,
  gconstpointer ptr2,gboolean dir_first,gboolean invert)
{
 FileData *vFileData1, *vFileData2;

  vFileData1 = ((GtkCListRow *)ptr1)->data;
  vFileData2 = ((GtkCListRow *)ptr2)->data;

  /* "." and ".." MUST ALWAYS be on the top */    
  if(!strcmp(vFileData1->name,"."))  return(0);
  if(!strcmp(vFileData2->name,"."))  return(1);
  if(!strcmp(vFileData1->name,"..")) return(0);
  if(!strcmp(vFileData2->name,"..")) return(1);

  if(dir_first == TRUE)
  {
    if((vFileData1->mode & FILE_DATA_IS_DIRECTORY) &&
       (vFileData2->mode & FILE_DATA_IS_DIRECTORY))
    {
      if(invert == TRUE)
        return(!(gint)(vFileData1->rights < vFileData2->rights));
      else
        return((gint)(vFileData1->rights < vFileData2->rights));
    }  
    else
    if(vFileData1->mode & FILE_DATA_IS_DIRECTORY)
      return(0);
    else
    if(vFileData2->mode & FILE_DATA_IS_DIRECTORY)
      return(1);
  }
  if(invert == TRUE)
    return(!(gint)(vFileData1->rights < vFileData2->rights));
  else
    return((gint)(vFileData1->rights < vFileData2->rights));
}
/*****************************************/

/***************************************/
/* function that made all sort by size */
static gint all_sort_by_size(GtkCList *clist,gconstpointer ptr1,
  gconstpointer ptr2,gboolean dir_first,gboolean invert)
{
 FileData *vFileData1, *vFileData2;

  vFileData1 = ((GtkCListRow *)ptr1)->data;
  vFileData2 = ((GtkCListRow *)ptr2)->data;

  /* "." and ".." MUST ALWAYS be on the top */    
  if(!strcmp(vFileData1->name,"."))  return(0);
  if(!strcmp(vFileData2->name,"."))  return(1);
  if(!strcmp(vFileData1->name,"..")) return(0);
  if(!strcmp(vFileData2->name,"..")) return(1);

  if(dir_first == TRUE)
  {
    if((vFileData1->mode & FILE_DATA_IS_DIRECTORY) &&
       (vFileData2->mode & FILE_DATA_IS_DIRECTORY))
    {
      if(invert == TRUE)
        return(!(gint)(vFileData1->size > vFileData2->size));
      else
        return((gint)(vFileData1->size > vFileData2->size));
    }  
    else
    if(vFileData1->mode & FILE_DATA_IS_DIRECTORY)
      return(0);
    else
    if(vFileData2->mode & FILE_DATA_IS_DIRECTORY)
      return(1);
  }
  if(invert == TRUE)
    return(!(gint)(vFileData1->size > vFileData2->size));
  else
    return((gint)(vFileData1->size > vFileData2->size));
}
/***************************************/

/***************************************/
/* function that made all sort nothing */
static gint all_sort_none(GtkCList *clist,gconstpointer ptr1,
  gconstpointer ptr2,gboolean dir_first,gboolean invert)
{
 FileData *vFileData1, *vFileData2;

  vFileData1 = ((GtkCListRow *)ptr1)->data;
  vFileData2 = ((GtkCListRow *)ptr2)->data;

  /* "." and ".." MUST ALWAYS be on the top */    
  if(!strcmp(vFileData1->name,"."))  return(0);
  if(!strcmp(vFileData2->name,"."))  return(1);
  if(!strcmp(vFileData1->name,"..")) return(0);
  if(!strcmp(vFileData2->name,"..")) return(1);

  if(dir_first == TRUE)
  {
    if((vFileData1->mode & FILE_DATA_IS_DIRECTORY) &&
       (vFileData2->mode & FILE_DATA_IS_DIRECTORY))
    {
      if(invert == TRUE)
        return(0);
      else
        return(1);
    }  
    else
    if(vFileData1->mode & FILE_DATA_IS_DIRECTORY)
      return(0);
    else
    if(vFileData2->mode & FILE_DATA_IS_DIRECTORY)
      return(1);
  }
  if(invert == TRUE)
    return(0);
  else
    return(1);
}
/***************************************/

/**********************************************/
/* function that made all sort by access date */
static gint all_sort_by_access_date(GtkCList *clist,gconstpointer ptr1,
  gconstpointer ptr2,gboolean dir_first,gboolean invert)
{
 FileData *vFileData1, *vFileData2;

  vFileData1 = ((GtkCListRow *)ptr1)->data;
  vFileData2 = ((GtkCListRow *)ptr2)->data;

  /* "." and ".." MUST ALWAYS be on the top */    
  if(!strcmp(vFileData1->name,"."))  return(0);
  if(!strcmp(vFileData2->name,"."))  return(1);
  if(!strcmp(vFileData1->name,"..")) return(0);
  if(!strcmp(vFileData2->name,"..")) return(1);

  if(dir_first == TRUE)
  {
    if((vFileData1->mode & FILE_DATA_IS_DIRECTORY) &&
       (vFileData2->mode & FILE_DATA_IS_DIRECTORY))
    {
      if(invert == TRUE)
        return(!(gint)(vFileData1->last_access_date > vFileData2->last_access_date));
      else
        return((gint)(vFileData1->last_access_date > vFileData2->last_access_date));
    }  
    else
    if(vFileData1->mode & FILE_DATA_IS_DIRECTORY)
      return(0);
    else
    if(vFileData2->mode & FILE_DATA_IS_DIRECTORY)
      return(1);
  }
  if(invert == TRUE)
    return(!(gint)(vFileData1->last_access_date > vFileData2->last_access_date));
  else
    return((gint)(vFileData1->last_access_date > vFileData2->last_access_date));
}
/**********************************************/

/*********************************************/
/* function that made all sort by modif date */
static gint all_sort_by_modif_date(GtkCList *clist,gconstpointer ptr1,
  gconstpointer ptr2,gboolean dir_first,gboolean invert)
{
 FileData *vFileData1, *vFileData2;

  vFileData1 = ((GtkCListRow *)ptr1)->data;
  vFileData2 = ((GtkCListRow *)ptr2)->data;

  /* "." and ".." MUST ALWAYS be on the top */    
  if(!strcmp(vFileData1->name,"."))  return(0);
  if(!strcmp(vFileData2->name,"."))  return(1);
  if(!strcmp(vFileData1->name,"..")) return(0);
  if(!strcmp(vFileData2->name,"..")) return(1);

  if(dir_first == TRUE)
  {
    if((vFileData1->mode & FILE_DATA_IS_DIRECTORY) &&
       (vFileData2->mode & FILE_DATA_IS_DIRECTORY))
    {
      if(invert == TRUE)
        return(!(gint)(vFileData1->last_modif_date > vFileData2->last_modif_date));
      else
        return((gint)(vFileData1->last_modif_date > vFileData2->last_modif_date));
    }  
    else
    if(vFileData1->mode & FILE_DATA_IS_DIRECTORY)
      return(0);
    else
    if(vFileData2->mode & FILE_DATA_IS_DIRECTORY)
      return(1);
  }
  if(invert == TRUE)
    return(!(gint)(vFileData1->last_modif_date > vFileData2->last_modif_date));
  else
    return((gint)(vFileData1->last_modif_date > vFileData2->last_modif_date));
}
/*********************************************/

/**********************************************/
/* function that made all sort by change date */
static gint all_sort_by_change_date(GtkCList *clist,gconstpointer ptr1,
  gconstpointer ptr2,gboolean dir_first,gboolean invert)
{
 FileData *vFileData1, *vFileData2;

  vFileData1 = ((GtkCListRow *)ptr1)->data;
  vFileData2 = ((GtkCListRow *)ptr2)->data;

  /* "." and ".." MUST ALWAYS be on the top */    
  if(!strcmp(vFileData1->name,"."))  return(0);
  if(!strcmp(vFileData2->name,"."))  return(1);
  if(!strcmp(vFileData1->name,"..")) return(0);
  if(!strcmp(vFileData2->name,"..")) return(1);

  if(dir_first == TRUE)
  {
    if((vFileData1->mode & FILE_DATA_IS_DIRECTORY) &&
       (vFileData2->mode & FILE_DATA_IS_DIRECTORY))
    {
      if(invert == TRUE)
        return(!(gint)(vFileData1->last_status_change_date > vFileData2->last_status_change_date));
      else
        return((gint)(vFileData1->last_status_change_date > vFileData2->last_status_change_date));
    }  
    else
    if(vFileData1->mode & FILE_DATA_IS_DIRECTORY)
      return(0);
    else
    if(vFileData2->mode & FILE_DATA_IS_DIRECTORY)
      return(1);
  }
  if(invert == TRUE)
    return(!(gint)(vFileData1->last_status_change_date > vFileData2->last_status_change_date));
  else
    return((gint)(vFileData1->last_status_change_date > vFileData2->last_status_change_date));
}
/**********************************************/

/****************************************/
/* function that made all sort by owner */
static gint all_sort_by_owner(GtkCList *clist,gconstpointer ptr1,
  gconstpointer ptr2,gboolean dir_first,gboolean invert)
{
 FileData *vFileData1, *vFileData2;

  vFileData1 = ((GtkCListRow *)ptr1)->data;
  vFileData2 = ((GtkCListRow *)ptr2)->data;

  /* "." and ".." MUST ALWAYS be on the top */    
  if(!strcmp(vFileData1->name,"."))  return(0);
  if(!strcmp(vFileData2->name,"."))  return(1);
  if(!strcmp(vFileData1->name,"..")) return(0);
  if(!strcmp(vFileData2->name,"..")) return(1);

  if(dir_first == TRUE)
  {
    if((vFileData1->mode & FILE_DATA_IS_DIRECTORY) &&
       (vFileData2->mode & FILE_DATA_IS_DIRECTORY))
    {
      if(invert == TRUE)
        return(!(gint)(vFileData1->uid > vFileData2->uid));
      else
        return((gint)(vFileData1->uid > vFileData2->uid));
    }  
    else
    if(vFileData1->mode & FILE_DATA_IS_DIRECTORY)
      return(0);
    else
    if(vFileData2->mode & FILE_DATA_IS_DIRECTORY)
      return(1);
  }
  if(invert == TRUE)
    return(!(gint)(vFileData1->uid > vFileData2->uid));
  else
    return((gint)(vFileData1->uid > vFileData2->uid));
}
/****************************************/

/****************************************/
/* function that made all sort by group */
static gint all_sort_by_group(GtkCList *clist,gconstpointer ptr1,
  gconstpointer ptr2,gboolean dir_first,gboolean invert)
{
 FileData *vFileData1, *vFileData2;

  vFileData1 = ((GtkCListRow *)ptr1)->data;
  vFileData2 = ((GtkCListRow *)ptr2)->data;

  /* "." and ".." MUST ALWAYS be on the top */    
  if(!strcmp(vFileData1->name,"."))  return(0);
  if(!strcmp(vFileData2->name,"."))  return(1);
  if(!strcmp(vFileData1->name,"..")) return(0);
  if(!strcmp(vFileData2->name,"..")) return(1);

  if(dir_first == TRUE)
  {
    if((vFileData1->mode & FILE_DATA_IS_DIRECTORY) &&
       (vFileData2->mode & FILE_DATA_IS_DIRECTORY))
    {
      if(invert == TRUE)
        return(!(gint)(vFileData1->gid > vFileData2->gid));
      else
        return((gint)(vFileData1->gid > vFileData2->gid));
    }  
    else
    if(vFileData1->mode & FILE_DATA_IS_DIRECTORY)
      return(0);
    else
    if(vFileData2->mode & FILE_DATA_IS_DIRECTORY)
      return(1);
  }
  if(invert == TRUE)
    return(!(gint)(vFileData1->gid > vFileData2->gid));
  else
    return((gint)(vFileData1->gid > vFileData2->gid));
}
/****************************************/


gint sort_none_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_none(clist,ptr1,ptr2,FALSE,FALSE)); }
gint sort_by_name_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_name(clist,ptr1,ptr2,FALSE,FALSE));}
gint sort_by_owner_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_owner(clist,ptr1,ptr2,FALSE,FALSE)); }
gint sort_by_group_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_group(clist,ptr1,ptr2,FALSE,FALSE)); }
gint sort_by_rights_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_rights(clist,ptr1,ptr2,FALSE,FALSE)); }
gint sort_by_size_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_size(clist,ptr1,ptr2,FALSE,FALSE)); }
gint sort_by_access_date_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_access_date(clist,ptr1,ptr2,FALSE,FALSE)); }
gint sort_by_modif_date_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_modif_date(clist,ptr1,ptr2,FALSE,FALSE)); }
gint sort_by_change_date_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_change_date(clist,ptr1,ptr2,FALSE,FALSE)); }


gint sort_none_inv_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_none(clist,ptr1,ptr2,FALSE,TRUE)); }
gint sort_by_name_inv_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_name(clist,ptr1,ptr2,FALSE,TRUE));}
gint sort_by_owner_inv_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_owner(clist,ptr1,ptr2,FALSE,TRUE)); }
gint sort_by_group_inv_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_group(clist,ptr1,ptr2,FALSE,TRUE)); }
gint sort_by_rights_inv_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_rights(clist,ptr1,ptr2,FALSE,TRUE)); }
gint sort_by_size_inv_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_size(clist,ptr1,ptr2,FALSE,TRUE)); }
gint sort_by_access_date_inv_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_access_date(clist,ptr1,ptr2,FALSE,TRUE)); }
gint sort_by_modif_date_inv_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_modif_date(clist,ptr1,ptr2,FALSE,TRUE)); }
gint sort_by_change_date_inv_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_change_date(clist,ptr1,ptr2,FALSE,TRUE)); }


gint sort_none_dir_first_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_none(clist,ptr1,ptr2,TRUE,FALSE)); }
gint sort_by_name_dir_first_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_name(clist,ptr1,ptr2,TRUE,FALSE));}
gint sort_by_owner_dir_first_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_owner(clist,ptr1,ptr2,TRUE,FALSE)); }
gint sort_by_group_dir_first_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_group(clist,ptr1,ptr2,TRUE,FALSE)); }
gint sort_by_rights_dir_first_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_rights(clist,ptr1,ptr2,TRUE,FALSE)); }
gint sort_by_size_dir_first_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_size(clist,ptr1,ptr2,TRUE,FALSE)); }
gint sort_by_access_date_dir_first_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_access_date(clist,ptr1,ptr2,TRUE,FALSE)); }
gint sort_by_modif_date_dir_first_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_modif_date(clist,ptr1,ptr2,TRUE,FALSE)); }
gint sort_by_change_date_dir_first_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_change_date(clist,ptr1,ptr2,TRUE,FALSE)); }


gint sort_none_inv_dir_first_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_none(clist,ptr1,ptr2,TRUE,TRUE)); }
gint sort_by_name_inv_dir_first_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_name(clist,ptr1,ptr2,TRUE,TRUE));}
gint sort_by_owner_inv_dir_first_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_owner(clist,ptr1,ptr2,TRUE,TRUE)); }
gint sort_by_group_inv_dir_first_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_group(clist,ptr1,ptr2,TRUE,TRUE)); }
gint sort_by_rights_inv_dir_first_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_rights(clist,ptr1,ptr2,TRUE,TRUE)); }
gint sort_by_size_inv_dir_first_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_size(clist,ptr1,ptr2,TRUE,TRUE)); }
gint sort_by_access_date_inv_dir_first_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_access_date(clist,ptr1,ptr2,TRUE,TRUE)); }
gint sort_by_modif_date_inv_dir_first_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_modif_date(clist,ptr1,ptr2,TRUE,TRUE)); }
gint sort_by_change_date_inv_dir_first_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_change_date(clist,ptr1,ptr2,TRUE,TRUE)); }
