/****************************************/
/*  CopyDialog.c body            	*/
/*					*/
/*  Description : Manage Copy (copy of  */
/*                files) widget		*/
/*					*/
/*  Written by : Daniel Lacroix	        */
/*  GFile is under the GPL2 licence     */
/****************************************/

#include "GFile.h"

static void copy_dialog_class_init(CopyDialogClass *pCopyDialogClass);
static void copy_dialog_init(CopyDialog *pCopyDialog);
static void destroy_cb(GtkWidget *Widget, CopyDialog *pCopyDialog);
static void cancel_cb(GtkWidget *Widget, CopyDialog *pCopyDialog);

/*************************************************************/
guint copy_dialog_get_type ()
{
  static guint copy_dialog_type = 0;

  if (!copy_dialog_type)
  {
    GtkTypeInfo copy_dialog_info =
    {
       "CopyDialog",
       sizeof (CopyDialog),
       sizeof (CopyDialogClass),
       (GtkClassInitFunc)  copy_dialog_class_init,
       (GtkObjectInitFunc) copy_dialog_init,
       (GtkArgSetFunc) NULL,
       (GtkArgGetFunc) NULL
     };
     copy_dialog_type = gtk_type_unique(gnome_dialog_get_type(), &copy_dialog_info);
  }
  return copy_dialog_type;
}
/*************************************************************/

/*************************************************************/
static void copy_dialog_class_init(CopyDialogClass *class)
{
  /* here we create the new signals if we want */
}
/*************************************************************/

void copy_current_file(CopyDialog *pCopyDialog);

static gint copy_dialog_timer_cb(CopyDialog *pCopyDialog)
{ static gfloat percent = 0.0;

  /*g_print("timer cb. \n");*/
  gtk_progress_bar_update(GTK_PROGRESS_BAR(pCopyDialog->all_file_progress_bar),
    percent);
  percent += 0.05;
  if(percent > 1.0) percent = 0.0;

}

static void list_all_we_need_to_copy_sub(CopyDialog *pCopyDialog,FileData *pFileData)
{ struct dirent **namelist;
  gint   nb_entry;
  gint   entry_pos = 0;
  gchar  *string;
  FileData *vFileData;
  
  string = g_malloc(strlen(pFileData->path)+strlen(pFileData->name)+2);
  strcpy(string,pFileData->path);
  strcat(string,pFileData->name);

  /* open the content of a dir */
  nb_entry = scandir(string, &namelist, 0, 0);
  if (nb_entry < 0)
  {g_print("COPY_DIALOG, list_all_we_need_to_copy_sub : dir not found\n"); return;} /* THE DIRECTORY DOESN'T EXIST */

  strcat(string,"/");
  while(entry_pos < nb_entry)
  {
    /* do something only if it is not . or .. */
    if(strcmp(namelist[entry_pos]->d_name,".") &&
       strcmp(namelist[entry_pos]->d_name,".."))
    {   
      vFileData = file_data_new_from_file(string,namelist[entry_pos]->d_name,NULL,NULL);
/*      if((vFileData->mode & FILE_DATA_IS_DIRECTORY) && 
         (!strcmp(string,pCopyDialog->DestPath)))
        g_print("Infinite loop.\n"); 
      else {*/
        file_data_list_append(pCopyDialog->AllCopyList, vFileData);
        pCopyDialog->current_all_file_total_size_cpt += vFileData->size;
        if(vFileData->mode & FILE_DATA_IS_DIRECTORY)
          list_all_we_need_to_copy_sub(pCopyDialog,vFileData);
/*      }	*/
      file_data_free(vFileData);
    }  
    entry_pos++;
  }
  
  g_free(string);  
  g_free(namelist);
}

static void list_all_we_need_to_copy(CopyDialog *pCopyDialog)
{/* gchar *string;*/

  pCopyDialog->AllCopyList = file_data_list_new();

  file_data_list_init(pCopyDialog->CopyList);
  while(!file_data_list_is_end(pCopyDialog->CopyList))
  { 
    /* do something only if it is not . or .. */
    if(strcmp(file_data_list_current(pCopyDialog->CopyList)->name,".") &&
       strcmp(file_data_list_current(pCopyDialog->CopyList)->name,".."))
    {
/*      string = g_malloc(strlen(file_data_list_current(pCopyDialog->CopyList)->path)+
        strlen(file_data_list_current(pCopyDialog->CopyList)->name)+2);
      strcpy(string,file_data_list_current(pCopyDialog->CopyList)->path);
      strcat(string,file_data_list_current(pCopyDialog->CopyList)->name);    
      strcat(string,"/");
      if((file_data_list_current(pCopyDialog->CopyList)->mode & FILE_DATA_IS_DIRECTORY) && 
         (!strcmp(string,pCopyDialog->DestPath)))
        g_print("Infinite loop.\n"); 
      else {*/
        file_data_list_append(pCopyDialog->AllCopyList, file_data_list_current(pCopyDialog->CopyList));
        pCopyDialog->current_all_file_total_size_cpt += file_data_list_current(pCopyDialog->CopyList)->size;
        if(file_data_list_current(pCopyDialog->CopyList)->mode & FILE_DATA_IS_DIRECTORY)
          list_all_we_need_to_copy_sub(pCopyDialog,file_data_list_current(pCopyDialog->CopyList));
/*      }	
      g_free(string);*/
    }  
    file_data_list_next(pCopyDialog->CopyList);
  }
}

static gchar *suppress_start(gchar *start,gchar *string)
{ guint16 pos = 0;
  guint16 end_pos;
  /*g_print("Start : %s, String : %s\n",start,string);*/

  while ((start[pos]) && (string[pos]) && (start[pos] == string[pos])){pos++;}
  /* if we are not at the end of start, that mean that string doesn't
     start like the "start" string. */  
  if(start[pos] != 0) {return(NULL);}
  return(&string[pos]);
}

void copy_device(CopyDialog *pCopyDialog,FileData *SrcFile,gchar *DstFile)
{ struct stat vstat;
  gchar *vSrcFile;
  
  vSrcFile = g_strconcat(SrcFile->path,SrcFile->name,NULL);
  gtk_progress_bar_update(GTK_PROGRESS_BAR(pCopyDialog->file_progress_bar),0.0);
  gtk_label_set_text(GTK_LABEL(pCopyDialog->copy_of),vSrcFile);
  gtk_label_set_text(GTK_LABEL(pCopyDialog->copy_to),DstFile);
  while(gtk_events_pending()) gtk_main_iteration();  
  lstat(vSrcFile, &vstat);
  g_free(vSrcFile);
  mknod(DstFile, vstat.st_mode, vstat.st_dev);
  gtk_progress_bar_update(GTK_PROGRESS_BAR(pCopyDialog->file_progress_bar),1.0);
  while(gtk_events_pending()) gtk_main_iteration();
}

void copy_dir(CopyDialog *pCopyDialog,FileData *SrcFile,gchar *DstFile)
{ struct stat vstat;
  gchar *vSrcFile;
  
  vSrcFile = g_strconcat(SrcFile->path,SrcFile->name,NULL);
  gtk_progress_bar_update(GTK_PROGRESS_BAR(pCopyDialog->file_progress_bar),0.0);
  gtk_label_set_text(GTK_LABEL(pCopyDialog->copy_of),vSrcFile);
  gtk_label_set_text(GTK_LABEL(pCopyDialog->copy_to),DstFile);
  while(gtk_events_pending()) gtk_main_iteration();  
  g_free(vSrcFile);
  mkdir(DstFile, file_data_mode(SrcFile) | S_IWUSR);
  gtk_progress_bar_update(GTK_PROGRESS_BAR(pCopyDialog->file_progress_bar),1.0);
  while(gtk_events_pending()) gtk_main_iteration();
}

void copy_symlink(CopyDialog *pCopyDialog,FileData *SrcFile,gchar *DstFile)
{ struct stat vstat;
  gchar *vSrcFile;
  
  vSrcFile = g_strconcat(SrcFile->path,SrcFile->name,NULL);
  gtk_progress_bar_update(GTK_PROGRESS_BAR(pCopyDialog->file_progress_bar),0.0);
  gtk_label_set_text(GTK_LABEL(pCopyDialog->copy_of),vSrcFile);
  gtk_label_set_text(GTK_LABEL(pCopyDialog->copy_to),DstFile);
  while(gtk_events_pending()) gtk_main_iteration();  
  g_free(vSrcFile);
  symlink(SrcFile->link_to_file, DstFile);
  gtk_progress_bar_update(GTK_PROGRESS_BAR(pCopyDialog->file_progress_bar),1.0);
  while(gtk_events_pending()) gtk_main_iteration();
}

void copy_regfile(CopyDialog *pCopyDialog,FileData *SrcFile,gchar *DstFile)
{
  gchar *buf;
  guint bytes_read;
  guint bytes_written;
  int   file_handle_src;
  int   file_handle_dst;
  struct stat vstat;
  gfloat percent;
  gint   wait_cpt;
  gchar *vSrcFile;
  
  vSrcFile = g_strconcat(SrcFile->path,SrcFile->name,NULL);
  gtk_progress_bar_update(GTK_PROGRESS_BAR(pCopyDialog->file_progress_bar),0.0);  
  gtk_label_set_text(GTK_LABEL(pCopyDialog->copy_of),vSrcFile);
  gtk_label_set_text(GTK_LABEL(pCopyDialog->copy_to),DstFile);
  while(gtk_events_pending()) gtk_main_iteration();
  
  lstat(vSrcFile, &vstat);
  buf = g_malloc(vstat.st_blksize); /*200*/
  if(buf == NULL){gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}
  pCopyDialog->current_file_size_cpt = 0;
  pCopyDialog->current_file_total_size_cpt = vstat.st_size;
  file_handle_src = open(vSrcFile, O_RDONLY);
  file_handle_dst = open(DstFile, O_CREAT | O_WRONLY | O_TRUNC, vstat.st_mode | S_IWUSR);
  do
  {	
    bytes_read = read(file_handle_src,buf,vstat.st_blksize); /*200*/
    bytes_written = write(file_handle_dst,buf,bytes_read);
    /* error to do when bytes_read != bytes_written */
    pCopyDialog->current_file_size_cpt += bytes_read;
    percent = ((gfloat)pCopyDialog->current_file_size_cpt) / 
      ((gfloat)pCopyDialog->current_file_total_size_cpt);
//    g_print("percent %f\n",percent);
    if(percent > 1.0) percent = 1.0; /* file as grown */
    gtk_progress_bar_update(GTK_PROGRESS_BAR(pCopyDialog->file_progress_bar),percent);
//    gtk_widget_show(pCopyDialog->file_progress_bar);
    while(gtk_events_pending()) gtk_main_iteration();

//    wait_cpt = 0xFFFFFF;
//    while(wait_cpt--);
    
  } while(bytes_read == vstat.st_blksize); /*200*/
  close(file_handle_dst);
  close(file_handle_src);
  
  g_free(buf);
  g_free(vSrcFile);
}

void copy_current_file(CopyDialog *pCopyDialog)
{ FileData *vFileData;
  gchar *DstPath;
  gchar *NewFile;
  gfloat percent;

  vFileData = file_data_list_current(pCopyDialog->AllCopyList);
  DstPath = g_strconcat(pCopyDialog->DestPath,
   suppress_start(pCopyDialog->SrcPath,vFileData->path),NULL);
  NewFile = g_strconcat(DstPath,vFileData->name,NULL);   
  
  if(vFileData-> mode & FILE_DATA_IS_DIRECTORY)
    copy_dir(pCopyDialog,vFileData,NewFile);
  else if(vFileData-> mode & FILE_DATA_IS_LINK)
    copy_symlink(pCopyDialog,vFileData,NewFile);
  else if((vFileData-> mode & FILE_DATA_IS_BLOCK_DEV) ||
     (vFileData-> mode & FILE_DATA_IS_CHAR_DEV))
    copy_device(pCopyDialog,vFileData,NewFile);
  else if (!(vFileData-> mode & FILE_DATA_IS_FIFO) &&
           !(vFileData-> mode & FILE_DATA_IS_SOCKET))
    copy_regfile(pCopyDialog,vFileData,NewFile);
  
  pCopyDialog->file_added_func(NewFile);
   
  g_free(NewFile);

  /* update the total progress bar */
  pCopyDialog->current_all_file_size_cpt += vFileData->size;
  percent = ((gfloat)pCopyDialog->current_all_file_size_cpt) / 
      ((gfloat)pCopyDialog->current_all_file_total_size_cpt);

  gtk_progress_bar_update(GTK_PROGRESS_BAR(pCopyDialog->all_file_progress_bar),percent);
  while(gtk_events_pending()) gtk_main_iteration();
   
  g_print("copy of : %s%s, to %s%s.\n",vFileData->path,vFileData->name,
    DstPath,vFileData->name);
  g_free(DstPath);    
}


/******************************************/
/* called when everything has been copied */
static gint copy_dialog_exit_timer_cb(CopyDialog *pCopyDialog)
{ gtk_timeout_remove (pCopyDialog->exit_timer);
  gnome_dialog_close (GNOME_DIALOG(pCopyDialog)); 
}
/******************************************/

/* given by copy_dialog_new as a parameter for copy_dialog_init */
static FileDataList *current_CopyList;
static gchar        *current_DestPath;
static gchar        *current_SrcPath;
static void (* current_file_added_func) (gchar *NewFile);

/*************************************************************/
static void copy_dialog_init(CopyDialog *pCopyDialog)
{ GtkWidget *label;
  GtkWidget *table;
    
  /* take parameters in account */
  pCopyDialog->CopyList = current_CopyList;
  pCopyDialog->DestPath = current_DestPath;
  pCopyDialog->SrcPath  = current_SrcPath;
  pCopyDialog->file_added_func  = current_file_added_func;
  
  gtk_window_set_title(GTK_WINDOW(pCopyDialog),"Copy");
  /* add the CANCEL button */
  gnome_dialog_append_buttons(GNOME_DIALOG(pCopyDialog),
    GNOME_STOCK_BUTTON_CANCEL,NULL);

  gtk_signal_connect(GTK_OBJECT(pCopyDialog), "destroy",
    GTK_SIGNAL_FUNC(destroy_cb),pCopyDialog);    
    
  /* boutton Cancel */
  gnome_dialog_button_connect(GNOME_DIALOG(pCopyDialog),0,
    GTK_SIGNAL_FUNC(cancel_cb),pCopyDialog);
    
  gtk_widget_realize (GTK_WIDGET(pCopyDialog));

  gtk_box_set_homogeneous(GTK_BOX(GNOME_DIALOG(pCopyDialog)->vbox),FALSE);

  /* table for packing all things */
  table = gtk_table_new(4,2,FALSE);
  gtk_widget_show(table);  
  
  gtk_table_set_row_spacings(GTK_TABLE(table),10);
  gtk_table_set_col_spacings(GTK_TABLE(table),10);  
  
  gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(pCopyDialog)->vbox),
    table,TRUE, TRUE, 2);

  /* copy of */
  label = gtk_label_new("Copy of :");
  gtk_label_set_justify (GTK_LABEL(label),GTK_JUSTIFY_LEFT);  
  gtk_widget_show(label);  
  gtk_table_attach_defaults(GTK_TABLE(table),label, 0, 1, 0, 1);
  pCopyDialog->copy_of = gtk_label_new("");
  gtk_label_set_justify (GTK_LABEL(pCopyDialog->copy_of),GTK_JUSTIFY_LEFT);  
  gtk_widget_show(pCopyDialog->copy_of);  
  gtk_table_attach_defaults(GTK_TABLE(table),pCopyDialog->copy_of,
    1, 2, 0, 1);

  /* copy to */
  label = gtk_label_new("to :");
  gtk_label_set_justify (GTK_LABEL(label),GTK_JUSTIFY_LEFT);  
  gtk_widget_show(label);
  gtk_table_attach_defaults(GTK_TABLE(table),label, 0, 1, 1, 2);
  pCopyDialog->copy_to = gtk_label_new("");
  gtk_label_set_justify (GTK_LABEL(pCopyDialog->copy_to),GTK_JUSTIFY_LEFT);  
  gtk_widget_show(pCopyDialog->copy_to);
  gtk_table_attach_defaults(GTK_TABLE(table),pCopyDialog->copy_to,
    1, 2, 1, 2);

  /* current file progress bar */
  label = gtk_label_new("current");
  gtk_label_set_justify (GTK_LABEL(label),GTK_JUSTIFY_LEFT);  
  gtk_widget_show(label);
  gtk_table_attach_defaults(GTK_TABLE(table),label, 0, 1, 2, 3);
  pCopyDialog->file_progress_bar = gtk_progress_bar_new();
    /* show the percentage */
  gtk_progress_set_show_text(GTK_PROGRESS(pCopyDialog->file_progress_bar),
    TRUE);  
  gtk_widget_show(pCopyDialog->file_progress_bar);
  gtk_table_attach_defaults(GTK_TABLE(table),pCopyDialog->file_progress_bar,
    1, 2, 2, 3);

  /* total file progress bar */
  label = gtk_label_new("total");
  gtk_label_set_justify (GTK_LABEL(label),GTK_JUSTIFY_LEFT);    
  gtk_widget_show(label);
  gtk_table_attach_defaults(GTK_TABLE(table),label, 0, 1, 3, 4);
  pCopyDialog->all_file_progress_bar = gtk_progress_bar_new();
    /* show the percentage */
  gtk_progress_set_show_text(GTK_PROGRESS(pCopyDialog->all_file_progress_bar),
    TRUE);
  gtk_widget_show(pCopyDialog->all_file_progress_bar);
  gtk_table_attach_defaults(GTK_TABLE(table),pCopyDialog->all_file_progress_bar,
    1, 2, 3, 4);
  
  /* function called each 100ms */  
//  pCopyDialog->timer = gtk_timeout_add(100,
//    (GtkFunction)(&copy_dialog_timer_cb), pCopyDialog);

  gtk_widget_realize(GTK_WIDGET(pCopyDialog));
  /* we must do this here if we want to see something */
  gtk_widget_show(GTK_WIDGET(pCopyDialog));
  
  while(gtk_events_pending())
    gtk_main_iteration();

  /* find all we have really to copy (directory...) */
  list_all_we_need_to_copy(pCopyDialog);

  while(gtk_events_pending())
    gtk_main_iteration();

  file_data_list_init(pCopyDialog->AllCopyList);
  while(!file_data_list_is_end(pCopyDialog->AllCopyList))
  { 
    copy_current_file(pCopyDialog);
    file_data_list_next(pCopyDialog->AllCopyList);
  }

  /* when copy is finished, close the dialog */
  pCopyDialog->exit_timer = gtk_timeout_add(0,
    (GtkFunction)(&copy_dialog_exit_timer_cb), pCopyDialog);
}
/*************************************************************/

/***************************************************/
/* called when the user click on the cancel button */
static void cancel_cb(GtkWidget *Widget, CopyDialog *pCopyDialog)
{ gnome_dialog_close (GNOME_DIALOG(pCopyDialog)); }
/***************************************************/

/***************************************/
/* close when the dialog box is closed */
static void destroy_cb(GtkWidget *Widget, CopyDialog *pCopyDialog)
{
//  gtk_timeout_remove (pCopyDialog->timer);

  file_data_list_free(pCopyDialog->CopyList);
  file_data_list_free(pCopyDialog->AllCopyList);  
  g_free(pCopyDialog->DestPath);
  g_free(pCopyDialog->SrcPath);
}
/***************************************/

/**************************************************/
/* Create a new GFileCopy widget                  */
/* GFileCopy keep a copy of CopyList and DestDir. */
GtkWidget *copy_dialog_new (FileDataList *CopyList,gchar *DestPath,
  void (* file_added_func) (gchar *NewFile))
{
  file_data_list_init(CopyList);
  /* if nothing to copy, exit*/
  if(file_data_list_is_end(CopyList)) return(NULL);
  /* if srcpath = dstpath, exit*/
  if(!strcmp(file_data_list_current(CopyList)->path,DestPath)) return(NULL);
  current_SrcPath = g_strdup(file_data_list_current(CopyList)->path);

  current_CopyList = file_data_list_new_copy(CopyList);  
  current_DestPath = g_strdup(DestPath); 
  current_file_added_func = file_added_func;
  return( GTK_WIDGET(gtk_type_new( copy_dialog_get_type() )) );
}
/**************************************************/
