/****************************************/
/*  DelDialog.c headers         	*/
/*					*/
/*  Description : Manage Remove (rem of */
/*                files) widget		*/
/*					*/
/*  Written by : Daniel Lacroix	        */
/*  GFile is under the GPL2 licence     */
/****************************************/

#include "GFile.h"

static void del_dialog_class_init(DelDialogClass *pDelDialogClass);
static void del_dialog_init(DelDialog *pDelDialog);
static void destroy_cb(GtkWidget *Widget, DelDialog *pDelDialog);
static void cancel_cb(GtkWidget *Widget, DelDialog *pDelDialog);

/*************************************************************/
guint del_dialog_get_type ()
{
  static guint del_dialog_type = 0;

  if (!del_dialog_type)
  {
    GtkTypeInfo del_dialog_info =
    {
       "DelDialog",
       sizeof (DelDialog),
       sizeof (DelDialogClass),
       (GtkClassInitFunc)  del_dialog_class_init,
       (GtkObjectInitFunc) del_dialog_init,
       (GtkArgSetFunc) NULL,
       (GtkArgGetFunc) NULL
     };
     del_dialog_type = gtk_type_unique(gnome_dialog_get_type(), &del_dialog_info);
  }
  return del_dialog_type;
}
/*************************************************************/

/*************************************************************/
static void del_dialog_class_init(DelDialogClass *class)
{
  /* here we create the new signals if we want */
}
/*************************************************************/

static void list_all_we_need_to_del_sub(DelDialog *pDelDialog,FileData *pFileData)
{ struct dirent **namelist;
  gint   nb_entry;
  gint   entry_pos = 0;
  gchar  *string;
  FileData *vFileData;
  
  string = g_malloc(strlen(pFileData->path)+strlen(pFileData->name)+2);
  strcpy(string,pFileData->path);
  strcat(string,pFileData->name);

  /* open the content of a dir */
  nb_entry = scandir(string, &namelist, 0, 0);
  if (nb_entry < 0)
  {g_print("DEL_DIALOG, list_all_we_need_to_del_sub : dir not found\n"); return;} /* THE DIRECTORY DOESN'T EXIST */

  strcat(string,"/");
  while(entry_pos < nb_entry)
  {
    /* do something only if it is not . or .. */
    if(strcmp(namelist[entry_pos]->d_name,".") &&
       strcmp(namelist[entry_pos]->d_name,".."))
    {   
      vFileData = file_data_new_from_file(string,namelist[entry_pos]->d_name,NULL,NULL);

      if(vFileData->mode & FILE_DATA_IS_DIRECTORY)
        list_all_we_need_to_del_sub(pDelDialog,vFileData);

      file_data_list_append(pDelDialog->AllDelList, vFileData);
      pDelDialog->current_all_file_total_nb_cpt++;

      file_data_free(vFileData);
    }  
    entry_pos++;
  }
  
  g_free(string);  
  g_free(namelist);
}

static void list_all_we_need_to_del(DelDialog *pDelDialog)
{
  pDelDialog->AllDelList = file_data_list_new();

  file_data_list_init(pDelDialog->DelList);
  while(!file_data_list_is_end(pDelDialog->DelList))
  { 
    /* do something only if it is not . or .. */
    if(strcmp(file_data_list_current(pDelDialog->DelList)->name,".") &&
       strcmp(file_data_list_current(pDelDialog->DelList)->name,".."))
    {
      pDelDialog->current_all_file_total_nb_cpt++;
      if(file_data_list_current(pDelDialog->DelList)->mode & FILE_DATA_IS_DIRECTORY)
        list_all_we_need_to_del_sub(pDelDialog,file_data_list_current(pDelDialog->DelList));
      file_data_list_append(pDelDialog->AllDelList, file_data_list_current(pDelDialog->DelList));
    }  
    file_data_list_next(pDelDialog->DelList);
  }
}

void del_current_file(DelDialog *pDelDialog)
{ FileData *vFileData;
  gchar *LastFile;
  gfloat percent;

  vFileData = file_data_list_current(pDelDialog->AllDelList);
  LastFile = g_strconcat(vFileData->path,vFileData->name,NULL);   

  pDelDialog->file_del_func(LastFile);
  
  remove(LastFile);   
   
  g_free(LastFile);

  /* update the total progress bar */
  pDelDialog->current_all_file_nb_cpt++;
  percent = ((gfloat)pDelDialog->current_all_file_nb_cpt) / 
      ((gfloat)pDelDialog->current_all_file_total_nb_cpt);

  gtk_progress_bar_update(GTK_PROGRESS_BAR(pDelDialog->all_file_progress_bar),percent);
  while(gtk_events_pending()) gtk_main_iteration();

}


/*******************************************/
/* called when everything has been removed */
static gint del_dialog_exit_timer_cb(DelDialog *pDelDialog)
{ gtk_timeout_remove (pDelDialog->exit_timer);
  gnome_dialog_close (GNOME_DIALOG(pDelDialog)); 
}
/*******************************************/

/* given by copy_dialog_new as a parameter for copy_dialog_init */
static FileDataList *current_DelList;
static void (* current_file_del_func) (gchar *LastFile);

/*************************************************************/
static void del_dialog_init(DelDialog *pDelDialog)
{ GtkWidget *label;
  GtkWidget *table;
    
  /* take parameters in account */
  pDelDialog->DelList = current_DelList;
  pDelDialog->file_del_func  = current_file_del_func;
  
  gtk_window_set_title(GTK_WINDOW(pDelDialog),"Del");
  /* add the CANCEL button */
  gnome_dialog_append_buttons(GNOME_DIALOG(pDelDialog),
    GNOME_STOCK_BUTTON_CANCEL,NULL);

  gtk_signal_connect(GTK_OBJECT(pDelDialog), "destroy",
    GTK_SIGNAL_FUNC(destroy_cb),pDelDialog);    
    
  /* boutton Cancel */
  gnome_dialog_button_connect(GNOME_DIALOG(pDelDialog),0,
    GTK_SIGNAL_FUNC(cancel_cb),pDelDialog);
    
  gtk_widget_realize (GTK_WIDGET(pDelDialog));

  gtk_box_set_homogeneous(GTK_BOX(GNOME_DIALOG(pDelDialog)->vbox),FALSE);

  /* table for packing all things */
  table = gtk_table_new(2,2,FALSE);
  gtk_widget_show(table);  
  
  gtk_table_set_row_spacings(GTK_TABLE(table),10);
  gtk_table_set_col_spacings(GTK_TABLE(table),10);  
  
  gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(pDelDialog)->vbox),
    table,TRUE, TRUE, 2);

  /* del of */
  label = gtk_label_new("Del of :");
  gtk_label_set_justify (GTK_LABEL(label),GTK_JUSTIFY_LEFT);  
  gtk_widget_show(label);  
  gtk_table_attach_defaults(GTK_TABLE(table),label, 0, 1, 0, 1);
  pDelDialog->del_of = gtk_label_new("");
  gtk_label_set_justify (GTK_LABEL(pDelDialog->del_of),GTK_JUSTIFY_LEFT);  
  gtk_widget_show(pDelDialog->del_of);  
  gtk_table_attach_defaults(GTK_TABLE(table),pDelDialog->del_of,
    1, 2, 0, 1);

  /* total file progress bar */
  label = gtk_label_new("total");
  gtk_label_set_justify (GTK_LABEL(label),GTK_JUSTIFY_LEFT);    
  gtk_widget_show(label);
  gtk_table_attach_defaults(GTK_TABLE(table),label, 0, 1, 1, 2);
  pDelDialog->all_file_progress_bar = gtk_progress_bar_new();
    /* show the percentage */
  gtk_progress_set_show_text(GTK_PROGRESS(pDelDialog->all_file_progress_bar),
    TRUE);
  gtk_widget_show(pDelDialog->all_file_progress_bar);
  gtk_table_attach_defaults(GTK_TABLE(table),pDelDialog->all_file_progress_bar,
    1, 2, 1, 2);
  
  gtk_widget_realize(GTK_WIDGET(pDelDialog));
  /* we must do this here if we want to see something */
  gtk_widget_show(GTK_WIDGET(pDelDialog));
  
  while(gtk_events_pending()) gtk_main_iteration();

  /* find all we have really to del (directory...) */
  list_all_we_need_to_del(pDelDialog);

  while(gtk_events_pending()) gtk_main_iteration();

  file_data_list_init(pDelDialog->AllDelList);
  while(!file_data_list_is_end(pDelDialog->AllDelList))
  { 
    del_current_file(pDelDialog);
    file_data_list_next(pDelDialog->AllDelList);
  }

  /* when del is finished, close the dialog */
  pDelDialog->exit_timer = gtk_timeout_add(0,
    (GtkFunction)(&del_dialog_exit_timer_cb), pDelDialog);
}
/*************************************************************/

/***************************************************/
/* called when the user click on the cancel button */
static void cancel_cb(GtkWidget *Widget, DelDialog *pDelDialog)
{ gnome_dialog_close (GNOME_DIALOG(pDelDialog)); }
/***************************************************/

/***************************************/
/* close when the dialog box is closed */
static void destroy_cb(GtkWidget *Widget, DelDialog *pDelDialog)
{
  file_data_list_free(pDelDialog->DelList);
  file_data_list_free(pDelDialog->AllDelList);
}
/***************************************/

/**************************************************/
/* Create a new Del widget                        */
/* DelDialog keep a copy of CopyList and DestDir. */
void del_dialog_new (FileDataList *DelList,
  void (* file_del_func) (gchar *LastFile))
{ GtkWidget *Widget;

  file_data_list_init(DelList);
  /* if nothing to del, exit*/
  if(file_data_list_is_end(DelList)) return;

  current_DelList = file_data_list_new_copy(DelList);  
  current_file_del_func = file_del_func;
  Widget = GTK_WIDGET(gtk_type_new( del_dialog_get_type() ));
}
/**************************************************/
