/****************************************/
/*  FileData.c headers         	        */
/*					*/
/*  Description : Datas attached t each */
/*		  file. 		*/
/*					*/
/*  Written by : Daniel Lacroix	        */
/*  GFile is under the GPL2 licence     */
/****************************************/

#include "GFile.h"

/*************************************************************/
/* create a new data 'ready to attach' to a clist row        */
/* keep is own copy of path and name. You can free to origin */
/* return : NULL if impossible.				     */
FileData *file_data_new(gchar *path,gchar *name,guint16 rights,
  guint32 uid,guint32 gid,guint64 size,guint64 last_access_date,guint64 last_modif_date,
  guint64 last_status_change_date,guint16  mode,file_type *type,gchar *link_to_file)
{ FileData *vFileData = NULL;
  vFileData = (FileData *)g_malloc(sizeof(FileData));
  if(vFileData == NULL){gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}
  vFileData->rights      = rights;
  vFileData->uid         = uid;
  vFileData->gid         = gid;
  vFileData->size        = size;
  vFileData->last_access_date = last_access_date;
  vFileData->last_modif_date  = last_modif_date;
  vFileData->last_status_change_date   = last_status_change_date;
  vFileData->mode        = mode;
  vFileData->type        = type;
  vFileData->path = (gchar *)g_malloc(strlen(path)+1);
  if(vFileData->path == NULL)
  {g_free(vFileData); gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}  
  strcpy(vFileData->path,path);
  vFileData->name = (gchar *)g_malloc(strlen(name)+1);
  if(vFileData->name == NULL)
  {
    g_free(vFileData); g_free(vFileData->path); gfile_error(MEM_ALLOCATION);
    exit(MEM_ALLOCATION);
  }  
  strcpy(vFileData->name,name);

  if( (mode&FILE_DATA_IS_LINK) && link_to_file)
  { /* if there is a name where the link go. Possible if we have a link. */
    vFileData->link_to_file = (gchar *)g_malloc(strlen(link_to_file)+1);
    if(vFileData->link_to_file == NULL)
    { g_free(vFileData); g_free(vFileData->path); 
      g_free(vFileData->link_to_file);
      gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);
     }  
    strcpy(vFileData->link_to_file,link_to_file);
  } else vFileData->link_to_file = NULL;
  return vFileData;
}  
/*************************************************************/  

/****************************/
/* free a FileData          */
/* the file type is not free*/
gboolean file_data_free(FileData *pFileData)
{
  if(pFileData->path != NULL) g_free(pFileData->path); 
  if(pFileData->name != NULL) g_free(pFileData->name);
  if(pFileData->link_to_file != NULL) g_free(pFileData->link_to_file);
  g_free(pFileData);
  return(TRUE);
}
/****************************/

/*******************************/
/* Create a copy of pFileData  */
/* return : null if impossible */
FileData *file_data_copy(FileData *pFileData)
{
  return file_data_new(pFileData->path,pFileData->name,
         pFileData->rights,pFileData->uid,pFileData->gid,
	 pFileData->size,pFileData->last_access_date,
	 pFileData->last_modif_date,pFileData->last_status_change_date,
	 pFileData->mode,pFileData->type,pFileData->link_to_file);
}
/*******************************/

/**********************************************/
/* Return TRUE if p1FileData and p2FileData   */
/* indicate the same file. Else FALSE.        */
gboolean file_data_is_equal(FileData *p1FileData,FileData *p2FileData)
{
  if(strcmp(p1FileData->path,p2FileData->path)) return FALSE;
  if(strcmp(p1FileData->name,p2FileData->name)) return FALSE;
  if(p1FileData->rights != p2FileData->rights) return FALSE;
  if(p1FileData->uid != p2FileData->uid) return FALSE;
  if(p1FileData->gid != p2FileData->gid) return FALSE;
  if(p1FileData->size != p2FileData->size) return FALSE; 
  /* test last access date only if it is not a link, because
     when GFile see a link, it read the link (readlink) so the
     last access date is changed. Same file will then be considere
     as a different file if we take last access date in account */
  if(!(p1FileData->mode & FILE_DATA_IS_LINK))
    if(p1FileData->last_access_date != p2FileData->last_access_date) return FALSE; 
  if(p1FileData->last_modif_date != p2FileData->last_modif_date) return FALSE;     
  if(p1FileData->last_status_change_date != p2FileData->last_status_change_date) return FALSE;
  if(p1FileData->mode != p2FileData->mode) return FALSE;
  if(p1FileData->type != p2FileData->type) return FALSE;
  if((p1FileData->link_to_file == NULL) && 
     (p2FileData->link_to_file != NULL)) return FALSE;
  if((p2FileData->link_to_file == NULL) && 
     (p1FileData->link_to_file != NULL)) return FALSE;
  if((p1FileData->link_to_file == NULL) && 
     (p2FileData->link_to_file == NULL)) return TRUE;   
  if(strcmp(p1FileData->link_to_file,p2FileData->link_to_file)) return FALSE;
  return TRUE;
}
/**********************************************/

/********************************************************************/
/* create a new file data filled with the info of file "file_name"  */
/* in directory "file_path" (file_path is ALWAYS ended by a /).     */
/* The corresponding file_type is attached to the file if pFileType */
/* is not NULL (pFileType is a list of all the define file_type)    */
/* pOwner is the list of all the known user on this computer, if    */
/* NULL then we don't check the FILE_DATA_IS_ALLOW_TO_EXEC          */
/* The FileData allocated can't be NULL, error is raised if needed  */
FileData *file_data_new_from_file(gchar *file_path,gchar *file_name,
  FileType *pFileType,Owner *pOwner)
{
  struct stat *danstat;
  
  guint16  mode;
  FileData *vFileData;
  gint nb_car_in_link;
  guint link_buffer_size = 50;

  gchar *link_to_file = NULL; /* link to file */
  gchar *currentfich;

  /* alloc some mem for detail on file  */
  danstat = (struct stat *)g_malloc(sizeof(struct stat));
  if(danstat == NULL) {gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}

  mode = 0;
  currentfich = g_malloc(strlen(file_path)+strlen(file_name)+1);
  if(currentfich == NULL) {gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}
  strcpy(currentfich,file_path);
  strcat(currentfich,file_name);
  if(lstat(currentfich,danstat))
  { /* IMPROVEMENT : define a broken TYPE */
    g_print("ATTENTION : stat error on file %s.\n",currentfich);
    mode = mode | FILE_DATA_NO_EXTENDED_INFO;
  } else 
  {
    if( danstat->st_mode & S_ISUID ) mode = mode | FILE_DATA_IS_UID_BIT;
    if( danstat->st_mode & S_ISGID ) mode = mode | FILE_DATA_IS_GID_BIT;
    if( danstat->st_mode & S_ISVTX ) mode = mode | FILE_DATA_IS_STICKY_BIT;

    /* try to know if it is a link or not */
    if( S_ISLNK(danstat->st_mode) )
    {
      mode = mode | FILE_DATA_IS_LINK;

      link_to_file = g_malloc(link_buffer_size);
      /* read the link name (try read while we have not enought to
        store the name string) */
      while ((nb_car_in_link = readlink(currentfich, link_to_file, 
              link_buffer_size-1)) == link_buffer_size-1)
      { link_buffer_size += 30;
        g_free(link_to_file);
        link_to_file = g_malloc(link_buffer_size);
      }
      
      if(nb_car_in_link == -1) /* error link case are not YET taken in account */
      {gfile_error(GFILE_LIST_LINK_ERROR); exit(GFILE_LIST_LINK_ERROR);}
      /* ended the link name by a 0 */
      link_to_file[nb_car_in_link] = 0;
    }
     
    if( S_ISDIR(danstat->st_mode) ) /* a directory */
    {
      mode = mode | FILE_DATA_IS_DIRECTORY;
    } else
    
    if( S_ISSOCK(danstat->st_mode) ) /* a socket */
    {
      mode = mode | FILE_DATA_IS_SPECIAL;
      mode = mode | FILE_DATA_IS_SOCKET;
    } else

    if( S_ISBLK(danstat->st_mode) ) /* a block device */
    {
      mode = mode | FILE_DATA_IS_SPECIAL;    
      mode = mode | FILE_DATA_IS_BLOCK_DEV;
    } else

    if( S_ISCHR(danstat->st_mode) ) /* a character device */
    {
      mode = mode | FILE_DATA_IS_SPECIAL;    
      mode = mode | FILE_DATA_IS_CHAR_DEV;
    } else

    if( S_ISFIFO(danstat->st_mode) ) /* a fifo */
    {
      mode = mode | FILE_DATA_IS_SPECIAL;    
      mode = mode | FILE_DATA_IS_FIFO;
    } else

    if( S_ISREG(danstat->st_mode) ) /* a regular file */
    {
      /* we check if it is executable for somebody */
      if((danstat->st_mode & 0x40)||(danstat->st_mode & 0x8)||
	 (danstat->st_mode & 0x1))
      {
        mode = mode | FILE_DATA_IS_EXECUTABLE;
	/* check if the gfile user can exec it */
	/* and if we want to check it */
	if(pOwner != NULL)
	{
  	  if((danstat->st_mode & 0x1)||
	     ((danstat->st_mode & 0x8) &&
	      (owner_current_user(pOwner)->grp->nb == danstat->st_gid) )||
	     ((danstat->st_mode & 0x40) &&
	      (owner_current_user(pOwner)->nb ==  danstat->st_uid) ))
            mode = mode | FILE_DATA_IS_ALLOW_TO_EXEC;
	}    
      }		
    } else { mode = mode | FILE_DATA_IS_SPECIAL; /* where are we here ? */}
  } /* end of if stat error */
  g_free(currentfich);

  vFileData = file_data_new(file_path,file_name,
    (guint16)danstat->st_mode,(guint32)danstat->st_uid,
    (guint32)danstat->st_gid,(guint64)danstat->st_size,
    (guint64)danstat->st_atime,(guint64)danstat->st_mtime,
    (guint64)danstat->st_ctime,(guint16)mode,NULL,
    link_to_file);

  /* if memory was allocated for the link file name, free it */
  if(link_to_file != NULL) g_free(link_to_file);

  /* free the stat structure */
  g_free(danstat);

  if(vFileData == NULL) {gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}
  /* attach the corresponding file type */

  /* if we want to attach the corresponding file_type */
  if(pFileType != NULL)
  {
    if(file_data_find_type(pFileType,vFileData) == NULL)
    {gfile_error(FILE_TYPE_ERROR); exit(FILE_TYPE_ERROR);}
  }  
  return(vFileData);
}
/********************************************************************/

/************************************************/
/* Find the filetype corresponding to pFileData */
/* and attach it to pFileData                   */
/* Return : NULL if there is an error.          */
file_type *file_data_find_type(FileType *pFileType, FileData *pFileData)
{ gboolean is_link;

  if(pFileData->mode & FILE_DATA_IS_LINK) is_link=TRUE; else is_link=FALSE;

  /* a broken file */
  if(pFileData->mode & FILE_DATA_NO_EXTENDED_INFO)
  { /* IMPROVEMENT : define a broken TYPE */
    pFileData->type = file_type_find_type(pFileType,SPECIAL,is_link);
    if(pFileData->type == NULL){g_print("SPECIAL not found ?\n");}
    return(pFileData->type);
  }
  /* a special file (socket, buffer...) */
  if(pFileData->mode & FILE_DATA_IS_SPECIAL)
  {
    pFileData->type = file_type_find_type(pFileType,SPECIAL,is_link);
    if(pFileData->type == NULL){g_print("SPECIAL not found ?\n");}
    return(pFileData->type);
  }
  /* a directory */   
  if(pFileData->mode & FILE_DATA_IS_DIRECTORY)
  {
    pFileData->type = file_type_find_type(pFileType,NORMAL_DIRECTORY,is_link);
    if(pFileData->type == NULL){g_print("NORMAL_DIRECTORY not found ?\n");}
    return(pFileData->type);
  }
  /* normal defined extension */
  pFileData->type = file_type_find_type(pFileType,pFileData->name,is_link);
  if(pFileData->type != NULL) /* we found a corresponding type */
    return(pFileData->type);
  /* an executable */   
  if(pFileData->mode & FILE_DATA_IS_EXECUTABLE)
  {
    pFileData->type = file_type_find_type(pFileType,EXECUTABLE,is_link);
    if(pFileData->type == NULL){g_print("EXECUTABLE not found ?\n");}
    else return(pFileData->type);
    /* if EXECUTABLE not define do like if it is a DEFAULT_TYPE */
  }
  /* not define */
  pFileData->type = file_type_find_type(pFileType,DEFAULT_TYPE,is_link);
  if(pFileData->type == NULL){g_print("DEFAULT_TYPE not found ?\n");}
  return(pFileData->type);
}
/************************************************/

/* convert the FileData structure into the stat file mode */
mode_t file_data_mode(FileData *pFileData)
{ mode_t mode = 0;

  if(pFileData->rights & S_IRUSR) mode |= S_IRUSR;
  if(pFileData->rights & S_IWUSR) mode |= S_IWUSR;
  if(pFileData->rights & S_IXUSR) mode |= S_IXUSR;
  if(pFileData->rights & S_IRGRP) mode |= S_IRGRP;
  if(pFileData->rights & S_IWGRP) mode |= S_IWGRP;
  if(pFileData->rights & S_IXGRP) mode |= S_IXGRP;
  if(pFileData->rights & S_IROTH) mode |= S_IROTH;    
  if(pFileData->rights & S_IWOTH) mode |= S_IWOTH;    
  if(pFileData->rights & S_IXOTH) mode |= S_IXOTH;

  if(pFileData->mode & FILE_DATA_IS_UID_BIT) mode |= S_ISUID;
  if(pFileData->mode & FILE_DATA_IS_GID_BIT) mode |= S_ISGID;
  if(pFileData->mode & FILE_DATA_IS_STICKY_BIT) mode |= S_ISVTX;
    
  if(pFileData->mode & FILE_DATA_IS_SOCKET)    mode |= S_IFSOCK; else
  if(pFileData->mode & FILE_DATA_IS_BLOCK_DEV) mode |= S_IFBLK;  else
  if(pFileData->mode & FILE_DATA_IS_CHAR_DEV)  mode |= S_IFCHR;  else
  if(pFileData->mode & FILE_DATA_IS_FIFO)      mode |= S_IFIFO;  else
  if(pFileData->mode & FILE_DATA_IS_LINK)      mode |= S_IFLNK;  else
  if(pFileData->mode & FILE_DATA_IS_DIRECTORY) mode |= S_IFDIR;  else
    mode |= S_IFREG;
    
  return(mode);
}
