/****************************************/
/*  FileData.c headers         	        */
/*					*/
/*  Description : Datas attached t each */
/*		  file. 		*/
/*					*/
/*  Written by : Daniel Lacroix	        */
/*  GFile is under the GPL2 licence     */
/****************************************/

#ifndef __FILE_DATA_H__
#define __FILE_DATA_H__

typedef struct _FileData FileData;

/* theses datas are attached to each row */
/* THIS MUST BE USED FOR DOING STUFF     */
/* theses define are for the 'mode' */
#define FILE_DATA_IS_LINK          1<<0
#define FILE_DATA_IS_DIRECTORY     1<<1
#define FILE_DATA_IS_EXECUTABLE    1<<2
#define FILE_DATA_IS_ALLOW_TO_EXEC 1<<3
/* set if SOCKET or BLOCK_DEV or CHAR_DEV or FIFO is set */
#define FILE_DATA_IS_SPECIAL       1<<4

#define FILE_DATA_IS_SOCKET        1<<5
#define FILE_DATA_IS_BLOCK_DEV     1<<6
#define FILE_DATA_IS_CHAR_DEV      1<<7
#define FILE_DATA_IS_FIFO          1<<8
#define FILE_DATA_IS_UID_BIT       1<<9
#define FILE_DATA_IS_GID_BIT       1<<10
#define FILE_DATA_IS_STICKY_BIT    1<<11
  
#define FILE_DATA_NO_EXTENDED_INFO 1<<14
/* and specialy for Trees. If set, this mean that the content of
   this directory have been listed */
#define FILE_DATA_IS_DIR_SCAN      1<<15
struct _FileData
{
  gchar *path;
  gchar *name;
  guint16 rights;
  guint32 uid;
  guint32 gid;
  guint64 size;
  guint64 last_access_date;
  guint64 last_modif_date;
  guint64 last_status_change_date;
  guint16 mode;  /* see the FILE_DATA_* define above to test do for example :*/
  		 /* mode & FILE_DATA_IS_DIRECTORY to know if it is a direcory*/
  file_type	*type;
  gchar *link_to_file; /* name of the file where the link go. NULL if not a link*/ 
};

/* create a new data 'ready to attach' to a clist row        */
/* keep is own copy of path and name. You can free to origin */
/* return : NULL if impossible.				     */
FileData *file_data_new(gchar *path,gchar *name,guint16 rights,
  guint32 uid,guint32 gid,guint64 size,guint64 last_access_date,guint64 last_modif_date,
  guint64 last_status_change_date,guint16  mode,file_type *type,gchar *link_to_file);

/* free a FileData          */
/* the file type is not free*/
gboolean file_data_free(FileData *pFileData);

/* Create a copy of pFileData  */
/* return : null if impossible */
FileData *file_data_copy(FileData *pFileData);

/* Return TRUE if p1FileData and p2FileData   */
/* indicate the same file. Else FALSE.        */
gboolean file_data_is_equal(FileData *p1FileData,FileData *p2FileData);

/* create a new file data filled with the info of file "file_name"  */
/* in directory "file_path" (file_path is ALWAYS ended by a /).     */
/* The corresponding file_type is attached to the file if pFileType */
/* is not NULL (pFileType is a list of all the define file_type)    */
/* pOwner is the list of all the known user on this computer, if    */
/* NULL then we don't check the FILE_DATA_IS_ALLOW_TO_EXEC          */
/* The FileData allocated can't be NULL, error is raised if needed  */
FileData *file_data_new_from_file(gchar *file_path,gchar *file_name,
  FileType *pFileType,Owner *pOwner);

/* Find the filetype corresponding to pFileData */
/* and attach it to pFileData                   */
/* Return : NULL if there is an error.          */
file_type *file_data_find_type(FileType *pFileType, FileData *pFileData);

/* convert the FileData structure into the stat file mode */
mode_t file_data_mode(FileData *pFileData);

#endif  /* __FILE_DATA_H__ */
