/****************************************/
/*  FileDataList.c body         	*/
/*					*/
/*  Description : List of FileData      */
/*					*/
/*  Written by : Daniel Lacroix	        */
/*  GFile is under the GPL2 licence     */
/****************************************/

#include "GFile.h"

/******************************/
/* Create a new FileDataList. */
/* Return NULL if impossible  */
FileDataList *file_data_list_new()
{ FileDataList *vFileDataList;

  vFileDataList = g_malloc(sizeof(FileDataList));  
  if(vFileDataList == NULL) return NULL;
  vFileDataList->list =  g_list_alloc();
  if(vFileDataList->list == NULL){g_free(vFileDataList); return(NULL);}
  vFileDataList->end = NULL;
  return(vFileDataList); 
}
/******************************/

/***************************************************************/
/* Create a new FileDataList which is a copy of pFileDataList. */
/* Return NULL if impossible                                   */
FileDataList *file_data_list_new_copy(FileDataList *pFileDataList)
{ FileDataList *vFileDataList;

  vFileDataList = file_data_list_new();
  if(vFileDataList == NULL) return NULL;
  file_data_list_init(pFileDataList);
  while(file_data_list_is_end(pFileDataList) == FALSE)
  {
    file_data_list_append(vFileDataList, file_data_list_current(pFileDataList));
    file_data_list_next(pFileDataList);
  }  
  return(vFileDataList); 
}
/***************************************************************/

/*****************************************/
/* Append a FileData to the FileDataList */
/* We keep A COPY of pFileData           */
void file_data_list_append(FileDataList *pFileDataList, FileData *pFileData)
{ FileData *vFileData;

  vFileData = file_data_copy(pFileData);
  if(vFileData == NULL){gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}
  g_list_append(pFileDataList->list,vFileData);
}
/*****************************************/

/********************************************/
/* Free the FileData list. 		    */
/* Return : TRUE = job done. FALSE = error. */
gboolean file_data_list_free(FileDataList *pFileDataList)
{
  file_data_list_init(pFileDataList);
  while(file_data_list_is_end(pFileDataList) == FALSE)
  {
    file_data_free(file_data_list_current(pFileDataList));
    file_data_list_next(pFileDataList);
  }
  g_list_free(pFileDataList->list);
  g_free(pFileDataList);
}
/********************************************/

/*********************************************************/
/************ FOR GOING OVER ALL THE FILE DATAS **********/

/**********************************************************/
/* After a call to this procedure, file_data_list_current */
/* return the first FileData                              */
void file_data_list_init(FileDataList *pFileDataList)
{
  pFileDataList->current = g_list_next(pFileDataList->list);
  pFileDataList->end = NULL;
}
/**********************************************************/

/*********************************/
/* Return the FileData we are on */
FileData *file_data_list_current(FileDataList *pFileDataList)
{
  return((FileData *)pFileDataList->current->data);
}
/*********************************/

/***************************/
/* Go to the next FileData */
void file_data_list_next(FileDataList *pFileDataList)
{
  pFileDataList->current = g_list_next(pFileDataList->current);
}
/***************************/

/************************************/
/* say if there is no more FileData */
gboolean file_data_list_is_end(FileDataList *pFileDataList)
{
  return(pFileDataList->current == pFileDataList->end);
}
/************************************/

/*********************************************************/
