/****************************************/
/*  FileDataList.c headers         	*/
/*					*/
/*  Description : List of FileData      */
/*					*/
/*  Written by : Daniel Lacroix	        */
/*  GFile is under the GPL2 licence     */
/****************************************/

#ifndef __FILE_DATA_LIST_H__
#define __FILE_DATA_LIST_H__

typedef struct _FileDataList  FileDataList;

struct _FileDataList
{
  /* the list of all the file datas */
  GList *list;
  /* the two above are for going over the list */
  /* current FileData */
  GList *current;
  /* after the last group (stop condition) */
  GList *end;  
};

/* Create a new FileDataList. */
/* Return NULL if impossible  */
FileDataList *file_data_list_new(void);

/* Create a new FileDataList which is a copy of pFileDataList. */
/* Return NULL if impossible                                   */
FileDataList *file_data_list_new_copy(FileDataList *pFileDataList);

/* Append a FileData to the FileDataList */
/* We keep A COPY of pFileData           */
void file_data_list_append(FileDataList *pFileDataList, FileData *pFileData);

/* Free the FileData list. 		    */
/* Return : TRUE = job done. FALSE = error. */
gboolean file_data_list_free(FileDataList *pFileDataList);

/*********************************************************/
/************ FOR GOING OVER ALL THE FILE DATAS **********/

/* After a call to this procedure, file_data_list_current */
/* return the first FileData                              */
void file_data_list_init(FileDataList *pFileDataList);

/* Return the FileData we are on */
FileData *file_data_list_current(FileDataList *pFileDataList);

/* Go to the next FileData */
void file_data_list_next(FileDataList *pFileDataList);

/* say if there is no more FileData */
gboolean file_data_list_is_end(FileDataList *pFileDataList);
/*********************************************************/

#endif  /* __FILE_DATA_LIST_H__ */
