/****************************************/
/*  FileProperties.c body               */
/*					*/
/*  Description : Dialog with the       */
/*                properties of a file  */
/*					*/
/*  Written by : Daniel Lacroix	        */
/*  GFile is under the GPL2 licence     */
/****************************************/

#include "GFile.h"

static void file_properties_class_init(FilePropertiesClass *pFilePropertiesClass);
static void file_properties_init(FileProperties *pFileProperties);

/*************************************************************/
guint file_properties_get_type ()
{
  static guint file_properties_type = 0;

  if (!file_properties_type)
  {
    GtkTypeInfo file_properties_info =
    {
       "FileProperties",
       sizeof (FileProperties),
       sizeof (FilePropertiesClass),
       (GtkClassInitFunc)  file_properties_class_init,
       (GtkObjectInitFunc) file_properties_init,
       (GtkArgSetFunc) NULL,
       (GtkArgGetFunc) NULL
     };
     file_properties_type = gtk_type_unique(gnome_dialog_get_type(), &file_properties_info);
  }
  return file_properties_type;
}
/*************************************************************/

/*************************************************************/
enum { FILE_MODE_CHANGED_SIGNAL, LAST_SIGNAL};
static gint file_properties_signals[LAST_SIGNAL] = { 0 };

static void two_file_data_marshal(GtkObject *object, GtkSignalFunc func, gpointer func_data, GtkArg *args) {
    typedef void (*two_file_data_marshal_func)(FileProperties *, FileData *, FileData *,gpointer);
    two_file_data_marshal_func rfunc = (two_file_data_marshal_func) func;
    (*rfunc) (FILE_PROPERTIES(object),
	      GTK_VALUE_POINTER(args[0]),GTK_VALUE_POINTER(args[1]),func_data );
}

static void file_properties_class_init(FilePropertiesClass *class)
{
  /* here we create the new signals if we want */
  GtkObjectClass *object_class;
  object_class = (GtkObjectClass*) class;

  file_properties_signals[FILE_MODE_CHANGED_SIGNAL] = gtk_signal_new
            ("file_mode_changed",
            GTK_RUN_FIRST,object_class->type,
            GTK_SIGNAL_OFFSET (FilePropertiesClass,file_mode_changed),
            two_file_data_marshal, GTK_TYPE_NONE, 2,GTK_TYPE_POINTER,
	    GTK_TYPE_POINTER);
	    
   gtk_object_class_add_signals(object_class, 
     file_properties_signals, LAST_SIGNAL);
   class->file_mode_changed = NULL;
}
/*************************************************************/

/* called when the file_properties dialog is destroy */
static void destroy_cb(GtkWidget *Widget,FileProperties *pFileProperties)
{
  file_data_list_free(pFileProperties->file_list);
  gtk_grab_remove(GTK_WIDGET(pFileProperties));
}

static void ur_cb(GtkWidget *Widget,FileProperties *pFileProperties)
{
  pFileProperties->change_ur = TRUE;
  pFileProperties->ur = !pFileProperties->ur;
}

static void uw_cb(GtkWidget *Widget,FileProperties *pFileProperties)
{
  pFileProperties->change_uw = TRUE;
  pFileProperties->uw = !pFileProperties->uw;
}

static void ux_cb(GtkWidget *Widget,FileProperties *pFileProperties)
{
  pFileProperties->change_ux = TRUE;
  pFileProperties->ux = !pFileProperties->ux;
}

static void gr_cb(GtkWidget *Widget,FileProperties *pFileProperties)
{
  pFileProperties->change_gr = TRUE;
  pFileProperties->gr = !pFileProperties->gr;
}

static void gw_cb(GtkWidget *Widget,FileProperties *pFileProperties)
{
  pFileProperties->change_gw = TRUE;
  pFileProperties->gw = !pFileProperties->gw;
}

static void gx_cb(GtkWidget *Widget,FileProperties *pFileProperties)
{
  pFileProperties->change_gx = TRUE;
  pFileProperties->gx = !pFileProperties->gx;
}

static void or_cb(GtkWidget *Widget,FileProperties *pFileProperties)
{
  pFileProperties->change_or = TRUE;
  pFileProperties->or = !pFileProperties->or;
}

static void ow_cb(GtkWidget *Widget,FileProperties *pFileProperties)
{
  pFileProperties->change_ow = TRUE;
  pFileProperties->ow = !pFileProperties->ow;
}

static void ox_cb(GtkWidget *Widget,FileProperties *pFileProperties)
{
  pFileProperties->change_ox = TRUE;
  pFileProperties->ox = !pFileProperties->ox;
}

static void setuid_cb(GtkWidget *Widget,FileProperties *pFileProperties)
{
  pFileProperties->change_setuid = TRUE;
  pFileProperties->setuid = !pFileProperties->setuid;
}

static void setgid_cb(GtkWidget *Widget,FileProperties *pFileProperties)
{
  pFileProperties->change_setgid = TRUE;
  pFileProperties->setgid = !pFileProperties->setgid;
}

static void setsticky_cb(GtkWidget *Widget,FileProperties *pFileProperties)
{
  pFileProperties->change_setsticky = TRUE;
  pFileProperties->setsticky = !pFileProperties->setsticky;
}


/* called when Cancel is pressed */
static void cancel_cb(GtkWidget *Widget,FileProperties *pFileProperties)
{
  gnome_dialog_close(GNOME_DIALOG(pFileProperties));
}

/* called when Ok is pressed */
static void ok_cb(GtkWidget *Widget,FileProperties *pFileProperties)
{ FileData *vFileData,*vFileData2;
  gchar *string;
  gchar *string2;  
  mode_t mode;

  file_data_list_init(pFileProperties->file_list);
  vFileData = file_data_list_current(pFileProperties->file_list);

  string2 = g_malloc(strlen(vFileData->path)+strlen(gtk_entry_get_text(GTK_ENTRY(pFileProperties->name)))+1);
  if(string2 == NULL){gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}
  strcpy(string2,vFileData->path);
  strcat(string2,gtk_entry_get_text(GTK_ENTRY(pFileProperties->name)));


  /* if the file was a link check if the link have changed */
  if((vFileData->mode & FILE_DATA_IS_LINK) && 
     (strcmp(vFileData->link_to_file,gtk_entry_get_text(GTK_ENTRY(pFileProperties->link_to)))))
  {   
    string = g_malloc(strlen(vFileData->path)+
      strlen(vFileData->name)+1);
    if(string == NULL){gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}
    strcpy(string,vFileData->path);
    strcat(string,vFileData->name);

    /* remove the last link before creating a new one */
    if(remove(string) == -1){g_print("remove ERROR. Case NOT DONE.\n");}
    if(symlink(gtk_entry_get_text(GTK_ENTRY(pFileProperties->link_to)),string2) == -1)
    {g_print("symlink ERROR. Case NOT DONE.\n");}

    g_free(string);
  /* if the name have changed and we are not a link */  
  } else if(strcmp(vFileData->name,gtk_entry_get_text(GTK_ENTRY(pFileProperties->name)))) {
  
    string = g_malloc(strlen(vFileData->path)+strlen(vFileData->name)+1);
    if(string == NULL){gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}
    strcpy(string,vFileData->path);
    strcat(string,vFileData->name);

    if(rename(string, string2) == -1) 
    {g_print("rename ERROR. Case NOT DONE.\n");} 
    
    g_free(string);
  }

  if(pFileProperties->change_ur||pFileProperties->change_uw||
     pFileProperties->change_ux||pFileProperties->change_gr||
     pFileProperties->change_gw||pFileProperties->change_gx||
     pFileProperties->change_or||pFileProperties->change_ow||
     pFileProperties->change_ox||pFileProperties->change_setuid||
     pFileProperties->change_setgid||pFileProperties->change_setsticky)
  {   
    g_print("Rights have changed\n");
    /* create the new rights */
    mode = 0;
    if(pFileProperties->ur) mode |= S_IRUSR;
    if(pFileProperties->uw) mode |= S_IWUSR;
    if(pFileProperties->ux) mode |= S_IXUSR;
    if(pFileProperties->gr) mode |= S_IRGRP;
    if(pFileProperties->gw) mode |= S_IWGRP;
    if(pFileProperties->gx) mode |= S_IXGRP;
    if(pFileProperties->or) mode |= S_IROTH;    
    if(pFileProperties->ow) mode |= S_IWOTH;    
    if(pFileProperties->ox) mode |= S_IXOTH;
    if(pFileProperties->setuid) mode |= S_ISUID;
    if(pFileProperties->setgid) mode |= S_ISGID;
    if(pFileProperties->setsticky) mode |= S_ISVTX;
    
    if(vFileData->mode & FILE_DATA_IS_SOCKET)    mode |= S_IFSOCK; else
    if(vFileData->mode & FILE_DATA_IS_BLOCK_DEV) mode |= S_IFBLK;  else
    if(vFileData->mode & FILE_DATA_IS_CHAR_DEV)  mode |= S_IFCHR;  else
    if(vFileData->mode & FILE_DATA_IS_FIFO)      mode |= S_IFIFO;  else
    if(vFileData->mode & FILE_DATA_IS_LINK)      mode |= S_IFLNK;  else
    if(vFileData->mode & FILE_DATA_IS_DIRECTORY) mode |= S_IFDIR;  else
      mode |= S_IFREG;

    if(chmod(string2, mode) == -1)
    {g_print("chmod ERROR when changing rights. Case NOT DONE.\n");}

  } else  
    g_print("Rights have NOT changed\n");

  vFileData2 = file_data_new_from_file(vFileData->path,gtk_entry_get_text(GTK_ENTRY(pFileProperties->name)),
    pFileProperties->file_type_list,pFileProperties->owner_list);

  g_print("New. Name : %s%s.\n",vFileData2->path,vFileData2->name);
  gtk_signal_emit(GTK_OBJECT(pFileProperties),
    file_properties_signals[FILE_MODE_CHANGED_SIGNAL],vFileData,vFileData2);

  file_data_free(vFileData2);

  g_free(string2);
  gnome_dialog_close(GNOME_DIALOG(pFileProperties));
}

static FileDataList *current_FileDataList;
static FileType     *current_FileType;
static Owner        *current_Owner;

static GtkWidget *left_align_label(gchar *text)
{ GtkWidget *hbox;
  GtkWidget *label;
  
  label = gtk_label_new(text);
  gtk_widget_show(label);
  hbox = gtk_hbox_new(FALSE,0);
  gtk_box_pack_start(GTK_BOX(hbox),label,FALSE,TRUE,0);  
  label = gtk_label_new("");
  gtk_widget_show(label);    
  gtk_box_pack_start(GTK_BOX(hbox),label,TRUE,TRUE,0);  
  
  return(hbox);
}

/*************************************************************/
static void file_properties_init(FileProperties *pFileProperties)
{ FileData *vFileData;
  GtkWidget *label;
  GtkWidget *table;
  gchar *string, *string2;
  GtkWidget *check_but;

  /* take parameters in account */
  pFileProperties->file_list       = current_FileDataList;
  pFileProperties-> file_type_list = current_FileType;
  pFileProperties-> owner_list     = current_Owner;

  file_data_list_init(pFileProperties->file_list);
  /* if there is no file in the list, exit */
  if(file_data_list_is_end(pFileProperties->file_list) == TRUE) return;

  gtk_window_set_title(GTK_WINDOW(pFileProperties),"Properties");

  /* add the OK and CANCEL buttons */
  gnome_dialog_append_buttons(GNOME_DIALOG(pFileProperties),
    GNOME_STOCK_BUTTON_OK,GNOME_STOCK_BUTTON_CANCEL,NULL);

  gtk_signal_connect (GTK_OBJECT (pFileProperties), 
		      "destroy",
		      (GtkSignalFunc) destroy_cb,
		      pFileProperties);

  /* connect ok_cb to the Ok button of the dialog */
  gnome_dialog_button_connect(GNOME_DIALOG(pFileProperties),0,
    GTK_SIGNAL_FUNC(ok_cb),pFileProperties);

  /* connect cancel_cb to the Cancel button of the dialog */
  gnome_dialog_button_connect(GNOME_DIALOG(pFileProperties),1,
    GTK_SIGNAL_FUNC(cancel_cb),pFileProperties);

  vFileData = file_data_list_current(pFileProperties->file_list);

  /* if it is a link, need "link to" field */
  if(vFileData->mode & FILE_DATA_IS_LINK)
    table = gtk_table_new(4,2,FALSE);
  else
    table = gtk_table_new(3,2,FALSE);
  gtk_table_set_row_spacings(GTK_TABLE(table),4);
  gtk_widget_show(table);  

  gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(pFileProperties)->vbox),
    table,TRUE,TRUE,4);  

  label = gtk_label_new("type :");
  gtk_widget_show(label);
  gtk_table_attach_defaults(GTK_TABLE(table),label,0,1,0,1);  

  if(vFileData->mode & FILE_DATA_IS_LINK)
    label = left_align_label("symbolic link"); else
  if(vFileData->mode & FILE_DATA_IS_DIRECTORY)
    label = left_align_label("directory"); else
  if(vFileData->mode & FILE_DATA_IS_SOCKET)
    label = left_align_label("socket"); else  
  if(vFileData->mode & FILE_DATA_IS_BLOCK_DEV)
    label = left_align_label("block device"); else  
  if(vFileData->mode & FILE_DATA_IS_CHAR_DEV)
    label = left_align_label("character device"); else  
  if(vFileData->mode & FILE_DATA_IS_FIFO)
    label = left_align_label("FIFO"); else  
    label = left_align_label("regular file");
  gtk_widget_show(label);  
  gtk_table_attach_defaults(GTK_TABLE(table),label,1,2,0,1);
      
  label = gtk_label_new("path :");
  gtk_widget_show(label);
  gtk_table_attach_defaults(GTK_TABLE(table),label,0,1,1,2);  
  
  label = left_align_label(vFileData->path);
  gtk_widget_show(label);
  gtk_table_attach_defaults(GTK_TABLE(table),label,1,2,1,2);  

  label = gtk_label_new("name :");
  gtk_widget_show(label);
  gtk_table_attach_defaults(GTK_TABLE(table),label,0,1,2,3);
  
  pFileProperties->name = gtk_entry_new();
  gtk_entry_set_text(GTK_ENTRY(pFileProperties->name),vFileData->name);
  gtk_widget_show(pFileProperties->name);
  gtk_table_attach_defaults(GTK_TABLE(table),pFileProperties->name,1,2,2,3);

  if(vFileData->mode & FILE_DATA_IS_LINK)
  {
    label = gtk_label_new("link to :");
    gtk_widget_show(label);
    gtk_table_attach_defaults(GTK_TABLE(table),label,0,1,3,4);

    pFileProperties->link_to = gtk_entry_new();
    gtk_entry_set_text(GTK_ENTRY(pFileProperties->link_to),vFileData->link_to_file);
    gtk_widget_show(pFileProperties->link_to);
    gtk_table_attach_defaults(GTK_TABLE(table),pFileProperties->link_to,1,2,3,4);
  }
  
  /* the file size */
  string2 = g_malloc(22+8);
  sprintf(string2,"size : %u",vFileData->size);
  label = left_align_label(string2);
  gtk_widget_show(label);

  gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(pFileProperties)->vbox),
    label,TRUE,TRUE,4);
  
  /* all dates informations */
  table = gtk_table_new(3,2,FALSE);
  gtk_table_set_row_spacings(GTK_TABLE(table),4);
  gtk_table_set_col_spacings(GTK_TABLE(table),4);
  gtk_widget_show(table);

  gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(pFileProperties)->vbox),
    table,TRUE,TRUE,4);

  label = left_align_label("last access");
  gtk_widget_show(label);
  gtk_table_attach_defaults(GTK_TABLE(table),label,0,1,0,1);
  
  string = ctime((const time_t *)(&vFileData->last_access_date));
  string[strlen(string)-1] = 0;  
  string2 = g_malloc(strlen(string)+3);
  sprintf(string2,": %s",string);
  label = gtk_label_new(string2);
  g_free(string2);
  gtk_widget_show(label);
  gtk_table_attach_defaults(GTK_TABLE(table),label,1,2,0,1);

  label = left_align_label("last modif");
  gtk_widget_show(label);
  gtk_table_attach_defaults(GTK_TABLE(table),label,0,1,1,2);
  
  string = ctime((const time_t *)(&vFileData->last_modif_date));
  string[strlen(string)-1] = 0;  
  string2 = g_malloc(strlen(string)+3);
  sprintf(string2,": %s",string);
  label = gtk_label_new(string2);
  g_free(string2);
  gtk_widget_show(label);
  gtk_table_attach_defaults(GTK_TABLE(table),label,1,2,1,2);

  label = left_align_label("last status change");
  gtk_widget_show(label);
  gtk_table_attach_defaults(GTK_TABLE(table),label,0,1,2,3);
  
  string = ctime((const time_t *)(&vFileData->last_status_change_date));
  string[strlen(string)-1] = 0;  
  string2 = g_malloc(strlen(string)+3);
  sprintf(string2,": %s",string);
  label = gtk_label_new(string2);
  g_free(string2);
  gtk_widget_show(label);
  gtk_table_attach_defaults(GTK_TABLE(table),label,1,2,2,3);

  /* if it is a link, don't display rights (useless) */
  if(!(vFileData->mode & FILE_DATA_IS_LINK))
  {
  /* rights */
  table = gtk_table_new(4,4,FALSE);
  gtk_table_set_row_spacings(GTK_TABLE(table),4);
  gtk_table_set_col_spacings(GTK_TABLE(table),4);
  gtk_widget_show(table);

  gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(pFileProperties)->vbox),
    table,TRUE,TRUE,4);

  /* owner rights */
  label = gtk_label_new("Owner");
  gtk_widget_show(label);
  gtk_table_attach_defaults(GTK_TABLE(table),label,0,1,0,1);

  check_but = gtk_check_button_new_with_label("read");
  if(vFileData->rights & 0x100) 
  {gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(check_but), TRUE); pFileProperties->ur = TRUE;}
  else pFileProperties->ur = FALSE;
  gtk_widget_show(check_but);
  gtk_table_attach_defaults(GTK_TABLE(table),check_but,0,1,1,2);
  gtk_signal_connect(GTK_OBJECT(check_but),"toggled",GTK_SIGNAL_FUNC(ur_cb),pFileProperties);
					   
  check_but = gtk_check_button_new_with_label("write");
  if(vFileData->rights & 0x80)
  {gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(check_but), TRUE); pFileProperties->uw = TRUE;}
  else pFileProperties->uw = FALSE;
  gtk_widget_show(check_but);
  gtk_table_attach_defaults(GTK_TABLE(table),check_but,0,1,2,3);
  gtk_signal_connect(GTK_OBJECT(check_but),"toggled",GTK_SIGNAL_FUNC(uw_cb),pFileProperties);

  check_but = gtk_check_button_new_with_label("exec");
  if(vFileData->rights & 0x40)
  {gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(check_but), TRUE); pFileProperties->ux = TRUE;}
  else pFileProperties->ux = FALSE;
  gtk_widget_show(check_but);
  gtk_table_attach_defaults(GTK_TABLE(table),check_but,0,1,3,4);
  gtk_signal_connect(GTK_OBJECT(check_but),"toggled",GTK_SIGNAL_FUNC(ux_cb),pFileProperties);

  /* group rights */
  label = gtk_label_new("Group");
  gtk_widget_show(label);
  gtk_table_attach_defaults(GTK_TABLE(table),label,1,2,0,1);

  check_but = gtk_check_button_new_with_label("read");
  if(vFileData->rights & 0x20)
  {gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(check_but), TRUE); pFileProperties->gr = TRUE;}
  else pFileProperties->gr = FALSE;
  gtk_widget_show(check_but);
  gtk_table_attach_defaults(GTK_TABLE(table),check_but,1,2,1,2);
  gtk_signal_connect(GTK_OBJECT(check_but),"toggled",GTK_SIGNAL_FUNC(gr_cb),pFileProperties);

  check_but = gtk_check_button_new_with_label("write");
  if(vFileData->rights & 0x10)
  {gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(check_but), TRUE); pFileProperties->gw = TRUE;}
  else pFileProperties->gw = FALSE;
  gtk_widget_show(check_but);
  gtk_table_attach_defaults(GTK_TABLE(table),check_but,1,2,2,3);
  gtk_signal_connect(GTK_OBJECT(check_but),"toggled",GTK_SIGNAL_FUNC(gw_cb),pFileProperties);

  check_but = gtk_check_button_new_with_label("exec");
  if(vFileData->rights & 0x8)
  {gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(check_but), TRUE); pFileProperties->gx = TRUE;}
  else pFileProperties->gx = FALSE;
  gtk_widget_show(check_but);
  gtk_table_attach_defaults(GTK_TABLE(table),check_but,1,2,3,4);
  gtk_signal_connect(GTK_OBJECT(check_but),"toggled",GTK_SIGNAL_FUNC(gx_cb),pFileProperties);

  /* others rights */
  label = gtk_label_new("Others");
  gtk_widget_show(label);
  gtk_table_attach_defaults(GTK_TABLE(table),label,2,3,0,1);

  check_but = gtk_check_button_new_with_label("read");
  if(vFileData->rights & 0x4)
  {gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(check_but), TRUE); pFileProperties->or = TRUE;}
  else pFileProperties->or = FALSE;
  gtk_widget_show(check_but);
  gtk_table_attach_defaults(GTK_TABLE(table),check_but,2,3,1,2);
  gtk_signal_connect(GTK_OBJECT(check_but),"toggled",GTK_SIGNAL_FUNC(or_cb),pFileProperties);

  check_but = gtk_check_button_new_with_label("write");
  if(vFileData->rights & 0x2)
  {gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(check_but), TRUE); pFileProperties->ow = TRUE;}
  else pFileProperties->ow = FALSE;
  gtk_widget_show(check_but);
  gtk_table_attach_defaults(GTK_TABLE(table),check_but,2,3,2,3);
  gtk_signal_connect(GTK_OBJECT(check_but),"toggled",GTK_SIGNAL_FUNC(ow_cb),pFileProperties);

  check_but = gtk_check_button_new_with_label("exec");
  if(vFileData->rights & 0x1)
  {gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(check_but), TRUE); pFileProperties->ox = TRUE;}
  else pFileProperties->ox = FALSE;
  gtk_widget_show(check_but);
  gtk_table_attach_defaults(GTK_TABLE(table),check_but,2,3,3,4);
  gtk_signal_connect(GTK_OBJECT(check_but),"toggled",GTK_SIGNAL_FUNC(ox_cb),pFileProperties);

  /* special rights */
  label = gtk_label_new("Special");
  gtk_widget_show(label);
  gtk_table_attach_defaults(GTK_TABLE(table),label,3,4,0,1);

  check_but = gtk_check_button_new_with_label("set UID");
  if(vFileData->mode & FILE_DATA_IS_UID_BIT)
  {gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(check_but), TRUE); pFileProperties->setuid = TRUE;}
  else pFileProperties->setuid = FALSE;
  gtk_widget_show(check_but);
  gtk_table_attach_defaults(GTK_TABLE(table),check_but,3,4,1,2);
  gtk_signal_connect(GTK_OBJECT(check_but),"toggled",GTK_SIGNAL_FUNC(setuid_cb),pFileProperties);

  check_but = gtk_check_button_new_with_label("set GID");
  if(vFileData->mode & FILE_DATA_IS_GID_BIT)
  {gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(check_but), TRUE); pFileProperties->setgid = TRUE;}
  else pFileProperties->setgid = FALSE;
  gtk_widget_show(check_but);
  gtk_table_attach_defaults(GTK_TABLE(table),check_but,3,4,2,3);
  gtk_signal_connect(GTK_OBJECT(check_but),"toggled",GTK_SIGNAL_FUNC(setgid_cb),pFileProperties);

  check_but = gtk_check_button_new_with_label("set Sticky");
  if(vFileData->mode & FILE_DATA_IS_STICKY_BIT)
  {gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(check_but), TRUE); pFileProperties->setsticky = TRUE;}
  else pFileProperties->setsticky = FALSE;
  gtk_widget_show(check_but);
  gtk_table_attach_defaults(GTK_TABLE(table),check_but,3,4,3,4);
  gtk_signal_connect(GTK_OBJECT(check_but),"toggled",GTK_SIGNAL_FUNC(setsticky_cb),pFileProperties);
    
  pFileProperties->change_ur = FALSE;
  pFileProperties->change_uw = FALSE;
  pFileProperties->change_ux = FALSE;
  pFileProperties->change_gr = FALSE;
  pFileProperties->change_gw = FALSE;
  pFileProperties->change_gx = FALSE;
  pFileProperties->change_or = FALSE;
  pFileProperties->change_ow = FALSE;
  pFileProperties->change_ox = FALSE;
  pFileProperties->change_setuid = FALSE;
  pFileProperties->change_setgid = FALSE;
  pFileProperties->change_setsticky = FALSE;
  } /* end of test if it is a SYMB LINK or not */
  
  /* we want that user can do nothing else when this dialog appear */
  gtk_grab_add(GTK_WIDGET(pFileProperties));
}
/*************************************************************/

/***************************************/
/* Create a new file properties dialog */
GtkWidget *file_properties_new(FileDataList *pFileDataList,FileType *pFileType,Owner *pOwner)
{
  current_FileDataList = file_data_list_new_copy(pFileDataList);
  current_FileType     = pFileType;
  current_Owner        = pOwner;

  return( GTK_WIDGET(gtk_type_new( file_properties_get_type() )) );
}
/***************************************/
