/****************************************/
/*  FileType.c body                     */
/*					*/
/*  Description : FileType take care of */
/*		  file type reconization*/
/*					*/
/*  Written by : Daniel Lacroix	        */
/*  GFile is under the GPL2 licence     */
/****************************************/

#include "GFile.h"

/* usefull for gdk_pixmap_create_from_xpm_d in load_special_type */
static GdkWindow *parent_window_app = NULL;

/***************************************/
/* insert after before in the list     */
/* if before = NULL, insert at the end */
void file_type_insert(FileType *pFileType,file_type *before,file_type *pfile_type)
{ SList *last_list = NULL;
  SList *new_list;

  new_list = g_malloc(sizeof(SList));
  if(new_list == NULL){gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}
  new_list->data = (gpointer)pfile_type;

  file_type_init(pFileType);
  while((file_type_is_end(pFileType) == FALSE) && 
        (file_type_current(pFileType) != before))
  { last_list = pFileType->current; file_type_next(pFileType); }

  if((before != NULL) && (file_type_current(pFileType) == before))
  { last_list = pFileType->current; file_type_next(pFileType); }

  new_list->next = pFileType->current;
  /* if the list is empty */
  if(last_list == NULL)
    pFileType->list = new_list;
  else
    last_list->next = new_list;
}
/***************************************/

/******************************/
/* add at the end of the list */
void file_type_append(FileType *pFileType,file_type *pfile_type)
{
  file_type_insert(pFileType,NULL,pfile_type);
}
/******************************/

/**********************************/
/* Create a new file type system. */
/* Return : NULL if impossible	  */
FileType *file_type_new(void)
{ 
  FileType *vFileType;
  
  vFileType = g_malloc(sizeof(FileType));
  if(vFileType == NULL){gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}
  vFileType->current = NULL;
  vFileType->end     = NULL;
  vFileType->list    = NULL;
  return (vFileType);
}  
/**********************************/

/****************************/
/* Free a file type system. */
void file_type_free(FileType *pFileType)
{
  /* free each file type (always destruct the first of the list)*/
  file_type_init(pFileType);
  while(file_type_is_end(pFileType) == FALSE)
  {
    file_type_remove_type(pFileType, file_type_current(pFileType) );
    file_type_init(pFileType);
  }
  g_free(pFileType);
}
/****************************/

/************************************************************************/
/* Try to find at what kind of file type "name" belong                  */
/* is_link is here because we consedere that a link is a different type */
/* Return :  NULL if not found or a pointer to the file_type is belong  */
file_type *file_type_find_type(FileType *pFileType,gchar *name,gboolean is_link)
{ file_type *tmptype;

  file_type_init(pFileType);

  while(!file_type_is_end(pFileType))
  {
    tmptype = file_type_current(pFileType);
     
    if( tmptype->is_link == is_link )
    {
      switch(tmptype->pos)
      {
        case 0 :   /* exactly */
	  	 if(!strcmp(tmptype->string,name))
		   return(tmptype);	
		 break;
	case 1 :  /* begin with */
	  	 if(!strncmp(tmptype->string,name,strlen(tmptype->string)))
		   return(tmptype);
	  	 break;
	case 2 :   /* end with */
	  	 if( !strncmp(tmptype->string,
		   &name[strlen(name)-strlen(tmptype->string)],
		   strlen(tmptype->string)) )
		   return(tmptype);
	  	 break;
      }
    } 
    file_type_next(pFileType);
  }  
  return(NULL);
}
/************************************************************************/

/****************************************************************************/
/* Load the file types from the file. Each special type exist after this    */
/* procedure even if they are not define in filetype or if filetype doesn't */
/* exist								    */
/* return : 1 = impossible  0 = job done and file_type_list is fill         */

/* some default pixmap if not defined */
static char * DEFAULT_TYPE_XPM[] = {
"16 16 4 1",
"       c None s None",
".      c black",
"X      c white",
"o      c #808080",
"                ",
"   .......      ",
"   .XXXXX..     ",
"   .XoooX.X.    ",
"   .XXXXX....   ",
"   .XooooXoo.o  ",
"   .XXXXXXXX.o  ",
"   .XooooooX.o  ",
"   .XXXXXXXX.o  ",
"   .XooooooX.o  ",
"   .XXXXXXXX.o  ",
"   .XooooooX.o  ",
"   .XXXXXXXX.o  ",
"   ..........o  ",
"    oooooooooo  ",
"                "};

static char *NORMAL_DIRECTORY_XPM[] = {
"16 16 6 1",
" 	c None s None",
".	c #808080",
"X	c #c0c0c0",
"o	c #ffff00",
"O	c black",
"#	c white",
"                ",
"  .....         ",
" .ooooo.        ",
".ooooooo......  ",
".############.O ",
".#ooooooooooo.O ",
".#ooooooooooo.O ",
".#ooooooooooo.O ",
".#ooooooooooo.O ",
".#ooooooooooo.O ",
".#ooooooooooo.O ",
".#ooooooooooo.O ",
"..............O ",
" OOOOOOOOOOOOOO ",
"                ",
"                "};

static char *CURRENT_DIRECTORY_XPM[] = {
"16 16 6 1",
" 	c None s None",
".	c #808080",
"X	c #c0c0c0",
"o	c #0000ff",
"O	c black",
"#	c white",
"                ",
"  .....         ",
" .ooooo.        ",
".ooooooo......  ",
".############.O ",
".#ooooooooooo.O ",
".#ooooooooooo.O ",
".#ooooooooooo.O ",
".#ooooooooooo.O ",
".#ooooooooooo.O ",
".#ooooooooooo.O ",
".#ooooooooooo.O ",
"..............O ",
" OOOOOOOOOOOOOO ",
"                ",
"                "};

static char *CURRENT_OPEN_DIRECTORY_XPM[] = {
/* width height num_colors chars_per_pixel */
"    16    16       6            1",
/* colors */
"  c None s None",
". c black",
"# c #808080",
"g c white",
"h c #c0c0c0",
"j c blue",
/* pixels */
"                ",
"   #####        ",
"  #ggggg#       ",
" #ghjhjhg###### ",
" #gjjjjjjggggg#.",
" #gjjjjjjjjjjj#.",
"#############j#.",
"#gggggggggg#.j#.",
"#gjjjjjjjjjj.##.",
" #gjjjjjjjjj#.#.",
" #gjjjjjjjjjj.#.",
"  #gjjjjjjjjj#..",
"  ############..",
"   .............",
"                ",
"                ",
};

static char *PAST_DIRECTORY_XPM[] = {
"16 16 6 1",
" 	c None s None",
".	c #808080",
"X	c #c0c0c0",
"o	c #ff0000",
"O	c black",
"#	c white",
"                ",
"  .....         ",
" .ooooo.        ",
".ooooooo......  ",
".############.O ",
".#ooooooooooo.O ",
".#ooooooooooo.O ",
".#ooooooooooo.O ",
".#ooooooooooo.O ",
".#ooooooooooo.O ",
".#ooooooooooo.O ",
".#ooooooooooo.O ",
"..............O ",
" OOOOOOOOOOOOOO ",
"                ",
"                "};


static char *NORMAL_OPEN_DIRECTORY_XPM[] = {
/* width height num_colors chars_per_pixel */
"    16    16       6            1",
/* colors */
"  c None s None",
". c black",
"# c #808080",
"g c white",
"h c #c0c0c0",
"j c yellow",
/* pixels */
"                ",
"   #####        ",
"  #ggggg#       ",
" #ghjhjhg###### ",
" #gjjjjjjggggg#.",
" #gjjjjjjjjjjj#.",
"#############j#.",
"#gggggggggg#.j#.",
"#gjjjjjjjjjj.##.",
" #gjjjjjjjjj#.#.",
" #gjjjjjjjjjj.#.",
"  #gjjjjjjjjj#..",
"  ############..",
"   .............",
"                ",
"                ",
};

static char *PAST_OPEN_DIRECTORY_XPM[] = {
/* width height num_colors chars_per_pixel */
"    16    16       6            1",
/* colors */
"  c None s None",
". c black",
"# c #808080",
"g c white",
"h c #c0c0c0",
"j c red",
/* pixels */
"                ",
"   #####        ",
"  #ggggg#       ",
" #ghjhjhg###### ",
" #gjjjjjjggggg#.",
" #gjjjjjjjjjjj#.",
"#############j#.",
"#gggggggggg#.j#.",
"#gjjjjjjjjjj.##.",
" #gjjjjjjjjj#.#.",
" #gjjjjjjjjjj.#.",
"  #gjjjjjjjjj#..",
"  ############..",
"   .............",
"                ",
"                ",
};

static char * SPECIAL_XPM[] = {
"16 16 5 1",
" 	s None	c None",
".	c red",
"X	c white",
"o	c black",
"O	c #808080",
"       ...      ",
"     ......     ",
"    .X......    ",
"   .X..oo...o   ",
"   ...o  ...o   ",
"    oo   ...o   ",
"        ...o    ",
"       ...o     ",
"      ..oo      ",
"      ....o     ",
"      O..O      ",
"       Ooo      ",
"      OX..      ",
"      ....O     ",
"       ..oO     ",
"        oO      "};

static char * EXECUTABLE_XPM[] = {
"16 16 5 1",
" 	s None	c None",
".	c red",
"X	c white",
"o	c black",
"O	c #808080",
"       ...      ",
"     ......     ",
"    .X......    ",
"   .X..oo...o   ",
"   ...o  ...o   ",
"   ...o  ...o   ",
"   ...o  ...o   ",
"   ...o  ...o   ",
"   ...o  ...o   ",
"   ...o  ...o   ",
"   ...O  ...o   ",
"   ...O  ...o   ",
"   ...O ....o   ",
"    .......o    ",
"     ....oO     ",
"      ..oO      "};

/* load the special type pTypeName if not define by user (link and normal) */
/* DO NOT DIRECTLY USE							   */
/* Return : TRUE = ok; FALSE = impossible                                  */
gboolean load_special_type(FileType *pFileType,gchar *pTypeName,char *pXpmPixmap[], gboolean pWantLink)
{ file_type *tempptr, *tempptr2;
  
  /* we check if this special types is define. Else we put the default one.*/
  if(file_type_find_type(pFileType,pTypeName,FALSE) == NULL) /* not the link */
  {
    tempptr = (file_type *)g_malloc(sizeof(file_type));
    if(!tempptr){gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}
    strcpy(tempptr->string,pTypeName);
    tempptr->pos = 0;
    tempptr->is_link = FALSE;
    tempptr->program[0] = 0;
    tempptr->icon[0] = 0;
    /* "load" the internal pixmap */
    tempptr->pixmap = gdk_pixmap_create_from_xpm_d (
			 parent_window_app,
			 &tempptr->mask,
			 NULL,
			 pXpmPixmap);
    if(!tempptr->pixmap) {gfile_error(INTERNAL_PIXMAP); exit(INTERNAL_PIXMAP);}
    /* special type not define are ALWAYS added at the end of the list */
    file_type_append(pFileType,tempptr);
  }
  if(pWantLink == TRUE) /* if we also want a special type link define */
  {
   /* link = same icon as not link will be used */
   if(file_type_find_type(pFileType,pTypeName,TRUE) == NULL)
   { /* not link is define else we can't be here */
     tempptr = (file_type *)g_malloc(sizeof(file_type));
     if(tempptr == NULL){gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}
     strcpy(tempptr->string,pTypeName);
     tempptr->pos = 0;
     tempptr->is_link = TRUE;
     tempptr->program[0]=0;
     tempptr->icon[0]=0;
     tempptr2 = file_type_find_type(pFileType,pTypeName,FALSE);
     if(tempptr2 == NULL)
     { gfile_error(INTERNAL_PIXMAP_COPY); exit(INTERNAL_PIXMAP_COPY); }
     /* declare that the pixmap is use one more time */		    
     tempptr->pixmap = gdk_pixmap_ref(tempptr2->pixmap);
     /* declare that the bitmap is use one more time */		    
     tempptr->mask = gdk_bitmap_ref(tempptr2->mask);
     file_type_append(pFileType,tempptr);
   }
  } 
  return(TRUE);
}

/********************************************************/
/* Load the file types from the file. Each special      */
/* type exist after this procedure even if they are     */
/* not define in filetype or if filetype doesn't exist. */
/* filename is the full path name of the file           */
/* Return : FALSE = impossible                          */
/*          TRUE  = job done and the file type list fill*/
gboolean file_type_load_from_file(FileType *pFileType,gchar *filename)
{
  gint file_handle;
  file_type temptype;

  /* if filename is null, then we just want to use internal type */
  if(filename == NULL) 
    file_handle = -1;
  else 
    file_handle = open(filename,O_RDONLY);
  
  if(file_handle != -1) /* If the file exist, we use it */
  {
    /* add the file type define by the user in filetype file */  
    while( read(file_handle,&temptype,sizeof(file_type)) > 0 )
      file_type_append_type(pFileType,temptype.string,temptype.pos,temptype.is_link,temptype.program,temptype.icon);
    close(file_handle);
  }

  /* check and create if needed the special type */
  if( !load_special_type(pFileType,NORMAL_DIRECTORY,NORMAL_DIRECTORY_XPM,TRUE) ||
      !load_special_type(pFileType,NORMAL_OPEN_DIRECTORY,NORMAL_OPEN_DIRECTORY_XPM,FALSE) ||
      !load_special_type(pFileType,CURRENT_DIRECTORY,CURRENT_DIRECTORY_XPM,FALSE) ||
      !load_special_type(pFileType,CURRENT_OPEN_DIRECTORY,CURRENT_OPEN_DIRECTORY_XPM,FALSE) ||      
      !load_special_type(pFileType,PAST_DIRECTORY,PAST_DIRECTORY_XPM,FALSE) ||
      !load_special_type(pFileType,PAST_OPEN_DIRECTORY,PAST_OPEN_DIRECTORY_XPM,FALSE) ||
      !load_special_type(pFileType,SPECIAL,SPECIAL_XPM,TRUE) ||      
      !load_special_type(pFileType,DEFAULT_TYPE,DEFAULT_TYPE_XPM,TRUE) ||
      !load_special_type(pFileType,EXECUTABLE,EXECUTABLE_XPM,TRUE) ) return(FALSE);   
      
  return(TRUE);
}
/****************************************************************************/

/************************************************/
/* Save the file type list in the filetype file */
/* filename is the full path name of the file   */
/* Return : FALSE = impossible; TRUE = job done */
gboolean file_type_save_to_file(FileType *pFileType,gchar *filename)
{
  gint file_handle;
  file_type *tmptype;

  file_handle = creat(filename,S_IREAD|S_IWRITE);
  if(file_handle == -1){gfile_error(CREATE_FILE); exit(CREATE_FILE);}
  
  file_type_init(pFileType);
  
  while(file_type_is_end(pFileType) == FALSE)
  {
    tmptype = file_type_current(pFileType);
    /* if this is not an internal file_type we save it to the disk */
    if( (tmptype->icon)[0] )
      write(file_handle,tmptype,sizeof(file_type));
    /* next file type */
    file_type_next(pFileType);  
  }      
  
  close(file_handle);
  return(TRUE);
}
/************************************************/

/*****************************************************************/
/* Insert a new file type to the file type list after before.    */
/* If before is NULL, add the type at the end of the list.       */
/* icon is the full path name of the icon file.                  */
/* Return : NULL = not possible else the new file_type structure */
file_type *file_type_insert_type(FileType *pFileType,file_type *before,
  gchar *string,guint8 pos, gboolean is_link,gchar *program,gchar *icon)
{ file_type *tempptr;
  file_type *tmptype;
  GdkImlibImage *pix;

  tempptr = (file_type *)g_malloc(sizeof(file_type));
  if(tempptr == NULL){gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}

  strcpy(tempptr->string,string);
  tempptr->pos = pos;
  tempptr->is_link = is_link;
  strcpy(tempptr->program,program);
  strcpy(tempptr->icon,icon);

  file_type_init(pFileType);
  while(file_type_is_end(pFileType) == FALSE)
  {
    tmptype = file_type_current(pFileType);
    
    if( !strcmp(tmptype->icon,tempptr->icon) )
    {
      tempptr->pixmap = gdk_pixmap_ref(tmptype->pixmap);
      tempptr->mask   = gdk_bitmap_ref(tmptype->mask);
      /* the new type is added in the file_type_list */
      file_type_insert(pFileType,before,tempptr);
      return(tempptr); /* image already loaded for another file type */
    }     
    file_type_next(pFileType);
  }
  /* The pixmap is not in memory, we must loaded it from the file */

  pix = gdk_imlib_load_image(icon);
  if(pix == NULL) return(NULL);
  gdk_imlib_render (pix, pix->rgb_width, pix->rgb_height); 
  tempptr->pixmap = gdk_imlib_copy_image(pix);
  if(tempptr->pixmap == NULL) return(NULL);
  tempptr->mask   = gdk_imlib_copy_mask(pix);
  if(tempptr->mask == NULL) return(NULL);
  gdk_imlib_destroy_image(pix);
  /* added in the file_type_list */
  file_type_insert(pFileType,before,tempptr);
  return(tempptr);
}
/*****************************************************************/

/*****************************************************************/
/* Add a new file type to the file type list.                    */
/* icon is the full path name of the icon file.                  */
/* Return : NULL = not possible else the new file_type structure */
file_type *file_type_append_type(FileType *pFileType,gchar *string,guint8 pos,
                         gboolean is_link,gchar *program,gchar *icon)
{
  return(file_type_insert_type(pFileType,NULL,string,pos,is_link,program,icon));
}			 
/*****************************************************************/

/************************************************/
/* Remove a file type to the file type list     */
/* Return : FALSE = impossible; TRUE = job done */
gboolean file_type_remove_type(FileType *pFileType,file_type *remove_type)
{   
  SList *last_type = NULL;
  /* free the element in the list*/
  file_type_init(pFileType);
  while((file_type_is_end(pFileType) == FALSE) &&
        (file_type_current(pFileType) != remove_type))
  {
    last_type = pFileType->current;
    file_type_next(pFileType);
  }  
  if(file_type_is_end(pFileType) == TRUE)
    return(FALSE);
  else {  
    /* if this is the first type we want to remove */
    if(last_type == NULL)
      pFileType->list = pFileType->current->next;
    else  
      last_type->next = pFileType->current->next;
    g_free(pFileType->current);
  }    

  /* free the pixmap attached to the type */
  gdk_bitmap_unref(remove_type->pixmap);
  /* free the mask attached to the type */
  gdk_bitmap_unref(remove_type->mask);
  /* free the type structure */
  g_free(remove_type);    

  return(TRUE);
}    
/************************************************/

/*****************************************************/
/* After a call to this procedure, file_type_current */
/* return the first file type                        */
void file_type_init(FileType *pFileType)
{
  /* current = first type */
  pFileType->current = pFileType->list;
  /* end = after the last type */
  pFileType->end = NULL;
}
/*****************************************************/

/*****************************/
/* Return the type we are on */
file_type *file_type_current(FileType *pFileType)
{
  return((file_type *)(pFileType->current->data));
}
/*****************************/

/****************************/
/* Go to the next file_type */
void file_type_next(FileType *pFileType)
{ pFileType->current = pFileType->current->next; }
/****************************/

/********************************/
/* say if there is no more type */
gboolean file_type_is_end(FileType *pFileType)
{ return(pFileType->current == pFileType->end); }
/********************************/

/**************************************************************/
/* determine if the file type test_type is a special one      */
/* Return : TRUE = it is a special type which can't be remove */
/*	    FALSE = not a special type			      */
gboolean file_type_is_a_special_type(file_type *test_type)
{
  if(      !(strcmp(test_type->string,NORMAL_DIRECTORY))	||
	   !(strcmp(test_type->string,NORMAL_OPEN_DIRECTORY))	||
	   !(strcmp(test_type->string,CURRENT_DIRECTORY))	||
	   !(strcmp(test_type->string,CURRENT_OPEN_DIRECTORY))	||
	   !(strcmp(test_type->string,PAST_DIRECTORY))		||
	   !(strcmp(test_type->string,PAST_OPEN_DIRECTORY))	||
	   !(strcmp(test_type->string,EXECUTABLE))		||
	   !(strcmp(test_type->string,SPECIAL))			||
	   !(strcmp(test_type->string,DEFAULT_TYPE)) ) return(TRUE); else return(FALSE);
}
/**************************************************************/

/*************************************/
/* swap two item in a file type list */
void file_type_swap_type(FileType *pFileType,file_type *type1,file_type *type2)
{ SList *data1_list,*data2_list;
  /* find type1 in the list */  
  file_type_init(pFileType);
  while((file_type_is_end(pFileType) == FALSE) && 
        (file_type_current(pFileType) != type1))
  { file_type_next(pFileType); }
  /* type not found */
  if(file_type_is_end(pFileType) == TRUE) return;
  data1_list = pFileType->current; 
  /* find type2 in the list */    
  file_type_init(pFileType);
  while((file_type_is_end(pFileType) == FALSE) && 
        (file_type_current(pFileType) != type2))
  { file_type_next(pFileType); }
  /* type not found */
  if(file_type_is_end(pFileType) == TRUE) return;
  data2_list = pFileType->current;
  /* data both exist => swap */
  data2_list->data = (gpointer)type1; data1_list->data = (gpointer)type2;
}
/*************************************/

/************************************************************/
/* swap two FileType.                                       */
/* You CAN'T CALLED THIS IF YOU ARE USING file_type_next... */
void file_type_swap_file_type(FileType *pFileType1,FileType *pFileType2)
{ SList *tmp_list;
 
  tmp_list = pFileType1->list;
  pFileType1->list = pFileType2->list;
  pFileType2->list = tmp_list;
}
/************************************************************/

/*******************************************************/
/* add all the file types of psrc at the end of pdest. */
/* If pdest is empty, this make a copy of psrc.        */
void file_type_append_file_type(FileType *pdest,FileType *psrc)
{ SList *dest_last = NULL;
  file_type *new_type,*src_type;

  file_type_init(psrc);
  while(file_type_is_end(psrc) == FALSE)
  {
    src_type = file_type_current(psrc);
    /* make a copy of src_type in new_type */
    new_type = (file_type *)g_malloc(sizeof(file_type));
    if(new_type == NULL){gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}    
    strcpy(new_type->string,src_type->string);
    new_type->pos = src_type->pos;
    new_type->is_link = src_type->is_link;
    strcpy(new_type->program,src_type->program);
    strcpy(new_type->icon,src_type->icon);    
    new_type->pixmap = gdk_pixmap_ref(src_type->pixmap);
    new_type->mask   = gdk_bitmap_ref(src_type->mask);
    /* add new type to the destination file type system */
    file_type_append(pdest,new_type);
    file_type_next(psrc);
  }
}
/************************************************************/

/**********************************************************/
/* set the parent application to avoid warning when using */
/* gdk_pixmap_create_from_xpm_d                           */
void file_type_set_parent(GtkWidget *parent)
{ parent_window_app = parent->window; }
/**********************************************************/
