/****************************************/
/*  FileType.c headers                  */
/*					*/
/*  Description : FileType take care of */
/*		  file type reconization*/
/*					*/
/*  Written by : Daniel Lacroix	        */
/*  GFile is under the GPL2 licence     */
/****************************************/

#ifndef __FILE_TYPE_H__
#define __FILE_TYPE_H__

typedef struct _FileType FileType;
typedef struct _SList SList;

struct _SList
{
  SList    *next;
  gpointer *data;
};

struct _FileType
{
  /* the list of all the file type */
  SList *list;
  /* the two above are for going over the list */
  /* current type */
  SList *current;
  /* after the last type (stop condition) */
  SList *end;
};

/* Special file types in the file_type_list GList */
#define NORMAL_DIRECTORY       "n_dir"
#define NORMAL_OPEN_DIRECTORY  "no_dir"
#define CURRENT_DIRECTORY      "c_dir"
#define CURRENT_OPEN_DIRECTORY "co_dir"
#define PAST_DIRECTORY         "p_dir"
#define PAST_OPEN_DIRECTORY    "po_dir"
#define EXECUTABLE             "exec"
#define SPECIAL		       "special"
#define DEFAULT_TYPE	       "default"

/* data attach to file_type_list GList */
#define FILE_TYPE_STRING_MAX  10
#define FILE_TYPE_PROGRAM_MAX 100
#define FILE_TYPE_ICON_MAX    100
typedef struct file_type
{
  gchar string[FILE_TYPE_STRING_MAX]; 
                                /* string to identifie the file        */
  guint8 pos;			/* 0 = exactly 1 = start 2 = end       */
  gboolean is_link;		/* 0 = NO      1 = YES                 */
  gchar program[FILE_TYPE_PROGRAM_MAX];
                                /* program to use to open the file     */
  gchar icon[FILE_TYPE_ICON_MAX];  
                                /* icon filename to use with this type */
  				/* special types not define have no icon*/
				/* name (icon[0]==0)		       */
  GdkPixmap *pixmap;            /* pixmap made with icon               */
  GdkBitmap *mask;              /* mask made with icon                 */
} file_type;

/* Create a new file type system. */
/* Return : NULL if impossible	  */
FileType *file_type_new(void);

/* Free a file type system. */
void file_type_free(FileType *pFileType);

/* Load the file types from the file. Each special      */
/* type exist after this procedure even if they are     */
/* not define in filetype or if filetype doesn't exist. */
/* filename is the full path name of the file           */
/* Return : FALSE = impossible                          */
/*          TRUE  = job done and the file type list fill*/
gboolean file_type_load_from_file(FileType *pFileType,gchar *filename);

/* Try to find at what kind of file type "name" belong                  */
/* is_link is here because we consedere that a link is a different type */
/* Return :  NULL if not found or a pointer to the file_type is belong  */
file_type *file_type_find_type(FileType *pFileType,gchar *name,gboolean is_link);

/* Save the file type list in the filetype file */
/* filename is the full path name of the file   */
/* Return : FALSE = impossible; TRUE = job done */
gboolean file_type_save_to_file(FileType *pFileType,gchar *filename);

/* Add a new file type to the file type list.                    */
/* icon is the full path name of the icon file.                  */
/* Return : NULL = not possible else the new file_type structure */
file_type *file_type_append_type(FileType *pFileType,gchar *string,guint8 pos,
  gboolean is_link,gchar *program,gchar *icon);
  
/* Insert a new file type to the file type list after before.    */
/* icon is the full path name of the icon file.                  */
/* Return : NULL = not possible else the new file_type structure */
file_type *file_type_insert_type(FileType *pFileType,file_type *before,
  gchar *string,guint8 pos, gboolean is_link,gchar *program,gchar *icon);  

/* Remove a file type to the file type list     */
/* Return : FALSE = impossible; TRUE = job done */
gboolean file_type_remove_type(FileType *pFileType,file_type *remove_type);

/*************************************************************/
/************** FOR GOING OVER ALL THE FILE TYPES ************/

/* After a call to this procedure, file_type_current */
/* return the first file type                        */
void file_type_init(FileType *pFileType);

/* Return the type we are on */
file_type *file_type_current(FileType *pFileType);

/* Go to the next file_type */
void file_type_next(FileType *pFileType);

/* say if there is no more type */
gboolean file_type_is_end(FileType *pFileType);
/*************************************************************/

/* determine if the file type test_type is a special one      */
/* Return : TRUE = it is a special type which can't be remove */
/*	    FALSE = not a special type			      */
gboolean file_type_is_a_special_type(file_type *test_type);

/* swap two item in a file type list */
void file_type_swap_type(FileType *pFileType,file_type *type1,file_type *type2);

/* swap two FileType.                                       */
/* You CAN'T CALLED THIS IF YOU ARE USING file_type_next... */
void file_type_swap_file_type(FileType *pFileType1,FileType *pFileType2);

/* add all the file types of psrc at the end of pdest. */
/* If pdest is empty, this make a copy of psrc.        */
void file_type_append_file_type(FileType *pdest,FileType *psrc);

/* set the parent application to avoid warning when using */
/* gdk_pixmap_create_from_xpm_d                           */
void file_type_set_parent(GtkWidget *parent);

#endif /* __FILE_TYPE_H__ */
