/*******************************************/
/*					   */
/*  The main of GFile.			   */
/*					   */
/*  GFile is a File manager for Gnome      */
/*					   */
/*  Written by : Daniel Lacroix	           */
/*  GFile is under the GPL2 licence        */
/*******************************************/

#include "GFile.h"

/* file where we save the file type list. (file in GFILEDIR) */
#define FILE_TYPE_FILE "gfile_file_type"
/* directory where icons are on. (directory in GFILEDIR) */
#define ICON_PATH      "icons"
/* gfile look specific file */
#define RC_FILE	       "gfilerc"

/* GFILEDIR indicate where is the GFile base directory */
/* it is based on the environment variable GFILEDIR    */
/* This is ALWAYS ended by a '/'                       */
char *GFILEDIR;

/*****************************************/
/* all preferences are in this structure */
typedef struct _Preference Preference;

struct _Preference
{
  /* MODE_TREE_LIST, MODE_LIST_LIST or MODE_LIST */
  guint8 general_mode;
  
  GFileTreePref TreePreference;
  
  GFileListPref LeftListPreference;
  /*when one list is selected, we take preferences of the Right List*/
  GFileListPref RightListPreference;  
};
/*****************************************/

/* List of all file in copy/cut mode */
FileDataList *copy_file_list = NULL;
gboolean cut_mode; /* if TRUE,we are in a cut else a copy */

/* all preferences = file "gfile_preference" */
Preference gfile_preference;

GtkWidget *vbox;
/* need for MODE_LIST_LIST and MODE_TREE_LIST */
GtkWidget *hpaned;
/* for the right file list */
GtkWidget *RightPart;
/* For the left part (tree, list or nothing) */
GtkWidget *LeftPart;

/* to know if the user is now working in the left or the right part */
/* like that, if copy, del... is called job will be done in the good*/
/* part                                                             */
enum {LEFT_PART, RIGHT_PART};
guint8 active_part = LEFT_PART;

/* list of all known file type */
FileType *in_use_file_type_list;

/* list of all known group on this computer */
struct _Group *in_use_group_list;

/* list of all known users on this computer */
struct _Owner *in_use_owner_list;

/* main application Widget */
GtkWidget *app;

/* callback for the application */
void keypress_cb (GtkWidget *widget, GdkEventKey *event);

/* callback for the File menu */
void about_cb (GtkWidget *widget, void *data);
void quit_cb (GtkWidget *widget, void *data); /* use this "cb" for leaving */
void preference_cb (GtkWidget *widget, void *data);

/* callback for the New submenu */
void new_folder_cb (GtkWidget *widget, void *data);
void new_file_cb   (GtkWidget *widget, void *data);

/* callback for the Edition menu */
void copy_cb  (GtkWidget *widget, void *data);
void cut_cb   (GtkWidget *widget, void *data);
void paste_cb (GtkWidget *widget, void *data);
void delete_cb (GtkWidget *widget, void *data);

/* create the application */
void prepare_app();

/* delete all selected files */
void del_select(void);

/*--------------------- menu bar ---------------------------*/
GnomeUIInfo new_menu[] = {
  { GNOME_APP_UI_ITEM, "Folder...", NULL, new_folder_cb, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL },
  { GNOME_APP_UI_ITEM, "File...", NULL, new_file_cb, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL },
  { GNOME_APP_UI_ENDOFINFO }
};

GnomeUIInfo file_menu[] = {
  { GNOME_APP_UI_SUBTREE, ("New"), NULL, new_menu, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL },
  { GNOME_APP_UI_SEPARATOR, NULL, NULL, NULL, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL },
  { GNOME_APP_UI_ITEM, "Preference...", NULL, preference_cb, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PREF, 'P',
    GDK_CONTROL_MASK, NULL },
  { GNOME_APP_UI_SEPARATOR, NULL, NULL, NULL, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL },
  { GNOME_APP_UI_ITEM, "Exit", NULL, quit_cb, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_EXIT, 'Q',
    GDK_CONTROL_MASK, NULL },
  { GNOME_APP_UI_ENDOFINFO }
};

GnomeUIInfo edition_menu[] = {
  { GNOME_APP_UI_ITEM, "Copy", NULL, copy_cb, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_COPY, 'C',
    GDK_CONTROL_MASK, NULL },
  { GNOME_APP_UI_ITEM, "Cut", NULL, cut_cb, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_CUT, 'X',
    GDK_CONTROL_MASK, NULL },    
  { GNOME_APP_UI_ITEM, "Paste", NULL, paste_cb, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PASTE, 'V',
    GDK_CONTROL_MASK, NULL },
  { GNOME_APP_UI_ITEM, "Delete", NULL, delete_cb, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_TRASH, 0,
    GDK_CONTROL_MASK, NULL },
  { GNOME_APP_UI_ENDOFINFO }
};

GnomeUIInfo help_menu[] = {
  { GNOME_APP_UI_ITEM, "About...", NULL, about_cb, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_ABOUT, 0, 0, NULL },
  { GNOME_APP_UI_ENDOFINFO }
};

GnomeUIInfo main_menu[] = {
  { GNOME_APP_UI_SUBTREE, ("_File"), NULL, file_menu, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL },
  { GNOME_APP_UI_SUBTREE, ("_Edition"), NULL, edition_menu, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL },
/*  { GNOME_APP_UI_JUSTIFY_RIGHT, NULL, NULL, NULL, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL },*/
  { GNOME_APP_UI_SUBTREE, ("_Help"), NULL, help_menu, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL },
  { GNOME_APP_UI_ENDOFINFO }
};
/*---------------------- menu bar end ------------------------------*/

/***************************************************/
/* Load last preferences written on the hard drive */
void gfile_load_preference(void)
{
  /* pour charger les anciennes dispositions */
/*  gchar *tmppath;
  gint file_handle;
*/
  /* charge les preferences */
/*  tmppath = (gchar *)g_malloc(strlen(GFILEDIR)+9);
  if(tmppath == NULL) { g_print("Can't allocate memory."); exit(1); }
*/  /*strcpy(tmppath,GFILEDIR);
  strcat(tmppath,"/aspect");
  file_handle = open(tmppath,O_RDONLY);
  if(file_handle != -1)
  { read(file_handle,&gfile_pref,sizeof(pref));
    close(file_handle);
  }*/
  gfile_preference.general_mode = MODE_TREE_LIST;
  gfiletreepref_copy(&gfile_preference.TreePreference,&DefaultGFileTreePref);
  gfile_preference.TreePreference.file_type_list = in_use_file_type_list;
  gfilelistpref_copy(&gfile_preference.LeftListPreference ,&DefaultGFileListPref);
  gfile_preference.LeftListPreference.owner_list = in_use_owner_list;
  gfile_preference.LeftListPreference.group_list = in_use_group_list;
  gfile_preference.LeftListPreference.file_type_list = in_use_file_type_list;
  gfilelistpref_copy(&gfile_preference.RightListPreference,&DefaultGFileListPref);
  gfile_preference.RightListPreference.owner_list = in_use_owner_list;
  gfile_preference.RightListPreference.group_list = in_use_group_list;
  gfile_preference.RightListPreference.file_type_list = in_use_file_type_list;
}
/***************************************************/

/******************************/
/* fill the GFILEDIR variable */
void gfile_init_base_direcory(void)
{ gchar *tmp_string;
  struct stat *stat_buf;

  /* find the base directory */
    /* if GFILEDIR env variable is define, we take it */
  if(g_getenv("GFILEDIR") != NULL)
  {
      GFILEDIR = g_malloc(strlen(g_getenv("GFILEDIR"))+2);
      if(GFILEDIR == NULL){gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}
      strcpy(GFILEDIR,g_getenv("GFILEDIR"));
      /* if not ended by a '/' add it */
      if((strlen(GFILEDIR)>0) && (GFILEDIR[strlen(GFILEDIR)-1] != '/'))
        strcat(GFILEDIR,"/");
    /* else we try to found in HOME/.GFile */
  } else {
     if(g_get_home_dir() == NULL)
     {gfile_error(CREATE_GFILEDIR); exit(CREATE_GFILEDIR);}
     GFILEDIR = (gchar *)g_malloc(strlen(g_get_home_dir())+9);
     if(GFILEDIR == NULL){gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}
     strcpy(GFILEDIR,g_get_home_dir());
     if((strlen(GFILEDIR)>0) && (GFILEDIR[strlen(GFILEDIR)-1] != '/'))
       strcat(GFILEDIR,"/.GFile"); else strcat(GFILEDIR,".GFile");
     stat_buf = (struct stat *)g_malloc(sizeof(struct stat));
     if(stat_buf == NULL){gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}
     if(stat(GFILEDIR,stat_buf) == -1) /* if it does not exist */
     {
       g_print("No base directory define for GFile, HOME/.GFile will be made."\
               "\nYou can choose another by defining the environement"\
	       " variable GFILEDIR.\n"\
	       "ex : export GFILEDIR=/usr/local/gfile\n"\
	       "all icons MUST be place in GFILEDIR/%s\n",ICON_PATH);
       /* create the directory */
       if(mkdir(GFILEDIR, S_IRWXU) == -1)
       {gfile_error(CREATE_GFILEDIR); exit(CREATE_GFILEDIR);}
     }
     g_free(stat_buf);
     strcat(GFILEDIR,"/");
  }
  g_print("\nGFile base directory = %s \n",GFILEDIR);
}
/******************************/

/****************/
/* main program */
int main(int argc, char *argv[])
{ gchar *tmp_string;

  /*argp_program_version = VERSION;
  gnome_init ("GFile", NULL, argc, argv, 0, NULL);*/
  /* REPLACE BY THAT IF THAT DON'T WANT TO COMPILE */
  gnome_init ("GFile", VERSION, argc, argv);

  /* fill GFILEDIR */ 
  gfile_init_base_direcory();

  app = gnome_app_new ("GFile", "GFile");
  gtk_widget_realize (app);

  file_type_set_parent(app);
  
  /* Init the file type system use for the file type reconisation */
  in_use_file_type_list = file_type_new();
  if(in_use_file_type_list == NULL){gfile_error(CREATE_FILE_TYPE); exit(CREATE_FILE_TYPE);}
  tmp_string = g_malloc(strlen(GFILEDIR)+strlen(FILE_TYPE_FILE)+1);
  if(tmp_string == NULL){gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}
  sprintf(tmp_string,"%s%s",GFILEDIR,FILE_TYPE_FILE);
  file_type_load_from_file(in_use_file_type_list,tmp_string);
  g_free(tmp_string);

  /* load information on users and groups */
  in_use_group_list = group_load();
  if(in_use_group_list == NULL){gfile_error(CREATE_GROUP_LIST); exit(CREATE_GROUP_LIST);} 
  in_use_owner_list = owner_load(in_use_group_list);
  if(in_use_owner_list == NULL){gfile_error(CREATE_OWNER_LIST); exit(CREATE_OWNER_LIST);} 

  /* load preferences */
  gfile_load_preference();

  /* build the application */
  prepare_app ();

  /* load the file which define the look */
  tmp_string = g_malloc(strlen(GFILEDIR)+strlen(RC_FILE)+1);
  if(tmp_string == NULL){gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}
  sprintf(tmp_string,"%s%s",GFILEDIR,RC_FILE);
  gtk_rc_parse(tmp_string);
  g_free(tmp_string);

  gtk_main();

  return(0);
}
/****************/

/* For the right part (LIST)(only use in MODE_TREE_LIST)*/
void right_down_dir_cb(GtkWidget *widget, void *data)
{
  g_print("right down directory\n");
  gfiletree_down(GFILETREE(LeftPart));
} 

/* For the right part (LIST)(only use in MODE_TREE_LIST)*/
void right_up_dir_cb(GtkWidget *widget, gchar *directory)
{
  g_print("right up to directory : %s\n",directory);
  gfiletree_up(GFILETREE(LeftPart), directory);
} 

/* For the left part (TREE)(only use in MODE_TREE_LIST) */
void left_change_dir_cb(GtkWidget *widget, gchar *directory)
{
/*  g_print("Tree move to directory : %s\n",directory);*/
  if(gfilelist_open_dir(GFILELIST(RightPart),directory) == FALSE)
  {
    g_print("the directory doesn't exist\n"); 
    /* __________ NOT FINISH, WE MUST GO TO THE PARENT DIRECTORY AND
      SUPPRESS THIS DIRECTORY ___________________________________ */
  }	   
}

void LeftPart_active_cb(GtkWidget *widget,void *data)
{
  if(active_part == RIGHT_PART)
  {
/*    g_print("Left part selected\n");*/
    active_part = LEFT_PART;
    /* unselect the right part cause we are in the left part */
    /* what ever the general mode is, RightPart is ALWAYS a list */
    gfilelist_unselect_all(GFILELIST(RightPart));
  }  
}

void RightPart_active_cb(GtkWidget *widget,void *data)
{
  if(active_part == LEFT_PART)
  {
/*    g_print("Right part selected\n");*/
    active_part = RIGHT_PART;
    /* unselect the left part cause we are in the right part */
    /* LeftPart depend on the choosen general mode */
    if(gfile_preference.general_mode == MODE_LIST_LIST)
      gfilelist_unselect_all(GFILELIST(LeftPart));
    else  
      gfiletree_unselect_all(GFILETREE(LeftPart));
  }  
}

/* called eached time a file changed (name, size, rights ...) */
void file_mode_changed_cb(GtkWidget *Widget,FileData *SrcFile,
  FileData *DstFile,gpointer data)
{ gchar *str_SrcFile,*str_DstFile;

  switch(gfile_preference.general_mode)
  {
    case MODE_LIST :
      gfilelist_modify_file(GFILELIST(RightPart),SrcFile,DstFile);
      break;
    case MODE_TREE_LIST :
      str_SrcFile = g_strconcat(SrcFile->path,SrcFile->name,NULL);
      str_DstFile = g_strconcat(DstFile->path,DstFile->name,NULL); 
      gfiletree_modify_dir_by_name(GFILETREE(LeftPart),str_SrcFile,str_DstFile);
      g_free(str_SrcFile); g_free(str_DstFile);
      gfilelist_modify_file(GFILELIST(RightPart),SrcFile,DstFile);    
      break;    
    case MODE_LIST_LIST :        
      gfilelist_modify_file(GFILELIST(LeftPart),SrcFile,DstFile);    
      gfilelist_modify_file(GFILELIST(RightPart),SrcFile,DstFile);
      break;    
  }  
}  

void prepare_app()
{
  GtkMenuFactory *mf;
  
  gtk_signal_connect (GTK_OBJECT (app), "delete_event",
                      GTK_SIGNAL_FUNC (quit_cb),
                      NULL);

  gtk_signal_connect (GTK_OBJECT (app), "key_press_event",
                      GTK_SIGNAL_FUNC (keypress_cb),
                      NULL);

  /* create the menu */
  gnome_app_create_menus ( GNOME_APP (app), main_menu);
  vbox = gtk_vbox_new(0,0);
  gtk_widget_show(vbox);
  gnome_app_set_contents(GNOME_APP (app), vbox);

  switch(gfile_preference.general_mode)
  {
    case MODE_LIST :
      /* create a list */
      RightPart = gfilelist_new(gfile_preference.RightListPreference);
      gtk_signal_connect (GTK_OBJECT(RightPart), "file_mode_changed",
        GTK_SIGNAL_FUNC (file_mode_changed_cb), NULL);
      
/*      gtk_signal_connect (GTK_OBJECT(RightPart), "down_dir",
        GTK_SIGNAL_FUNC (right_down_dir_cb), NULL);
      gtk_signal_connect (GTK_OBJECT(RightPart), "up_dir",
        GTK_SIGNAL_FUNC (right_up_dir_cb), NULL);*/
      /* don't need to know where we are if there is only a list */	

      gtk_widget_show (GTK_WIDGET(RightPart));
      gfilelist_open_dir(GFILELIST(RightPart),"/");
   
      gtk_box_pack_start_defaults(GTK_BOX(vbox),RightPart);
      
      active_part = RIGHT_PART; /* always RIGHT_PART */
      break;
    case MODE_TREE_LIST :
      /* create the vertical division */        
      hpaned = gtk_hpaned_new ();
      gtk_box_pack_start_defaults(GTK_BOX(vbox),hpaned);      
      gtk_widget_show (hpaned);

      /* create a gfiletree in the left part */
      LeftPart = gfiletree_new(gfile_preference.TreePreference);
      gtk_signal_connect (GTK_OBJECT(LeftPart), "change_dir",
        GTK_SIGNAL_FUNC (left_change_dir_cb), NULL);
      gtk_signal_connect (GTK_OBJECT(((GFileTree *)LeftPart)->Ctree),"tree_select_row",
        GTK_SIGNAL_FUNC(LeftPart_active_cb), NULL);
		      
      gtk_widget_show (GTK_WIDGET(LeftPart));
      gtk_paned_add1 (GTK_PANED(hpaned), GTK_WIDGET(LeftPart));

      /* create a list in the right part */
      RightPart = gfilelist_new(gfile_preference.RightListPreference);
      gtk_signal_connect (GTK_OBJECT(RightPart), "file_mode_changed",
        GTK_SIGNAL_FUNC (file_mode_changed_cb), NULL);
 
      gtk_signal_connect (GTK_OBJECT(RightPart), "down_dir",
        GTK_SIGNAL_FUNC (right_down_dir_cb), NULL);
      gtk_signal_connect (GTK_OBJECT(RightPart), "up_dir",
        GTK_SIGNAL_FUNC (right_up_dir_cb), NULL);
      gtk_signal_connect (GTK_OBJECT(((GFileList *)RightPart)->Clist),"select_row",
        GTK_SIGNAL_FUNC(RightPart_active_cb), NULL);

      gtk_widget_show (GTK_WIDGET(RightPart));
      gfilelist_open_dir(GFILELIST(RightPart),"/");
      
      gtk_paned_add2 (GTK_PANED(hpaned), GTK_WIDGET(RightPart) );

      break;
    case MODE_LIST_LIST :
      /* create the vertical division */        
      hpaned = gtk_hpaned_new ();
      gtk_box_pack_start_defaults(GTK_BOX(vbox),hpaned);
      gtk_widget_show (hpaned);

      /* create a list in the left part */
      LeftPart = gfilelist_new(gfile_preference.LeftListPreference);
      gtk_signal_connect (GTK_OBJECT(LeftPart), "file_mode_changed",
        GTK_SIGNAL_FUNC (file_mode_changed_cb), NULL);
      
/*      gtk_signal_connect (GTK_OBJECT(LeftPart), "down_dir",
        GTK_SIGNAL_FUNC (left_down_dir_cb), NULL);
      gtk_signal_connect (GTK_OBJECT(LeftPart), "up_dir",
        GTK_SIGNAL_FUNC (left_up_dir_cb), NULL);*/
      gtk_signal_connect (GTK_OBJECT(((GFileList *)LeftPart)->Clist),"select_row",
        GTK_SIGNAL_FUNC(LeftPart_active_cb), NULL);

      gtk_widget_show (GTK_WIDGET(LeftPart));
      gfilelist_open_dir(GFILELIST(LeftPart),"/");
      
      gtk_paned_add1 (GTK_PANED(hpaned), GTK_WIDGET(LeftPart) );

      /* create a list in the right part */
      RightPart = gfilelist_new(gfile_preference.RightListPreference);
      gtk_signal_connect (GTK_OBJECT(RightPart), "file_mode_changed",
        GTK_SIGNAL_FUNC (file_mode_changed_cb), NULL);
      
/*      gtk_signal_connect (GTK_OBJECT(RightPart), "down_dir",
        GTK_SIGNAL_FUNC (right_down_dir_cb), NULL);
      gtk_signal_connect (GTK_OBJECT(RightPart), "up_dir",
        GTK_SIGNAL_FUNC (right_up_dir_cb), NULL);*/
      gtk_signal_connect (GTK_OBJECT(((GFileList *)RightPart)->Clist),"select_row",
        GTK_SIGNAL_FUNC(RightPart_active_cb), NULL);

      gtk_widget_show (GTK_WIDGET(RightPart));
      gfilelist_open_dir(GFILELIST(RightPart),"/");
      
      gtk_paned_add2 (GTK_PANED(hpaned), GTK_WIDGET(RightPart) );
    
      break;    
  }      
  gtk_widget_show (app);
}

/* Callbacks functions */

/* This is called eached time a key is pressed */
void keypress_cb (GtkWidget *widget, GdkEventKey *event)
{
  /*g_print("Key pressed : %d\n",event->keyval);*/
  switch(event->keyval)
  {
     case 65307 : /* Esc = quit */
                  quit_cb(widget,event);
     		  break;
     case 65535 : /* Del = delete all the selected files */
     		  g_print("Del pressed\n");
		  delete_cb(NULL,NULL);
     		  break;
  }
}

/* delete the file selected in the list only for now */
void del_select(void)
{// GList *select_row;
//  gchar *tmp_string,*tmp_string2;
 
//  select_row = gfilelist_list_del_selected(GFILELIST(RightPart));

//  while(select_row)
//  { /* if there is data and file is not . or .. */
//    if((select_row->data)&&
//       strcmp(((GFileListData *)select_row->data)->name,".") &&
//       strcmp(((GFileListData *)select_row->data)->name,"..") )
//    { /* allocate the needed memory to write the command */
//      tmp_string = g_malloc(strlen(((GFileListData *)select_row->data)->path)+
//      			    strlen(((GFileListData *)select_row->data)->name)+
//			    9);
//      if(tmp_string == NULL){g_print("Can't allocate memory.\n"); exit(1);}
//      if( ((GFileListData *)select_row->data)->mode & GFILE_LIST_IS_DIRECTORY )
//      {
//        strcpy(tmp_string,"rm -R \""); /*if this is a directory del it recursively*/
//        /*_________ WORK TO FINISH HERE ______________*/
//        tmp_string2 = g_malloc(strlen(((GFileListData *)select_row->data)->path)+
//      		      strlen(((GFileListData *)select_row->data)->name)+1);
//        if(tmp_string2 == NULL){g_print("Can't allocate memory.\n"); exit(1);}
//        strcpy(tmp_string2,((GFileListData *)select_row->data)->path);
//        strcat(tmp_string2,((GFileListData *)select_row->data)->name);
//        gfiletree_remove_dir_by_name(GFILETREE(LeftPart),tmp_string2);
//	g_free(tmp_string2);
//	/*________ HERE ADD DEL IN CTREE _____________*/
//      } 
//      else 
//        strcpy(tmp_string,"rm \"");
//      strcat(tmp_string,((GFileListData *)select_row->data)->path);
//      strcat(tmp_string,((GFileListData *)select_row->data)->name);
//      strcat(tmp_string,"\"");
//      g_print("> %s\n",tmp_string);
//      system(tmp_string);
//      /* remove the entry in the list */
//      gfilelist_remove_file_by_full_name(GFILELIST(RightPart),(GFileListData *)select_row->data);
//      g_free(tmp_string);
//    }
//    select_row = select_row->next;
//  }
  
//  gfilelist_unselect_all(GFILELIST(RightPart));
}

/* callback du menu File */

void new_folder_cb (GtkWidget *widget, void *data)
{
  printf("new folder appele\n");
}

void new_file_cb (GtkWidget *widget, void *data)
{
//  GtkWidget *window;

  printf("new file appele\n");

//  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
//  gtk_window_set_title (GTK_WINDOW (window), "New file");
//  gtk_signal_connect (GTK_OBJECT (window), "destroy",
//  GTK_SIGNAL_FUNC (gtk_widget_destroyed), &window);

//  gtk_widget_show (window);
}

/**********************************************************/
/* change the general mode (choice are : MODE_TREE_LIST,  */ 
/* MODE_LIST_LIST and MODE_LIST)                          */
static void change_general_mode(guint8 new_general_mode)
{
  /* if not necessary, leave */
  if(gfile_preference.general_mode == new_general_mode) return;

  /* RightPart is separate from its parent and suppress */
  gtk_container_remove(GTK_CONTAINER(RightPart->parent), RightPart);

  /* create a list in the right part */
  RightPart = gfilelist_new(gfile_preference.RightListPreference);
  gtk_signal_connect (GTK_OBJECT(RightPart), "file_mode_changed",
    GTK_SIGNAL_FUNC (file_mode_changed_cb), NULL);
  
  if(new_general_mode == MODE_TREE_LIST)
  {
    gtk_signal_connect (GTK_OBJECT(RightPart), "down_dir",
      GTK_SIGNAL_FUNC (right_down_dir_cb), NULL);
    gtk_signal_connect (GTK_OBJECT(RightPart), "up_dir",
      GTK_SIGNAL_FUNC (right_up_dir_cb), NULL);
  }    
  gtk_signal_connect (GTK_OBJECT(((GFileList *)RightPart)->Clist),"select_row",
    GTK_SIGNAL_FUNC(RightPart_active_cb), NULL);

  gtk_widget_show (GTK_WIDGET(RightPart));
  gfilelist_open_dir(GFILELIST(RightPart),"/");

  /* destroy the left part because we are sure that it is no more good */
  if((gfile_preference.general_mode == MODE_LIST_LIST) || 
     (gfile_preference.general_mode == MODE_TREE_LIST))
  {   
    /* LeftPart will be separate from its parent and remove */
    gtk_container_remove(GTK_CONTAINER(LeftPart->parent), LeftPart);
  }
  
  /* build the new left part */
  switch(new_general_mode)
  {
    case MODE_LIST_LIST :
      /* create a list in the left part */
      LeftPart = gfilelist_new(gfile_preference.LeftListPreference);
      gtk_signal_connect (GTK_OBJECT(LeftPart), "file_mode_changed",
        GTK_SIGNAL_FUNC (file_mode_changed_cb), NULL);
/*      gtk_signal_connect (GTK_OBJECT(LeftPart), "down_dir",
        GTK_SIGNAL_FUNC (left_down_dir_cb), NULL);
      gtk_signal_connect (GTK_OBJECT(LeftPart), "up_dir",
        GTK_SIGNAL_FUNC (left_up_dir_cb), NULL);*/
      gtk_signal_connect (GTK_OBJECT(((GFileList *)LeftPart)->Clist),"select_row",
        GTK_SIGNAL_FUNC(LeftPart_active_cb), NULL);

      gtk_widget_show (GTK_WIDGET(LeftPart));
      gfilelist_open_dir(GFILELIST(LeftPart),"/");
      break;
    case MODE_TREE_LIST :
      /* create a gfiletree in the left part */
      LeftPart = gfiletree_new(gfile_preference.TreePreference);
      gtk_signal_connect (GTK_OBJECT(LeftPart), "change_dir",
        GTK_SIGNAL_FUNC (left_change_dir_cb), NULL);
      gtk_signal_connect (GTK_OBJECT(((GFileTree *)LeftPart)->Ctree),"tree_select_row",
        GTK_SIGNAL_FUNC(LeftPart_active_cb), NULL);
		      
      gtk_widget_show (GTK_WIDGET(LeftPart));
      /* right part restart to "/" to be in armony with the tree */
      gfilelist_open_dir(GFILELIST(RightPart),"/");
      break;      
  }
  
  /* if last mode was MODE_LIST, hpaned must be add */
  if(gfile_preference.general_mode == MODE_LIST)
  {
    /* create the vertical division */
    hpaned = gtk_hpaned_new ();
    gtk_widget_show (hpaned);
  } 
  
  if((new_general_mode == MODE_LIST_LIST) || 
     (new_general_mode == MODE_TREE_LIST))
  {   
    gtk_paned_add1 (GTK_PANED(hpaned), GTK_WIDGET(LeftPart) );
    gtk_paned_add2 (GTK_PANED(hpaned), GTK_WIDGET(RightPart) );
    if(gfile_preference.general_mode == MODE_LIST)
      gtk_box_pack_start_defaults(GTK_BOX(vbox),hpaned);
      
  } else {
    /* new_general_mode is MODE_LIST */
    /* hpaned is separate from its parent and suppress */
    gtk_container_remove(GTK_CONTAINER(hpaned->parent), hpaned);    
    gtk_box_pack_start_defaults(GTK_BOX(vbox),RightPart);
  }  

  gfile_preference.general_mode = new_general_mode;
}
/**********************************************************/

/* procedure executed when Apply is pressed in the preference dialog */
static void preference_dialog_apply_cb(PreferenceDialog *pPreferenceDialog,
  PreferenceDialogPref *pPreferenceDialogPref, void *data)
{
  g_print("Apply pressed\n");
  
  /* replace old file type list by the new one */
  file_type_free(in_use_file_type_list);
  /* copy the new one at the place of the old one */
  in_use_file_type_list = file_type_new();
  file_type_append_file_type(in_use_file_type_list,
    pPreferenceDialogPref->file_type_list);

  gfile_preference.RightListPreference.file_type_list = in_use_file_type_list;
  gfile_preference.LeftListPreference.file_type_list  = in_use_file_type_list;
  gfile_preference.TreePreference.file_type_list      = in_use_file_type_list;


  /* change the file type list for tree and lists */
  if(gfile_preference.general_mode == MODE_TREE_LIST)
    gfiletree_set_filetype(GFILETREE(LeftPart),in_use_file_type_list);
  else if(gfile_preference.general_mode == MODE_LIST_LIST)
    gfilelist_set_filetype(GFILELIST(LeftPart),in_use_file_type_list);
  
  /* RightPart is always a list */
  gfilelist_set_filetype(GFILELIST(RightPart),in_use_file_type_list);

  /* we change the general mode if needed */
  change_general_mode(pPreferenceDialogPref->general_mode);
  
  if(gfile_preference.LeftListPreference.sort_mode != pPreferenceDialogPref->left_list_sort_mode)
  {
    gfile_preference.LeftListPreference.sort_mode = pPreferenceDialogPref->left_list_sort_mode;
    if(gfile_preference.general_mode == MODE_LIST_LIST)
      gfilelist_set_sort(GFILELIST(LeftPart),gfile_preference.LeftListPreference.sort_mode);
  }
  if(gfile_preference.RightListPreference.sort_mode != pPreferenceDialogPref->right_list_sort_mode)
  {
    gfile_preference.RightListPreference.sort_mode = pPreferenceDialogPref->right_list_sort_mode;
    gfilelist_set_sort(GFILELIST(RightPart),gfile_preference.RightListPreference.sort_mode);
  }  
}

void preference_cb (GtkWidget *widget, void *data)
{ PreferenceDialogPref *vPreferenceDialogPref;
  GtkWidget *vPreferenceDialog;
  
  vPreferenceDialogPref = (PreferenceDialogPref *)g_malloc(sizeof(PreferenceDialogPref));
  if(vPreferenceDialogPref == NULL){gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}
  vPreferenceDialogPref->general_mode = gfile_preference.general_mode;
  /* replace this by the sort mode use now */
  if(gfile_preference.general_mode == MODE_LIST_LIST)
    vPreferenceDialogPref->left_list_sort_mode  = 
      gfilelist_get_sort(GFILELIST(LeftPart));
  else
    vPreferenceDialogPref->left_list_sort_mode  = 
      gfile_preference.LeftListPreference.sort_mode;
  /* replace this by the sort mode use now (right part is always a list)*/
  vPreferenceDialogPref->right_list_sort_mode = 
    gfilelist_get_sort(GFILELIST(RightPart));
    
  vPreferenceDialogPref->owner_list = in_use_owner_list;
  vPreferenceDialogPref->group_list = in_use_group_list;
  vPreferenceDialogPref->file_type_list = in_use_file_type_list;
  vPreferenceDialogPref->icon_path = g_malloc(strlen(GFILEDIR)+strlen(ICON_PATH)+2);
  if(vPreferenceDialogPref->icon_path == NULL)
  {gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}
  sprintf(vPreferenceDialogPref->icon_path,"%s%s/",GFILEDIR,ICON_PATH);
  vPreferenceDialogPref->file_type_save_file = g_malloc(strlen(GFILEDIR)+strlen(FILE_TYPE_FILE)+1);
  if(vPreferenceDialogPref->file_type_save_file == NULL)
  {gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}
  sprintf(vPreferenceDialogPref->file_type_save_file,"%s%s",GFILEDIR,FILE_TYPE_FILE);

  vPreferenceDialog = preference_dialog_new(*vPreferenceDialogPref);

  gtk_signal_connect(GTK_OBJECT(vPreferenceDialog),"apply",
    GTK_SIGNAL_FUNC(preference_dialog_apply_cb),NULL);
    
  gtk_widget_show(vPreferenceDialog);

  g_free(vPreferenceDialogPref->icon_path);
  g_free(vPreferenceDialogPref->file_type_save_file);
  g_free(vPreferenceDialogPref);
  
/*  printf("preferences called\n");*/
}

void quit_cb (GtkWidget *widget, void *data)
{// gchar *tmppath;
//  gint file_handle;
//  guint16 taille[4];
//  gint col;
  
/*  tmppath = (gchar *)g_malloc(strlen(GFILEDIR)+8);*/
  
  /* on enregistre les infos de position */
/*  gfile_pref.ctree_width  = (GTK_WIDGET(ctree))->allocation.width;
  gfile_pref.ctree_height = (GTK_WIDGET(ctree))->allocation.height;
  gfile_pref.clist_width  = (GTK_WIDGET(clist))->allocation.width;
  gfile_pref.clist_height = (GTK_WIDGET(clist))->allocation.height;
  gfile_pref.nb_list_col  = ((GtkCList *)clist)->columns;*/
/*
  if(gfile_pref.nb_list_col > 0) 
    gfile_pref.col1_width = ((GtkCList *)clist)->column[0].width;
  if(gfile_pref.nb_list_col > 1) 
    gfile_pref.col2_width = ((GtkCList *)clist)->column[1].width;
  if(gfile_pref.nb_list_col > 2) 
    gfile_pref.col3_width = ((GtkCList *)clist)->column[2].width;
  if(gfile_pref.nb_list_col > 3) 
    gfile_pref.col4_width = ((GtkCList *)clist)->column[3].width;
  if(gfile_pref.nb_list_col > 4) 
    gfile_pref.col5_width = ((GtkCList *)clist)->column[4].width;
*/  
/*  g_print("ctree w : %u, h : %u\n",(GTK_WIDGET(ctree))->allocation.width,
          (GTK_WIDGET(ctree))->allocation.height); */
/*
  g_print("clist w : %u, h : %u\n",(GTK_WIDGET(clist))->allocation.width,
          (GTK_WIDGET(clist))->allocation.height); 

  for(col=0;col<((GtkCList *)clist)->columns;col++)
  {
    g_print("col %d width : %d\n",col,((GtkCList *)clist)->column[col].width);
  }
*/
/*  strcpy(tmppath,GFILEDIR);
  strcat(tmppath,"/aspect");
  file_handle = creat(tmppath,S_IREAD|S_IWRITE);
  if(file_handle != -1)
  {
    write(file_handle,&gfile_pref,sizeof(pref));
    close(file_handle);
  }
  g_free(tmppath);*/

  
  /* Free the file type system */  
  file_type_free(in_use_file_type_list);
  
  /* Free the users/groups */
  owner_free(in_use_owner_list);
  group_free(in_use_group_list);
  
  gtk_main_quit();
  return;
}

/* callback du menu Edition */
/* pas de limitation de taille de dir/fich */
void copy_cb (GtkWidget *widget, void *data)
{// GList *select_row;
 
//  select_row = gfilelist_list_copy_selected(GFILELIST(RightPart));
//  copy_file_list = select_row;
//  select_row = select_row->next;
//  while(select_row)
//  {
//    g_print("copy file : %s\n",((GFileListData *)select_row->data)->name);
//    select_row = select_row->next;
//  }
//  cut_mode = FALSE;
  if(copy_file_list != NULL){file_data_list_free(copy_file_list); copy_file_list = NULL;}
  switch(active_part)
  { 
    case LEFT_PART :
      if(gfile_preference.general_mode == MODE_LIST_LIST)
        copy_file_list = gfilelist_list_selected(GFILELIST(LeftPart));
      else
        g_print("Tree part not done.\n"); 
      break;
    case RIGHT_PART :
      copy_file_list = gfilelist_list_selected(GFILELIST(RightPart));
      break;  
  }      
  cut_mode = FALSE;
}

void cut_cb (GtkWidget *widget, void *data)
{// GList *select_row;
 
//  select_row = gfilelist_list_copy_selected(GFILELIST(RightPart));
//  copy_file_list = select_row;
//  select_row = select_row->next;
//  while(select_row)
//  {
//    g_print("cut file : %s\n",((GFileListData *)select_row->data)->name);
//    select_row = select_row->next;
//  }
//  cut_mode = TRUE;
  if(copy_file_list != NULL){file_data_list_free(copy_file_list); copy_file_list = NULL;}
  switch(active_part)
  { 
    case LEFT_PART :
      if(gfile_preference.general_mode == MODE_LIST_LIST)
        copy_file_list = gfilelist_list_selected(GFILELIST(LeftPart));
      else
        g_print("Tree part not done.\n"); 
      break;
    case RIGHT_PART :
      copy_file_list = gfilelist_list_selected(GFILELIST(RightPart));
      break;  
  }      
  cut_mode = TRUE;
}

/* transform an abolute name into a relative one */
void abs_2_rel(gchar *full_name,gchar **path,gchar **name)
{
  guint lastchar = 0;
  guint length = 0;
  guint16 compt;

  if(strcmp(full_name,"/")) /* if we are not a the root ("/") */
  {
     /* find */
     for(lastchar=0;full_name[lastchar];lastchar++);
     length = lastchar;
     lastchar-=2;
     for(;lastchar && (full_name[lastchar]!='/');lastchar--);
     g_print("length : %u, path length : %u.\n",length,lastchar);
     /* make a copy of the good part */
     *path = g_malloc(lastchar+2);
     for(compt=0;compt<=lastchar;compt++) (*path)[compt]=full_name[compt];
     (*path)[lastchar+1]=0;
     *name = g_malloc(length-lastchar);
     for(;(compt<length) && full_name[compt] != '/';compt++) (*name)[compt-lastchar-1]=full_name[compt];
     (*name)[compt-lastchar-1] = 0;
  } else 
  { *path = g_malloc(1); path[0] = 0;
    *name = g_malloc(2); strcpy(*name,"/");
  }
}

void file_added_cb (gchar *NewFile)
{
  g_print("GFILE : New file to add : %s.\n",NewFile);
  switch(gfile_preference.general_mode)
  {
    case MODE_LIST :
      gfilelist_add_file_by_full_name(GFILELIST(RightPart),NewFile);
      break;
    case MODE_LIST_LIST :
      gfilelist_add_file_by_full_name(GFILELIST(LeftPart),NewFile);
      gfilelist_add_file_by_full_name(GFILELIST(RightPart),NewFile);      
      break;
    case MODE_TREE_LIST :
      gfiletree_add_dir_by_name(GFILETREE(LeftPart),NewFile);
      gfilelist_add_file_by_full_name(GFILELIST(RightPart),NewFile);    
      break;      
  }
}

void file_move_cb (gchar *LastFile,gchar *NewFile)
{
  g_print("GFILE : File to move : %s.\n",LastFile);
  switch(gfile_preference.general_mode)
  {
    case MODE_LIST :
      gfilelist_remove_file_by_full_name(GFILELIST(RightPart),LastFile);
      gfilelist_add_file_by_full_name(GFILELIST(RightPart),NewFile);
      break;
    case MODE_LIST_LIST :
      gfilelist_remove_file_by_full_name(GFILELIST(LeftPart),LastFile);
      gfilelist_add_file_by_full_name(GFILELIST(LeftPart),NewFile);
      gfilelist_remove_file_by_full_name(GFILELIST(RightPart),LastFile);      
      gfilelist_add_file_by_full_name(GFILELIST(RightPart),NewFile);      
      break;
    case MODE_TREE_LIST :
      gfiletree_remove_dir_by_name(GFILETREE(LeftPart),LastFile);
      gfiletree_add_dir_by_name(GFILETREE(LeftPart),NewFile);
      gfilelist_remove_file_by_full_name(GFILELIST(RightPart),LastFile);
      gfilelist_add_file_by_full_name(GFILELIST(RightPart),NewFile);
      break;      
  }
}

/* TEMPORARY IMPLEMENTATION */
void paste_cb (GtkWidget *widget, void *data)
{
  if(copy_file_list == NULL) return;
  /* cut */
  if(cut_mode == TRUE)
  {
    if(active_part == RIGHT_PART)
      move_dialog_new(copy_file_list,GFILELIST(RightPart)->current_path,file_move_cb);
    else {
      if(gfile_preference.general_mode == MODE_LIST_LIST)
        move_dialog_new(copy_file_list,GFILELIST(LeftPart)->current_path,file_move_cb);
      else 
        move_dialog_new(copy_file_list,GFILETREE(LeftPart)->current_path,file_move_cb);
    }  
  /* else copy */    
  } else {
    if(active_part == RIGHT_PART)
      copy_dialog_new(copy_file_list,GFILELIST(RightPart)->current_path,file_added_cb);
    else {
      if(gfile_preference.general_mode == MODE_LIST_LIST)
        copy_dialog_new(copy_file_list,GFILELIST(LeftPart)->current_path,file_added_cb);
      else 
        copy_dialog_new(copy_file_list,GFILETREE(LeftPart)->current_path,file_added_cb);    
    }  
  }
// GList *pos_list;
//  guint path_size = 0;
//  guint name_max_size = 0;
//  gchar *tmpstring;
//  gchar *org_path;

//  if(copy_file_list == NULL) return; /* rien a faire */
//  pos_list = copy_file_list->next;
//  while(pos_list)
//  {
//    if(pos_list->data)
//    {
//      tmpstring=(gchar *)g_malloc(strlen(((GFileListData *)pos_list->data)->path)+
//                          strlen(((GFileListData *)pos_list->data)->name)+
//			  strlen(GFILELIST(RightPart)->current_path)+11+1);
//      if(tmpstring == NULL){g_print("Can't allocate memory\n"); exit(1);}
      /*____ DO A COPY OR CUT ____________*/
      /* if we it is a directory */
//      if( ((GFileListData *)pos_list->data)->mode & GFILE_LIST_IS_DIRECTORY )
//        sprintf(tmpstring,"cp -R \"%s%s\" \"%s\"",
//                ((GFileListData *)pos_list->data)->path,
//	        ((GFileListData *)pos_list->data)->name,
//                GFILELIST(RightPart)->current_path);
      /* else */
//      else
//        sprintf(tmpstring,"cp \"%s%s\" \"%s\"",
//                ((GFileListData *)pos_list->data)->path,
//	        ((GFileListData *)pos_list->data)->name,
//                GFILELIST(RightPart)->current_path);
//      if(cut_mode == TRUE)
//      { tmpstring[0] = 'm';  tmpstring[1] = 'v'; 
//        if(tmpstring[3] =='-'){tmpstring[3] = ' ';  tmpstring[4] = ' ';}
//      }
//      g_print("%s\n",tmpstring);
//      system(tmpstring); /* on execute la commande */
//      /* update the tree and the list */
//      strcpy(tmpstring,GFILELIST(RightPart)->current_path);
//      strcat(tmpstring,((GFileListData *)pos_list->data)->name);				 
//      gfilelist_add_file_by_full_name(GFILELIST(RightPart),
//                                 tmpstring);
//      if( ((GFileListData *)pos_list->data)->mode & GFILE_LIST_IS_DIRECTORY )
//      {
//        gfiletree_add_dir_by_name(GFILETREE(LeftPart),tmpstring);
//        /* if it is a cut, suppress the source just in the tree because */
//        /* we can't in the source directory (or we have done nothing)*/				 
//        if(cut_mode == TRUE) 
//	{
//	  strcpy(tmpstring,((GFileListData *)pos_list->data)->path);
//          strcat(tmpstring,((GFileListData *)pos_list->data)->name);				 
//          gfiletree_remove_dir_by_name(GFILETREE(LeftPart),tmpstring);
//	}  
//      }
//	
//      g_free(tmpstring);
//    }  
//    pos_list = pos_list->next;
//  }
}

void file_del_func_cb(gchar *LastFile)
{
  g_print("GFILE : File to remove : %s.\n",LastFile);
  switch(gfile_preference.general_mode)
  {
    case MODE_LIST :
      gfilelist_remove_file_by_full_name(GFILELIST(RightPart),LastFile);    
      break;
    case MODE_LIST_LIST :
      gfilelist_remove_file_by_full_name(GFILELIST(LeftPart),LastFile);
      gfilelist_remove_file_by_full_name(GFILELIST(RightPart),LastFile);
      break;
    case MODE_TREE_LIST :
      gfiletree_remove_dir_by_name(GFILETREE(LeftPart),LastFile);
      gfilelist_remove_file_by_full_name(GFILELIST(RightPart),LastFile);
      break;      
  }
}

void delete_cb (GtkWidget *widget, void *data)
{ FileDataList *del_file_list;

  switch(active_part)
  { 
    case LEFT_PART :
      if(gfile_preference.general_mode == MODE_LIST_LIST)
        del_file_list = gfilelist_list_selected(GFILELIST(LeftPart));
      else
        g_print("Tree part not done.\n"); return;
      break;
    case RIGHT_PART :
      del_file_list = gfilelist_list_selected(GFILELIST(RightPart));
      break;  
  }
  del_dialog_new(del_file_list,file_del_func_cb);
  
  file_data_list_free(del_file_list);
}

/* Help menu callback */
void about_cb (GtkWidget *widget, void *data)
{
  GtkWidget *about;
  const gchar *authors[] = {
	  "Daniel Lacroix",
          NULL
          };
  about = gnome_about_new ("GFile", VERSION,
        		/* copyright notice */
                        "(C) 1999 the Free Software Foundation",
                        authors,
                        /* another comments */
                        "GFile is a File Manager",
                        NULL);
  gtk_widget_show (about);
}
