/****************************************/
/*  GFileList.c body         		*/
/*					*/
/*  Description : Manage the file list  */
/*		  widget		*/
/*					*/
/*  Written by : Daniel Lacroix	        */
/*  GFile is under the GPL2 licence     */
/****************************************/

/*********************************************/
/* A FAIRE : 			             */
/* - creer un signal qui previent lorsque    */
/*   le repertoire est relu et qu'il peut    */
/*   y avoir des changements (directory qui  */
/*   n'existe plus). 			     */
/* - probleme de la taille minimum et de la  */
/*   taille de preference au demarrage.      */
/* - pouvoir changer a chaux les preferences */
/*********************************************/

#include "GFile.h"

/* can be used for external use */
guint gfilelist_get_type ();
void gfilelist_set_sort(GFileList *pGFileList, guint8 sort_mode);
GtkWidget* gfilelist_new (GFileListPref pGFileListPref);
void gfilelist_free(GFileList *pGFileList);
void gfilelist_add_file(GFileList *pGFileList,FileData *pFileData);
void gfilelist_add_file_by_name(GFileList *pGFileList,gchar *name);
void gfilelist_add_file_by_full_name(GFileList *pGFileList,gchar *full_name);
void gfilelist_remove_file_with_check(GFileList *pGFileList,FileData *pFileData);
void gfilelist_remove_file(GFileList *pGFileList,FileData *pFileData);
void gfilelist_remove_all(GFileList *pGFileList);
void gfilelist_fill_status(GFileList *pGFileList);
gboolean gfilelist_open_dir(GFileList *pGFileList,gchar *name);
void gfilelist_set_filetype(GFileList *pGFileList,FileType *pFileType);
FileDataList *gfilelist_list_selected(GFileList *pGFileList);
void gfilelist_unselect_all(GFileList *pGFileList);
void gfilelistpref_copy(GFileListPref *pGFileListPref1,
  GFileListPref *pGFileListPref2);

/* internal use only */
static void string_marshal(GtkObject *object, GtkSignalFunc func, gpointer func_data, GtkArg *args);
static void gfilelist_class_init(GFileListClass *class);
static void gfilelist_init (GFileList *pGFileList);  
static gchar *gfilelist_date_2_string(GFileList *pGFileList,guint64 date);  
static void gfile_list_rights_2_string(guint16 rights,guint16 mode,gchar string[]);
static void down_dir(gchar *path,gchar *pathdest);
static void gfilelist_row_select_cb(GtkCList *clist,gint row,gint column, GdkEvent
  *event,GFileList *pGFileList);
static void gfilelist_row_unselect_cb(GtkCList *clist,gint row,gint column, GdkEvent
  *event,GFileList *pGFileList);  
static gint gfilelist_prop_menu_cb(GtkCList *clist, GdkEvent *event,GFileList
  *pGFileList);

static void prop_menu_properties_cb(GtkWidget *Widget,GFileList *pGFileList);
			
/* The default Preference */
GFileListPref DefaultGFileListPref =
{
  FALSE,       /* don't hide the clist title */
  "Name",0,60,
  "Owner",1,40,
  "Group",2,40,
  "Rights",3,60,
  "Size",4,40,
  "Access date",5,70,
  "Modif date",6,70,
  "Change date",7,70,  
  18,
  300,200,
  TRUE,		/* allow the status bar */
  129,		/* sort mode */
  TRUE,         /* allow the right click menu */
  NULL,         /* owner_list can't have a default value */
  NULL,         /* group_list can't have a default value */
  NULL          /* file_type_list can't have a default value */
}; 

/*************************************************************/
guint gfilelist_get_type ()
{
  static guint gfilelist_type = 0;

  if (!gfilelist_type)
  {
    GtkTypeInfo gfilelist_info =
    {
       "GFileList",
       sizeof (GFileList),
       sizeof (GFileListClass),
       (GtkClassInitFunc)  gfilelist_class_init,
       (GtkObjectInitFunc) gfilelist_init,
       (GtkArgSetFunc) NULL,
       (GtkArgGetFunc) NULL
     };
     gfilelist_type = gtk_type_unique(gtk_vbox_get_type(), &gfilelist_info);
  }
  return gfilelist_type;
}
/*************************************************************/

/*************************************************************/
enum { UP_DIR_SIGNAL, DOWN_DIR_SIGNAL, FILE_MODE_CHANGED_SIGNAL, LAST_SIGNAL};
static gint gfilelist_signals[LAST_SIGNAL] = { 0 };

static void string_marshal(GtkObject *object, GtkSignalFunc func, gpointer func_data, GtkArg *args) {
    typedef void (*string_marshal_func)(GFileList *, gchar *,gpointer);
    string_marshal_func rfunc = (string_marshal_func) func;
    (*rfunc) (GFILELIST(object),
	      GTK_VALUE_POINTER(args[0]), func_data);
}

static void two_file_data_marshal(GtkObject *object, GtkSignalFunc func, gpointer func_data, GtkArg *args) {
    typedef void (*two_file_data_marshal_func)(GFileList *, FileData *, FileData *,gpointer);
    two_file_data_marshal_func rfunc = (two_file_data_marshal_func) func;
    (*rfunc) (GFILELIST(object),
	      GTK_VALUE_POINTER(args[0]),GTK_VALUE_POINTER(args[1]),func_data );
}

static void gfilelist_class_init(GFileListClass *class)
{
/* here we create the new signals if we want */
  GtkObjectClass *object_class;
  object_class = (GtkObjectClass*) class;
       
  gfilelist_signals[UP_DIR_SIGNAL] = gtk_signal_new ("up_dir",
            GTK_RUN_FIRST,object_class->type,
            GTK_SIGNAL_OFFSET (GFileListClass,up_dir),
            string_marshal, GTK_TYPE_NONE, 1,GTK_TYPE_POINTER);

  gfilelist_signals[DOWN_DIR_SIGNAL] = gtk_signal_new ("down_dir",
            GTK_RUN_FIRST,object_class->type,
            GTK_SIGNAL_OFFSET (GFileListClass,down_dir),
            gtk_signal_default_marshaller, GTK_TYPE_NONE, 0);

  gfilelist_signals[FILE_MODE_CHANGED_SIGNAL] = gtk_signal_new
            ("file_mode_changed",
            GTK_RUN_FIRST,object_class->type,
            GTK_SIGNAL_OFFSET (GFileListClass,file_mode_changed),
            two_file_data_marshal, GTK_TYPE_NONE, 2,GTK_TYPE_POINTER,
	    GTK_TYPE_POINTER);

   gtk_object_class_add_signals (object_class, gfilelist_signals, LAST_SIGNAL);
   class->up_dir = NULL; class->down_dir = NULL; class->file_mode_changed = NULL;
}
/*************************************************************/

/* copy the content of  pGFileListPref2 in pGFileListPref1 */
void gfilelistpref_copy(GFileListPref *pGFileListPref1,
  GFileListPref *pGFileListPref2)
{
  pGFileListPref1->hide_title = pGFileListPref2->hide_title;
  pGFileListPref1->name_title = g_malloc(strlen(pGFileListPref2->name_title)+1);
  if(pGFileListPref1->name_title == NULL){gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}
  strcpy(pGFileListPref1->name_title,pGFileListPref2->name_title);
  pGFileListPref1->name_pos = pGFileListPref2->name_pos;
  pGFileListPref1->name_width = pGFileListPref2->name_width;
  pGFileListPref1->owner_title = g_malloc(strlen(pGFileListPref2->owner_title)+1);
  if(pGFileListPref1->owner_title == NULL){gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}
  strcpy(pGFileListPref1->owner_title,pGFileListPref2->owner_title);
  pGFileListPref1->owner_pos = pGFileListPref2->owner_pos;
  pGFileListPref1->owner_width = pGFileListPref2->owner_width;
  pGFileListPref1->group_title = g_malloc(strlen(pGFileListPref2->group_title)+1);
  if(pGFileListPref1->group_title == NULL){gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}
  strcpy(pGFileListPref1->group_title,pGFileListPref2->group_title);
  pGFileListPref1->group_pos = pGFileListPref2->group_pos;
  pGFileListPref1->group_width = pGFileListPref2->group_width;
  pGFileListPref1->rights_title = g_malloc(strlen(pGFileListPref2->rights_title)+1);
  if(pGFileListPref1->rights_title == NULL){gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}
  strcpy(pGFileListPref1->rights_title,pGFileListPref2->rights_title);
  pGFileListPref1->rights_pos = pGFileListPref2->rights_pos;
  pGFileListPref1->rights_width = pGFileListPref2->rights_width;
  pGFileListPref1->size_title = g_malloc(strlen(pGFileListPref2->size_title)+1);
  if(pGFileListPref1->size_title == NULL){gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}
  strcpy(pGFileListPref1->size_title,pGFileListPref2->size_title);
  pGFileListPref1->size_pos = pGFileListPref2->size_pos;
  pGFileListPref1->size_width = pGFileListPref2->size_width;
  pGFileListPref1->last_access_date_title = g_malloc(strlen(pGFileListPref2->last_access_date_title)+1);
  if(pGFileListPref1->last_access_date_title == NULL){gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}
  strcpy(pGFileListPref1->last_access_date_title,pGFileListPref2->last_access_date_title);
  pGFileListPref1->last_access_date_pos = pGFileListPref2->last_access_date_pos;
  pGFileListPref1->last_access_date_width = pGFileListPref2->last_access_date_width;
  pGFileListPref1->last_modif_date_title = g_malloc(strlen(pGFileListPref2->last_modif_date_title)+1);
  if(pGFileListPref1->last_modif_date_title == NULL){gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}
  strcpy(pGFileListPref1->last_modif_date_title,pGFileListPref2->last_modif_date_title);
  pGFileListPref1->last_modif_date_pos = pGFileListPref2->last_modif_date_pos;
  pGFileListPref1->last_modif_date_width = pGFileListPref2->last_modif_date_width;
  pGFileListPref1->last_status_change_date_title = g_malloc(strlen(pGFileListPref2->last_status_change_date_title)+1);
  if(pGFileListPref1->last_status_change_date_title == NULL){gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}
  strcpy(pGFileListPref1->last_status_change_date_title,pGFileListPref2->last_status_change_date_title);
  pGFileListPref1->last_status_change_date_pos = pGFileListPref2->last_status_change_date_pos;
  pGFileListPref1->last_status_change_date_width = pGFileListPref2->last_status_change_date_width;
  pGFileListPref1->ligne_height = pGFileListPref2->ligne_height;
  pGFileListPref1->min_width = pGFileListPref2->min_width;
  pGFileListPref1->min_height = pGFileListPref2->min_height;
  pGFileListPref1->statusbar = pGFileListPref2->statusbar;
  pGFileListPref1->sort_mode = pGFileListPref2->sort_mode;
  pGFileListPref1->pop_up_prop_menu = pGFileListPref2->pop_up_prop_menu;
  pGFileListPref1->owner_list = pGFileListPref2->owner_list;
  pGFileListPref1->group_list = pGFileListPref2->group_list;
  pGFileListPref1->file_type_list = pGFileListPref2->file_type_list;  
}  

/*********************/
/* set the sort mode */
void gfilelist_set_sort(GFileList *pGFileList, guint8 sort_mode)
{
  switch(sort_mode)
  {
    case SORT_NONE           :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_none_func); break;    
    case SORT_BY_NAME        :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_name_func); break;    
    case SORT_BY_OWNER       :  
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_owner_func); break;    
    case SORT_BY_GROUP       :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_group_func); break;    
    case SORT_BY_RIGHTS      :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_rights_func); break;
    case SORT_BY_SIZE        :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_size_func); break;    
    case SORT_BY_ACCESS_DATE :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_access_date_func); break;        
    case SORT_BY_MODIF_DATE  :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_modif_date_func); break;    
    case SORT_BY_CHANGE_DATE   :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_change_date_func); break;    

    case SORT_NONE | SORT_INVERT_ORDER           :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_none_inv_func); break;    
    case SORT_BY_NAME | SORT_INVERT_ORDER        :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_name_inv_func); break;    
    case SORT_BY_OWNER | SORT_INVERT_ORDER       :  
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_owner_inv_func); break;    
    case SORT_BY_GROUP | SORT_INVERT_ORDER       :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_group_inv_func); break;    
    case SORT_BY_RIGHTS | SORT_INVERT_ORDER      :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_rights_inv_func); break;
    case SORT_BY_SIZE | SORT_INVERT_ORDER        :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_size_inv_func); break;    
    case SORT_BY_ACCESS_DATE | SORT_INVERT_ORDER :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_access_date_inv_func); break;        
    case SORT_BY_MODIF_DATE | SORT_INVERT_ORDER  :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_modif_date_inv_func); break;    
    case SORT_BY_CHANGE_DATE | SORT_INVERT_ORDER   :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_change_date_inv_func); break;

    case SORT_NONE | SORT_DIR_FIRST          :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_none_dir_first_func); break;    
    case SORT_BY_NAME | SORT_DIR_FIRST        :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_name_dir_first_func); break;    
    case SORT_BY_OWNER | SORT_DIR_FIRST       :  
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_owner_dir_first_func); break;    
    case SORT_BY_GROUP | SORT_DIR_FIRST       :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_group_dir_first_func); break;    
    case SORT_BY_RIGHTS | SORT_DIR_FIRST      :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_rights_dir_first_func); break;
    case SORT_BY_SIZE | SORT_DIR_FIRST        :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_size_dir_first_func); break;    
    case SORT_BY_ACCESS_DATE | SORT_DIR_FIRST :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_access_date_dir_first_func); break;        
    case SORT_BY_MODIF_DATE | SORT_DIR_FIRST  :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_modif_date_dir_first_func); break;    
    case SORT_BY_CHANGE_DATE | SORT_DIR_FIRST   :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_change_date_dir_first_func); break;    

    case SORT_NONE | SORT_INVERT_ORDER | SORT_DIR_FIRST           :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_none_inv_dir_first_func); break;    
    case SORT_BY_NAME | SORT_INVERT_ORDER | SORT_DIR_FIRST        :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_name_inv_dir_first_func); break;    
    case SORT_BY_OWNER | SORT_INVERT_ORDER | SORT_DIR_FIRST       :  
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_owner_inv_dir_first_func); break;    
    case SORT_BY_GROUP | SORT_INVERT_ORDER | SORT_DIR_FIRST       :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_group_inv_dir_first_func); break;    
    case SORT_BY_RIGHTS | SORT_INVERT_ORDER | SORT_DIR_FIRST      :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_rights_inv_dir_first_func); break;
    case SORT_BY_SIZE | SORT_INVERT_ORDER | SORT_DIR_FIRST        :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_size_inv_dir_first_func); break;    
    case SORT_BY_ACCESS_DATE | SORT_INVERT_ORDER | SORT_DIR_FIRST :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_access_date_inv_dir_first_func); break;        
    case SORT_BY_MODIF_DATE | SORT_INVERT_ORDER | SORT_DIR_FIRST  :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_modif_date_inv_dir_first_func); break;    
    case SORT_BY_CHANGE_DATE | SORT_INVERT_ORDER | SORT_DIR_FIRST   :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_change_date_inv_dir_first_func); break;	    
  }
  pGFileList->Preference.sort_mode = sort_mode;
  gtk_clist_sort(GTK_CLIST(pGFileList->Clist));
}
/*********************/

/****************************************************************/
/* get the sort mode used (usefull because users can changed is */
/* by clicking on the titles)                                   */
guint8 gfilelist_get_sort(GFileList *pGFileList)
{
  return(pGFileList->Preference.sort_mode);
}
/****************************************************************/

/**************************************************************/
/* Function called eached time a titles of the list is cliked */
static void title_cb(GtkCList *clist,gint column,GFileList *pGFileList)
{ guint8 sort_mode, short_last_mode;

  short_last_mode = pGFileList->Preference.sort_mode;

  /* what ever happend, keep the dir first choice */
  if(pGFileList->Preference.sort_mode & SORT_DIR_FIRST){
    sort_mode = SORT_DIR_FIRST;
    short_last_mode -= SORT_DIR_FIRST;
  } else
    sort_mode = 0;
  
  if(short_last_mode & SORT_INVERT_ORDER)
    short_last_mode -= SORT_INVERT_ORDER;
  

  if((guint8)column == pGFileList->Preference.name_pos)
  { /*g_print("name sort\n");*/
    if((short_last_mode == SORT_BY_NAME) &&
       !(pGFileList->Preference.sort_mode &  SORT_INVERT_ORDER))
      gfilelist_set_sort(pGFileList, sort_mode | SORT_INVERT_ORDER | SORT_BY_NAME);
    else      
      gfilelist_set_sort(pGFileList, sort_mode | SORT_BY_NAME);
  } else
  if((guint8)column == pGFileList->Preference.owner_pos)
  { /*g_print("owner sort\n");*/
    if((short_last_mode == SORT_BY_OWNER) &&
       !(pGFileList->Preference.sort_mode &  SORT_INVERT_ORDER))
      gfilelist_set_sort(pGFileList, sort_mode | SORT_INVERT_ORDER | SORT_BY_OWNER);
    else      
      gfilelist_set_sort(pGFileList, sort_mode | SORT_BY_OWNER);
  } else
  if((guint8)column == pGFileList->Preference.group_pos)
  { /*g_print("group sort\n");*/
    if((short_last_mode == SORT_BY_GROUP) &&
       !(pGFileList->Preference.sort_mode &  SORT_INVERT_ORDER))
      gfilelist_set_sort(pGFileList, sort_mode | SORT_INVERT_ORDER | SORT_BY_GROUP);
    else      
      gfilelist_set_sort(pGFileList, sort_mode | SORT_BY_GROUP);
  } else
  if((guint8)column == pGFileList->Preference.rights_pos)
  { /*g_print("rights sort\n");*/
    if((short_last_mode == SORT_BY_RIGHTS) &&
       !(pGFileList->Preference.sort_mode &  SORT_INVERT_ORDER))
      gfilelist_set_sort(pGFileList, sort_mode | SORT_INVERT_ORDER | SORT_BY_RIGHTS);
    else      
      gfilelist_set_sort(pGFileList, sort_mode | SORT_BY_RIGHTS);
  } else
  if((guint8)column == pGFileList->Preference.size_pos)
  { /*g_print("size sort\n");*/
    if((short_last_mode == SORT_BY_SIZE) &&
       !(pGFileList->Preference.sort_mode &  SORT_INVERT_ORDER))
      gfilelist_set_sort(pGFileList, sort_mode | SORT_INVERT_ORDER | SORT_BY_SIZE);
    else      
      gfilelist_set_sort(pGFileList, sort_mode | SORT_BY_SIZE);
  } else
  if((guint8)column == pGFileList->Preference.last_access_date_pos)
  { /*g_print("last access date sort\n");*/
    if((short_last_mode == SORT_BY_ACCESS_DATE) &&
       !(pGFileList->Preference.sort_mode &  SORT_INVERT_ORDER))
      gfilelist_set_sort(pGFileList, sort_mode | SORT_INVERT_ORDER | SORT_BY_ACCESS_DATE);
    else      
      gfilelist_set_sort(pGFileList, sort_mode | SORT_BY_ACCESS_DATE);
  } else
  if((guint8)column == pGFileList->Preference.last_modif_date_pos)
  { /*g_print("last modif date sort\n");*/
    if((short_last_mode == SORT_BY_MODIF_DATE) &&
       !(pGFileList->Preference.sort_mode &  SORT_INVERT_ORDER))
      gfilelist_set_sort(pGFileList, sort_mode | SORT_INVERT_ORDER | SORT_BY_MODIF_DATE);
    else      
      gfilelist_set_sort(pGFileList, sort_mode | SORT_BY_MODIF_DATE);
  } else
  if((guint8)column == pGFileList->Preference.last_status_change_date_pos)
  { /*g_print("last status change date sort\n");*/
    if((short_last_mode == SORT_BY_CHANGE_DATE) &&
       !(pGFileList->Preference.sort_mode &  SORT_INVERT_ORDER))
      gfilelist_set_sort(pGFileList, sort_mode | SORT_INVERT_ORDER | SORT_BY_CHANGE_DATE);
    else      
      gfilelist_set_sort(pGFileList, sort_mode | SORT_BY_CHANGE_DATE);
  }

}
/**************************************************************/


/* use as a parameter given by gfilelist_new to gfilelist_init */
GFileListPref current_GFileListPref;

/*************************************************************/
static void gfilelist_init (GFileList *pGFileList)
{
  gint  nb_col = 0;
  gchar	*titles[9];

  GtkWidget *menu_items; /* for popup menu items */
  GtkWidget *scrolled_widget;
  GtkWidget *title_button; /* for replacing clist titles by buttons */
  GtkWidget *event_box;

  /* take parameters in account */
  gfilelistpref_copy(&(pGFileList->Preference),&current_GFileListPref);  
  /* free strings in the Preference */
  g_free(current_GFileListPref.name_title);
  g_free(current_GFileListPref.owner_title);
  g_free(current_GFileListPref.group_title); 
  g_free(current_GFileListPref.rights_title);  
  g_free(current_GFileListPref.size_title);
  g_free(current_GFileListPref.last_access_date_title); 
  g_free(current_GFileListPref.last_modif_date_title);
  g_free(current_GFileListPref.last_status_change_date_title);

  pGFileList->current_path = NULL;

  if(pGFileList->Preference.name_pos<255) 
  { titles[pGFileList->Preference.name_pos] = pGFileList->Preference.name_title; nb_col++; }
  if(pGFileList->Preference.owner_pos<255) 
  { titles[pGFileList->Preference.owner_pos] = pGFileList->Preference.owner_title; nb_col++; }
  if(pGFileList->Preference.group_pos<255) 
  { titles[pGFileList->Preference.group_pos] = pGFileList->Preference.group_title; nb_col++; }
  if(pGFileList->Preference.rights_pos<255) 
  { titles[pGFileList->Preference.rights_pos] = pGFileList->Preference.rights_title; nb_col++; }
  if(pGFileList->Preference.size_pos<255) 
  { titles[pGFileList->Preference.size_pos] = pGFileList->Preference.size_title; nb_col++; }
  if(pGFileList->Preference.last_access_date_pos<255) 
  { titles[pGFileList->Preference.last_access_date_pos] = pGFileList->Preference.last_access_date_title; nb_col++; }
  if(pGFileList->Preference.last_modif_date_pos<255) 
  { titles[pGFileList->Preference.last_modif_date_pos] = pGFileList->Preference.last_modif_date_title; nb_col++; }
  if(pGFileList->Preference.last_status_change_date_pos<255) 
  { titles[pGFileList->Preference.last_status_change_date_pos] = pGFileList->Preference.last_status_change_date_title; nb_col++; }

  /* The last one is empty */
  titles[nb_col++] = " ";

  /* Create the widget */ 
  pGFileList->Clist = gtk_clist_new_with_titles(nb_col ,titles);

  /* chose the selection mode (extended) */
  gtk_clist_set_selection_mode(GTK_CLIST(pGFileList->Clist), GTK_SELECTION_EXTENDED);
  /* ligne height */
  gtk_clist_set_row_height(GTK_CLIST(pGFileList->Clist), pGFileList->Preference.ligne_height);
  /* justify right the file size */
  if(pGFileList->Preference.size_pos<255)
    gtk_clist_set_column_justification(GTK_CLIST (pGFileList->Clist),
		 pGFileList->Preference.size_pos,GTK_JUSTIFY_RIGHT);

  gtk_signal_connect (GTK_OBJECT (pGFileList), 
		      "destroy",
		      (GtkSignalFunc) gfilelist_free,
		      pGFileList);

  gtk_signal_connect (GTK_OBJECT (pGFileList->Clist), 
		      "select_row",
		      (GtkSignalFunc) gfilelist_row_select_cb,
		      pGFileList);

  gtk_signal_connect (GTK_OBJECT (pGFileList->Clist), 
		      "unselect_row",
		      (GtkSignalFunc) gfilelist_row_unselect_cb,
		      pGFileList);

  /* Connect all titles */
  gtk_signal_connect(GTK_OBJECT(pGFileList->Clist), "click_column",
    GTK_SIGNAL_FUNC (title_cb), pGFileList);

  /* choose the order function */
  gfilelist_set_sort(pGFileList,pGFileList->Preference.sort_mode);

  /* hide the titles if we don't want to see them */
  if(pGFileList->Preference.hide_title)
    gtk_clist_column_titles_hide (GTK_CLIST(pGFileList->Clist));

  /* set the colonne size */
  if(pGFileList->Preference.name_pos<255) 
    gtk_clist_set_column_width(GTK_CLIST(pGFileList->Clist),pGFileList->Preference.name_pos,
                               pGFileList->Preference.name_width);
  if(pGFileList->Preference.owner_pos<255) 
    gtk_clist_set_column_width(GTK_CLIST(pGFileList->Clist),pGFileList->Preference.owner_pos,
                               pGFileList->Preference.owner_width);
  if(pGFileList->Preference.group_pos<255) 
    gtk_clist_set_column_width(GTK_CLIST(pGFileList->Clist),pGFileList->Preference.group_pos,
                               pGFileList->Preference.group_width);
  if(pGFileList->Preference.rights_pos<255) 
    gtk_clist_set_column_width(GTK_CLIST(pGFileList->Clist),pGFileList->Preference.rights_pos,
                               pGFileList->Preference.rights_width);
  if(pGFileList->Preference.size_pos<255) 
    gtk_clist_set_column_width(GTK_CLIST(pGFileList->Clist),pGFileList->Preference.size_pos,
                               pGFileList->Preference.size_width);
  if(pGFileList->Preference.last_access_date_pos<255) 
    gtk_clist_set_column_width(GTK_CLIST(pGFileList->Clist),pGFileList->Preference.last_access_date_pos,
                               pGFileList->Preference.last_access_date_width);
  if(pGFileList->Preference.last_modif_date_pos<255) 
    gtk_clist_set_column_width(GTK_CLIST(pGFileList->Clist),pGFileList->Preference.last_modif_date_pos,
                               pGFileList->Preference.last_modif_date_width);
  if(pGFileList->Preference.last_status_change_date_pos<255) 
    gtk_clist_set_column_width(GTK_CLIST(pGFileList->Clist),pGFileList->Preference.last_status_change_date_pos,
                               pGFileList->Preference.last_status_change_date_width);

  /* set the minimum size of the list ___________*/
  gtk_widget_set_usize(GTK_WIDGET(pGFileList->Clist),pGFileList->Preference.min_width,
                       pGFileList->Preference.min_height);

  /* define the scroll properties */
  scrolled_widget = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(scrolled_widget),
    GTK_POLICY_AUTOMATIC,GTK_POLICY_ALWAYS);
  gtk_widget_show(scrolled_widget);
  gtk_container_add(GTK_CONTAINER(scrolled_widget), pGFileList->Clist);
  
  gtk_box_pack_start (GTK_BOX (pGFileList), scrolled_widget, TRUE, TRUE, 0);

  /* create a statusbar if we want one */
  if(pGFileList->Preference.statusbar == TRUE)
  {
    pGFileList->Statusbar = gtk_statusbar_new();
    gtk_box_pack_start(GTK_BOX(pGFileList),pGFileList->Statusbar,FALSE,TRUE,0);
    gtk_widget_show(pGFileList->Statusbar);    
  } else { pGFileList->Statusbar = NULL; }

  /* create a popup menu if wanted */
  /* for the "propertie" popup menu */
  if(pGFileList->Preference.pop_up_prop_menu == TRUE)
  {
    gtk_signal_connect (GTK_OBJECT(pGFileList->Clist), "event",
	 GTK_SIGNAL_FUNC (gfilelist_prop_menu_cb), pGFileList);

     pGFileList->PopUpMenu = gtk_menu_new();
     
     menu_items = gtk_menu_item_new_with_label("Open");
     gtk_widget_show(menu_items);
     gtk_menu_append(GTK_MENU (pGFileList->PopUpMenu), menu_items);
     /* insert a line */
     menu_items = gtk_menu_item_new();
     gtk_widget_show(menu_items);
     gtk_menu_append(GTK_MENU (pGFileList->PopUpMenu), menu_items);
     /* insert a line */
     menu_items = gtk_menu_item_new();
     gtk_widget_show(menu_items);
     gtk_menu_append(GTK_MENU (pGFileList->PopUpMenu), menu_items);
		
     menu_items = gtk_menu_item_new_with_label("Properties...");
    gtk_signal_connect (GTK_OBJECT(menu_items), "activate",
	 GTK_SIGNAL_FUNC (prop_menu_properties_cb), pGFileList);
     gtk_widget_show(menu_items);
     gtk_menu_append(GTK_MENU (pGFileList->PopUpMenu), menu_items);
  }

  /* display the clist widget */
  gtk_widget_show (pGFileList->Clist);  
}
/**********************************************************/

/*************************************************************/
GtkWidget* gfilelist_new (GFileListPref pGFileListPref)
{
  gfilelistpref_copy(&current_GFileListPref,&pGFileListPref);
  return GTK_WIDGET ( gtk_type_new( gfilelist_get_type() ) );
}
/*************************************************************/

/*************************************************/
/* Free a GFileList				 */
void gfilelist_free(GFileList *pGFileList)
{
  /* remove all the thing that are in the clist */
  gfilelist_remove_all(pGFileList);
  /* if we have a popup menu, free it */
  if(pGFileList->Preference.pop_up_prop_menu)
    gtk_widget_destroy(GTK_WIDGET(pGFileList->PopUpMenu));
  /* free strings in the Prefence */
  g_free(pGFileList->Preference.name_title);
  g_free(pGFileList->Preference.owner_title);
  g_free(pGFileList->Preference.group_title); 
  g_free(pGFileList->Preference.rights_title);  
  g_free(pGFileList->Preference.size_title);
  g_free(pGFileList->Preference.last_access_date_title); 
  g_free(pGFileList->Preference.last_modif_date_title);
  g_free(pGFileList->Preference.last_status_change_date_title);
  /* free all the other widget */
  gtk_widget_destroy(GTK_WIDGET(pGFileList));
}
/*************************************************/

/**************************************************************/
/* create a string which represent the date. This string MUST */
/* BE FREE after use.                                         */
static gchar *gfilelist_date_2_string(GFileList *pGFileList,guint64 date)
{ struct tm *vtm;
  gchar *date_string;

  date_string = g_malloc(11);
  if(date_string == NULL){gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}
  vtm = localtime((time_t *)(&date));
  /*European notation*/
  sprintf(date_string,"%2u/%2u/%4u",vtm->tm_mday,vtm->tm_mon+1,vtm->tm_year+1900);
  if(vtm->tm_mday<10)date_string[0] = '0';
  if(vtm->tm_mon+1<10)date_string[3] = '0';
  return(date_string);
}
/**************************************************************/

/******************************************************/
/* For local use only. This procedure create a string */
/* which correspond to the rights.		      */
/* string must be an array of 10 char		      */
static void gfile_list_rights_2_string(guint16 rights,guint16 mode,gchar string[])
{
  /* owner rights */
  if(rights & 0x100) string[0]='r';
  else string[0]='-';
  if(rights & 0x80) string[1]='w';
  else string[1]='-';
  if(rights & 0x40) string[2]='x';
  else string[2]='-';
  if(mode & FILE_DATA_IS_UID_BIT)
    if(string[2] != '-')
      string[2]='s';
    else
      string[2]='S';

  /* owner group rights */
  if(rights & 0x20) string[3]='r';
  else string[3]='-';
  if(rights & 0x10) string[4]='w';
  else string[4]='-';
  if(rights & 0x8) string[5]='x';
  else string[5]='-';
  if(mode & FILE_DATA_IS_GID_BIT) 
    if(string[5] != '-')
      string[5]='s';    
    else
      string[5]='S';

  /* all the world */
  if(rights & 0x4) string[6]='r';
  else string[6]='-';
  if(rights & 0x2) string[7]='w';
  else string[7]='-';
  if(rights & 0x1) string[8]='x';
  else string[8]='-';
  if(mode & FILE_DATA_IS_STICKY_BIT) 
    if(string[8] != '-')
      string[8]='t';    
    else
      string[8]='T';
  string[9]=0; /* end the string */
}
/******************************************************/

static gboolean status_bar_refresh = TRUE;

/***********************************************************/
/* add an element to the list 				   */
/* THIS MUST BE THE ONLY WAY TO ADD AN ELEMENT TO THE LIST */
static gboolean select_added_file = FALSE; /* usefull for gfilelist_modify_file */
 			             /* if TRUE, the new added file is selected */
void gfilelist_add_file(GFileList *pGFileList,FileData *pFileData)
{
  gchar *info[9];
  gchar temp_uid[10];    /* for the owner  */
  gchar temp_gid[10];    /* for the group  */
  gchar temp_rights[10]; /* for the rights */
  gchar temp_size[15];   /* for the size   */
  gchar *empty="";
  own *own;
  grp *grp;
  gint row;
  
  info[0]=empty; info[1]=empty; info[2]=empty; info[3]=empty;
  info[4]=empty; info[5]=empty; info[6]=empty; info[7]=empty;
  info[8]=empty;

  if(pGFileList->Preference.name_pos<255) /* display the file name */
    info[pGFileList->Preference.name_pos] = pFileData->name;

  if(pGFileList->Preference.owner_pos<255) /* display the owner */
  {
    if(pFileData->mode & FILE_DATA_NO_EXTENDED_INFO)
       info[pGFileList->Preference.owner_pos] = empty;
    else 
    {
      own = owner_find_by_nb(pGFileList->Preference.owner_list,pFileData->uid);
      if(own != NULL) info[pGFileList->Preference.owner_pos] = own->name;
      else {
        sprintf(temp_uid,"%u",pFileData->uid);
	info[pGFileList->Preference.owner_pos] = temp_uid;
      }
    }  
  }

  if(pGFileList->Preference.group_pos<255)   /* display the group */
  {
    if(pFileData->mode & FILE_DATA_NO_EXTENDED_INFO)
      info[pGFileList->Preference.group_pos] = empty;
    else
    {
      grp = group_find_by_nb(pGFileList->Preference.group_list,pFileData->gid);
      if(grp != NULL) info[pGFileList->Preference.group_pos] = grp->name;
      else{
        sprintf(temp_gid,"%u",pFileData->gid);
	info[pGFileList->Preference.group_pos] = temp_gid;
      }
    }
  }
 
  if(pGFileList->Preference.rights_pos<255)  /* display the rights */
  {
    if(pFileData->mode & FILE_DATA_NO_EXTENDED_INFO)
      info[pGFileList->Preference.rights_pos] = empty;
    else 
    {
      gfile_list_rights_2_string(pFileData->rights,pFileData->mode,temp_rights);
      info[pGFileList->Preference.rights_pos] = temp_rights;
    }  
  }

  if(pGFileList->Preference.size_pos<255)    /* display the size */
  {
    if(pFileData->mode & FILE_DATA_NO_EXTENDED_INFO)
      info[pGFileList->Preference.size_pos] = empty;
    else
    {
      sprintf(temp_size,"%u",pFileData->size);
      info[pGFileList->Preference.size_pos] = temp_size;
    }
  }

  if(pGFileList->Preference.last_access_date_pos<255)    /* display the create date */
  {
    if(pFileData->mode & FILE_DATA_NO_EXTENDED_INFO)
      info[pGFileList->Preference.last_access_date_pos] = empty;
    else
      info[pGFileList->Preference.last_access_date_pos] = 
        gfilelist_date_2_string(pGFileList,pFileData->last_access_date);
  }    
  
  if(pGFileList->Preference.last_modif_date_pos<255)    /* display the modif date */
  {
    if(pFileData->mode & FILE_DATA_NO_EXTENDED_INFO)
      info[pGFileList->Preference.last_modif_date_pos] = empty;
    else
      info[pGFileList->Preference.last_modif_date_pos] = 
        gfilelist_date_2_string(pGFileList,pFileData->last_modif_date);
  }

  if(pGFileList->Preference.last_status_change_date_pos<255)    /* display the last access date */
  {
    if(pFileData->mode & FILE_DATA_NO_EXTENDED_INFO)
      info[pGFileList->Preference.last_status_change_date_pos] = empty;
    else
      info[pGFileList->Preference.last_status_change_date_pos] = 
        gfilelist_date_2_string(pGFileList,pFileData->last_status_change_date);
  }

  /* add to the list */
  row = gtk_clist_append(GTK_CLIST(pGFileList->Clist), info);
  /* take care of the icon */
  gtk_clist_set_pixtext(GTK_CLIST(pGFileList->Clist), row, 0, info[0],
                        5,pFileData->type->pixmap,
			pFileData->type->mask);
  /* attach the source information */
  gtk_clist_set_row_data(GTK_CLIST(pGFileList->Clist),row,
    		         pFileData);
			 
  /* select the added row if wanted */			 
  if(select_added_file)
    gtk_clist_select_row(GTK_CLIST(pGFileList->Clist),row,0);
			  
  /* free allocated strings */
  if(!(pFileData->mode & FILE_DATA_NO_EXTENDED_INFO))
  {
    if(pGFileList->Preference.last_access_date_pos<255)
      g_free(info[pGFileList->Preference.last_access_date_pos]);

    if(pGFileList->Preference.last_modif_date_pos<255)
      g_free(info[pGFileList->Preference.last_modif_date_pos]);
            
    if(pGFileList->Preference.last_status_change_date_pos<255)
      g_free(info[pGFileList->Preference.last_status_change_date_pos]); 
  }  

  /* auto_sort is not used because sort is made with attached datas
     and when an item is inserted, datas are not attached */
  gtk_clist_sort(GTK_CLIST(pGFileList->Clist));

  /* if wanted, we fill the statusbar. This can be avoid
     by setting status_bar_refresh = FALSE when adding many files */
  if((pGFileList->Statusbar) && (status_bar_refresh == TRUE))
  { gfilelist_fill_status(pGFileList); }

}
/***********************************************************/

/*********************************************/
/* Add a file to the list by giving its name */
/* the file MUST be in the current viewed    */
/* directory. ex name = file1		     */
void gfilelist_add_file_by_name(GFileList *pGFileList,gchar *name)
{ FileData *vFileData;

  /* create a new FileData with info of the file "name"*/
  vFileData = file_data_new_from_file(pGFileList->current_path,name,
    pGFileList->Preference.file_type_list,pGFileList->Preference.owner_list);
  
  /* add to the list */
  gfilelist_add_file(pGFileList,vFileData);
}
/*********************************************/

/*******************************************************/
/* Del a file to the list by giving its full path name */
/* ex name = /dir1/subdir1/file1		       */
void gfilelist_remove_file_by_full_name(GFileList *pGFileList,gchar *full_name)
{ gchar *tmp_string;
  guint current_pos,copy_pos;
  gint  index_row;
  FileData *vFileData;
  
  if(full_name[0]!='/') return;

  tmp_string = (gchar *)g_malloc(strlen(full_name)+1);
  if(tmp_string == NULL){gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}

  current_pos = 0;
  copy_pos = 0;
  while(full_name[current_pos] != 0 )
  { 
    tmp_string[current_pos] = full_name[current_pos];
    if(full_name[current_pos++] == '/') copy_pos = current_pos-1;
  }
  tmp_string[copy_pos+1] = 0; /* tmp_string = path */

  /* if it is in the current dir, we remove the file */
  if( !strcmp(tmp_string,pGFileList->current_path) )
  {
    for(index_row=0;index_row<((GtkCList *)(pGFileList->Clist))->rows;index_row++)
    {
      vFileData=(FileData *)gtk_clist_get_row_data(
        GTK_CLIST(pGFileList->Clist),index_row);
      /* if the content are the same */
      if((vFileData != NULL ) && (!strcmp(vFileData->name,&full_name[copy_pos+1])))
      { 
        gfilelist_remove_file(pGFileList,vFileData); 
	g_free(tmp_string);
        return;
      }	
    }
  }

  g_free(tmp_string);
}
/*******************************************************/

/*******************************************************/
/* Add a file to the list by giving its full path name */
/* ex name = /dir1/subdir1/file1		       */
void gfilelist_add_file_by_full_name(GFileList *pGFileList,gchar *full_name)
{ gchar *tmp_string;
  guint current_pos,copy_pos;
  
  if(full_name[0]!='/') return;

  tmp_string = (gchar *)g_malloc(strlen(full_name)+1);
  if(tmp_string == NULL){gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}

  current_pos = 0;
  copy_pos = 0;
  while(full_name[current_pos] != 0 )
  { 
    tmp_string[current_pos] = full_name[current_pos];
    if(full_name[current_pos++] == '/') copy_pos = current_pos-1;
  }
  tmp_string[copy_pos+1] = 0; /* tmp_string = path */

  /*g_print("path : %s, name : %s.\n",tmp_string,&full_name[copy_pos+1]);*/

  /* if it is in the current dir, we add the file */
  if( !strcmp(tmp_string,pGFileList->current_path) )
    gfilelist_add_file_by_name(pGFileList,&full_name[copy_pos+1]);

  g_free(tmp_string);
}
/*******************************************************/

/***********************************************************/
/* add an element to the list 				   */
/* add it only if the element is in the current directory  */
/* a copy of pFileData will be used so we are free to do   */
/* what you want with pFileData after this called          */
void gfilelist_add_file_with_check(GFileList *pGFileList,FileData *pFileData)
{ FileData *vFileData;

  if((pGFileList == NULL) || (pFileData == NULL)) return;
  /* if it is not in the current path we have nothing to do */
  if( strcmp(pFileData->path,pGFileList->current_path) ) return;
  vFileData = file_data_copy(pFileData);
  gfilelist_add_file(pGFileList,vFileData);
}
/***********************************************************/

/***********************************************************/
/* return the row corresponding to pFileData. pFileData is */
/* find by comparing info (not pointer). If not found,     */
/* return -1.                                              */
gint gfilelist_find_row(GFileList *pGFileList,FileData *pFileData)
{ gint index_row;
  FileData *vFileData;

  for(index_row=0;index_row<((GtkCList *)(pGFileList->Clist))->rows;index_row++)
  {
    vFileData=(FileData *)gtk_clist_get_row_data(
      GTK_CLIST(pGFileList->Clist),index_row);

    if((vFileData != NULL ) && (file_data_is_equal(pFileData,
      vFileData)))
      return(index_row); 
  }
  return(-1);
}
/***********************************************************/

/************************************************************/
/* remove a file in the list using the FileData of the file */
/* The diff with gfilelist_remove_file is that we check if  */
/* the file really belong to the list.                      */
static gboolean unselect_remove_file = FALSE; /* usefull for gfilelist_modify_file */
   		                /* if TRUE, the file is unselected before removing */
static gboolean remove_file_was_selected = FALSE; /* let know if the file was */
                    /* selected or not (only if unselect_remove_file == TRUE) */
static gboolean remove_file_was_in_focus = FALSE;
void gfilelist_remove_file_with_check(GFileList *pGFileList,FileData *pFileData)
{
  gint index_row;
  FileData *vFileData;
  
  if((pGFileList == NULL) || (pFileData == NULL)) return;
  /* if it is not in the current path we have nothing to do */
  if( strcmp(pFileData->path,pGFileList->current_path) ) return;
  /* we make a check on data not pointer */
  /* find the corresponding file in our list */
  for(index_row=0;index_row<((GtkCList *)(pGFileList->Clist))->rows;index_row++)
  {
    vFileData=(FileData *)gtk_clist_get_row_data(
      GTK_CLIST(pGFileList->Clist),index_row);
    /* if the content are the same */ 
    if((vFileData != NULL ) && (file_data_is_equal(pFileData,
      vFileData)))
    { /* unselect file if wanted */
      if(unselect_remove_file == TRUE)
      {
        remove_file_was_selected = gfilelist_list_is_row_selected(pGFileList,index_row);
        remove_file_was_in_focus = 
	  (GTK_CLIST(pGFileList->Clist)->focus_row == index_row);
        if(remove_file_was_selected == TRUE)
	{
          gtk_clist_unselect_row(GTK_CLIST(pGFileList->Clist),index_row,0);
	}  
      }	
      gfilelist_remove_file(pGFileList,vFileData); 
      return; 
    }
  }
}
/******************************************************************/

/******************************************************************/
/* modify an entry in the list. The file corresponding to SrcFile */
/* is replaced by DstFile. SrcFile is found using informations    */
/* test (not pinter). A copy of DstFile is keeped.                */
void gfilelist_modify_file(GFileList *pGFileList,FileData *SrcFile,
  FileData *DstFile)
{ gboolean is_selected = FALSE;
  FileData *vFileData,*vNewFocusFileData;
  gint row_pos,last_focus_row;
  gboolean something_selected = TRUE;

  /* if the file we want to modify is not in the current path, exit */
  if(strcmp(pGFileList->current_path,SrcFile->path)) return;

  /* freeze the list to avoid flicker */
  gtk_clist_freeze(GTK_CLIST(pGFileList->Clist));

  /* check if there is something selected in this list */
  if(GTK_CLIST(pGFileList->Clist)->selection == NULL) 
    something_selected = FALSE;
  /* if something was selected, keep the focus row item */
  if(something_selected)
  {
    last_focus_row = GTK_CLIST(pGFileList->Clist)->focus_row;
    vNewFocusFileData = gtk_clist_get_row_data(GTK_CLIST(pGFileList->Clist),last_focus_row);
  }
  
  row_pos = gfilelist_find_row(pGFileList,SrcFile);
  if(row_pos == -1) 
    g_print("GFILELIST, gfilelist_modify_file : SrcFile not found. name :%s%s.\n",
      SrcFile->path,SrcFile->name);
  if(something_selected)
  { is_selected = gfilelist_list_row_is_selected(pGFileList, row_pos);
    if(is_selected)
      select_added_file = TRUE;
  }    
  vFileData = file_data_copy(DstFile);    
  gfilelist_add_file(pGFileList,vFileData);
  select_added_file = FALSE;

  /* if focus row was the modified row, new focus row is the new modified row */
  if(something_selected)
    if(row_pos == last_focus_row)
      vNewFocusFileData = vFileData;

  /* if the modified row was selected, unselect it before removing */
  if(something_selected)
    if(is_selected)
      unselect_remove_file = TRUE;
  gfilelist_remove_file_with_check(pGFileList,SrcFile);
  unselect_remove_file = FALSE;

  /* if something was selected, set the new focus row */
  if(something_selected)
    GTK_CLIST(pGFileList->Clist)->focus_row = 
      gtk_clist_find_row_from_data(GTK_CLIST(pGFileList->Clist),vNewFocusFileData);

  /* unfreeze the list */
  gtk_clist_thaw(GTK_CLIST(pGFileList->Clist));
} 
/******************************************************************/

/**************************************************************/
/* remove an element to the list 			      */
/* THIS MUST BE THE ONLY WAY TO REMOVE AN ELEMENT TO THE LIST */
/* except the total destruction of the list                   */
void gfilelist_remove_file(GFileList *pGFileList,FileData *pFileData)
{ gint row_to_remove;
  /* if bad parameters we have nothing to do */
  if( (pGFileList==NULL) || (pFileData==NULL) ) return;
  /* try to find the row which correspond */
  row_to_remove = gtk_clist_find_row_from_data(GTK_CLIST(pGFileList->Clist),
                                               pFileData);
  if(row_to_remove != -1) /* if it existe */
    gtk_clist_remove(GTK_CLIST(pGFileList->Clist),row_to_remove);
  file_data_free(pFileData);
}
/**************************************************************/

/******************************************/
/* free all the file display in the clist */
void gfilelist_remove_all(GFileList *pGFileList)
{ gint index_row;
  FileData *vFileData;
  /* remove all the attached datas to the rows */
  for(index_row=0;index_row<(GTK_CLIST(pGFileList->Clist))->rows;index_row++)
  {
    vFileData=(FileData *)gtk_clist_get_row_data(
      GTK_CLIST(pGFileList->Clist),index_row);
    if(vFileData != NULL ) file_data_free(vFileData);
  }
  /* remove the all row from the clist */
  gtk_clist_clear(GTK_CLIST(pGFileList->Clist));
}
/******************************************/

/***********************************/
/* Fill the statusbar of GFileList */
void gfilelist_fill_status(GFileList *pGFileList)
{ GList *select_row;
  gint index_row;
  FileData *vFileData;
  guint64 total_size = 0;
  guint32 nb_file = 0;
  gchar *to_display;
  /* if statusbar is not wanted, we have nothing to do here */
  if(pGFileList->Preference.statusbar == FALSE) return;
  
  select_row = (GList *)(GTK_CLIST(pGFileList->Clist)->selection);
  if(select_row == NULL) /* nothing selected, we display the total */
  { /* size of the file (directory are not counted). */
INFO_ON_ALL :
    for(index_row=0;index_row<((GtkCList *)(pGFileList->Clist))->rows;index_row++)
    {
      vFileData=(FileData *)gtk_clist_get_row_data(
        GTK_CLIST(pGFileList->Clist),index_row);
      /*g_print("fich : %s\n",vGFileListData->name);*/
      if(vFileData != NULL )
        if( !(vFileData->mode & FILE_DATA_IS_DIRECTORY) &&
	    !(vFileData->mode & FILE_DATA_NO_EXTENDED_INFO) )
	{ nb_file++; total_size += vFileData->size; }
    }
    to_display = g_malloc(27+20+10);
    sprintf(to_display," files : %u   total : %u bytes",nb_file,total_size);
  } else {
    /* 2 cases. 1 selected = we display more infos on the file */
    /* else we display the size of all the selected files. */
    if(select_row->next == NULL) 
    {
      vFileData=(FileData *)gtk_clist_get_row_data(
        GTK_CLIST(pGFileList->Clist),(gint)select_row->data);
      if(vFileData == NULL) return; /* not possible*/
      /* special : if we are on . or .. we display info on all files */
      if(!strcmp(vFileData->name,".") ) goto INFO_ON_ALL;
      if(!strcmp(vFileData->name,"..") ) goto INFO_ON_ALL;

      if( vFileData->mode & FILE_DATA_NO_EXTENDED_INFO ){
        to_display = g_malloc(16+strlen(vFileData->name));
        sprintf(to_display," broken file : %s",vFileData->name);
      }else
      if( vFileData->mode & FILE_DATA_IS_LINK ){
        to_display = g_malloc(12+strlen(vFileData->link_to_file));
        sprintf(to_display," link to : %s",vFileData->link_to_file);
      }else
      if( vFileData->mode & FILE_DATA_IS_DIRECTORY ){
        to_display = g_malloc(14+strlen(vFileData->name));
        sprintf(to_display," directory : %s",vFileData->name);
      }else
      if( vFileData->mode & FILE_DATA_IS_SOCKET ){
        to_display = g_malloc(11+strlen(vFileData->name));      
        sprintf(to_display," socket : %s",vFileData->name);
      }else
      if( vFileData->mode & FILE_DATA_IS_BLOCK_DEV ){
        to_display = g_malloc(17+strlen(vFileData->name));      
        sprintf(to_display," block device : %s",vFileData->name);
      }else
      if( vFileData->mode & FILE_DATA_IS_CHAR_DEV ){
        to_display = g_malloc(20+strlen(vFileData->name));      
        sprintf(to_display," character device : %s",vFileData->name);
      }else
      if( vFileData->mode & FILE_DATA_IS_FIFO ){
        to_display = g_malloc(9+strlen(vFileData->name));      
        sprintf(to_display," fifo : %s",vFileData->name);
      }else{
        to_display = g_malloc(25+20+strlen(vFileData->name));      
        sprintf(to_display," file : %s   size : %u bytes",
	  vFileData->name,vFileData->size);
      }

    } else { /* we display the size of all the selected files. */
      while(select_row)
      {
        vFileData=(FileData *)gtk_clist_get_row_data(
          GTK_CLIST(pGFileList->Clist),(gint)select_row->data);
        /*g_print("fich : %s\n",vFileData->name);*/
        if(vFileData != NULL )
          if( !(vFileData->mode & FILE_DATA_IS_DIRECTORY) &&
	      !(vFileData->mode & FILE_DATA_NO_EXTENDED_INFO) )
  	  { nb_file++; total_size += vFileData->size; }
        select_row = select_row->next;
      }
      to_display = g_malloc(27+20+10);
      sprintf(to_display," files : %u   total : %u bytes",nb_file,total_size);
    } 
  }
  if ((GTK_STATUSBAR(pGFileList->Statusbar))->messages) 
     gtk_statusbar_pop (GTK_STATUSBAR(pGFileList->Statusbar), 1);
  gtk_statusbar_push (GTK_STATUSBAR(pGFileList->Statusbar), 1,to_display);
  
  g_free(to_display);
}
/***********************************/

/*************************************************************/
/* Fill a GFileList with the content of the directory 'name' */
/* 'name' is the full path of the directory. DON'T ENDED by  */
/* a '/' (except for '/' directory)                          */
/* Return : TRUE = ok, FALSE = the directory doesn't exit    */
gboolean gfilelist_open_dir(GFileList *pGFileList,gchar *name)
{
  struct dirent **namelist;
  gint   nb_entry;
  gint   entry_pos = 0;

  /* open the content of a dir */  
  nb_entry = scandir(name, &namelist, 0, 0);
  if (nb_entry < 0) return(FALSE); /* THE DIRECTORY DOESN'T EXIST */
  /* Like we exit, the GFileList stay as we recieved it */

  /* say that there is no row selected in the clist */
  pGFileList->Clist_last_selected_row = -1;

  /* Freeze the clist to avoid flicker */
  gtk_clist_freeze(GTK_CLIST(pGFileList->Clist));
  /* Free the list before fill it again */
  gfilelist_remove_all(pGFileList);
  /* Free the current_path */
  if(pGFileList->current_path) g_free(pGFileList->current_path);
  /* write the new one */
  if( (pGFileList->current_path = g_malloc(strlen(name)+2)) == NULL )
  { gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}
  strcpy(pGFileList->current_path,name);
  /* if we are not at the root add a '/' */
  if(!(name[0]=='/' && name[1]==0))
    strcat(pGFileList->current_path,"/");

  /* devalidate the status bar refresh to speed a bit */
  status_bar_refresh = FALSE;

  while(entry_pos < nb_entry)
  {gfilelist_add_file_by_name(pGFileList,namelist[entry_pos++]->d_name);}

/*  g_print("All file added\n");*/

  status_bar_refresh = TRUE;

  g_free(namelist);

  /* Unfreeze the clist */
  gtk_clist_thaw(GTK_CLIST(pGFileList->Clist));

  /* if wanted, we fill the statusbar */
  if(pGFileList->Statusbar)
  { gfilelist_fill_status(pGFileList); }

  /* reset the last selected row */
  pGFileList->Clist_last_selected_row = -1;

  return(TRUE);
}
/*************************************************************/

/* transform a string to go back to the parent directory */
static void down_dir(gchar *path,gchar *pathdest)
{
  guint lastchar = 0;
  guint16 compt;

  if(strcmp(path,"/")) /* if we are not a the root ("/") */
  {
     /* find */
     for(lastchar=0;(lastchar<2000)&& (path[lastchar]);lastchar++);
     lastchar-=2;
     for(;lastchar && (path[lastchar]!='/');lastchar--);
     /* make a copy of the good part */
     for(compt=0;compt<=lastchar;compt++) pathdest[compt]=path[compt];
     pathdest[lastchar+1]=0;
  } else { pathdest[0]='/'; pathdest[1]=0; }
}

/**********************************************************************/
/* procedure called each time there is something selected in the list */
static void gfilelist_row_select_cb(GtkCList *clist,gint row,gint column, GdkEvent
*event,GFileList *pGFileList)
{
  gchar *tempbuf,*temp_dir_name;
  FileData *vFileData;

  /* do something only if the user has made a double click */
  if(row==pGFileList->Clist_last_selected_row)
  {
    /* look at the attached datas */
    vFileData = (FileData *)gtk_clist_get_row_data(
                GTK_CLIST(pGFileList->Clist),row);
				      
    /* if it is a directory */
    if( vFileData->mode & FILE_DATA_IS_DIRECTORY )
    {
       /*g_print("Directory path : %s, Name : %s\n",vFileData->path,
       						  vFileData->name);*/
       if( !strcmp(vFileData->name,".") ) /* reload the current directory */
       {
         tempbuf = g_malloc(strlen(vFileData->path)+1);
	 if(tempbuf == NULL){gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}
	 strcpy(tempbuf,vFileData->path);
	 /* if we are not at the root, suppress the last '/' */
	 if(tempbuf[1] != 0)
	   tempbuf[strlen(tempbuf)-1] = 0;
	 gfilelist_open_dir(pGFileList,tempbuf);
	 g_free(tempbuf);
       } else
       if( !strcmp(vFileData->name,"..") )  /* go to the parent directory */
       {
         tempbuf = g_malloc(strlen(vFileData->path)+1);
	 if(tempbuf == NULL){gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}
	 /* find the parent directory name */
	 down_dir(vFileData->path,tempbuf);
	 /* if we are not at the root, suppress the last '/' */
	 if(tempbuf[1] != 0)
	   tempbuf[strlen(tempbuf)-1] = 0;	 
	 gfilelist_open_dir(pGFileList,tempbuf);
	 g_free(tempbuf);

	 /* emit the corresponding signal */
	 gtk_signal_emit(GTK_OBJECT(pGFileList),gfilelist_signals[DOWN_DIR_SIGNAL]);

       } else {	/* we go to the selected directory */
       
	 /* make a copy of the name of the directory we try to enter */
	 temp_dir_name = g_malloc(strlen(vFileData->name)+1);
	 if(temp_dir_name == NULL){gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}
	 strcpy(temp_dir_name,vFileData->name);

         tempbuf = g_malloc(strlen(vFileData->path)+
	                    strlen(vFileData->name)+1);
	 if(tempbuf == NULL){gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}
	 strcpy(tempbuf,vFileData->path);
	 strcat(tempbuf,vFileData->name);

	 if(gfilelist_open_dir(pGFileList,tempbuf))
	  /* let the others known that we go to the upper directory
	     temp_dir_name if we successful open if */
	   gtk_signal_emit(GTK_OBJECT(pGFileList),gfilelist_signals[UP_DIR_SIGNAL],
	                   temp_dir_name);
	 g_free(tempbuf);	 
	 g_free(temp_dir_name);
       }
    } else { /* this is not a directory */
      if(vFileData->type->program[0]) /* if type defined*/
      {
        g_print("program : %s\n",vFileData->type->program);
	tempbuf = g_malloc(strlen(vFileData->type->program)+
	                   5+strlen(vFileData->path)+
			   strlen(vFileData->name));
	if(tempbuf == NULL){gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}
	strcpy(tempbuf,vFileData->type->program);
	strcat(tempbuf," \"");
	strcat(tempbuf,vFileData->path);
	strcat(tempbuf,vFileData->name);
	strcat(tempbuf,"\"&");
	system(tempbuf);
	g_free(tempbuf);
      } else if(vFileData->mode & FILE_DATA_IS_ALLOW_TO_EXEC) {
	tempbuf = g_malloc(3+strlen(vFileData->path)+
			   strlen(vFileData->name));
	if(tempbuf == NULL){gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}
	strcpy(tempbuf," \"");
	strcat(tempbuf,vFileData->path);
	strcat(tempbuf,vFileData->name);
	strcat(tempbuf,"\"&");
	system(tempbuf);
	g_free(tempbuf);      
      }
    }
  } else {
    pGFileList->Clist_last_selected_row = row;
        /* update the statusbar*/
    if(pGFileList->Preference.statusbar)
      gfilelist_fill_status(pGFileList);
  }
}
/**********************************************************************/

/****************************************/
/* called each time a row is unselected */
static void gfilelist_row_unselect_cb(GtkCList *clist,gint row,gint column, GdkEvent
*event,GFileList *pGFileList)
{
  if(pGFileList->Preference.statusbar)
     gfilelist_fill_status(pGFileList);
  /* more than one selected => double click no more valid */
  pGFileList->Clist_last_selected_row = -1;   
}
/****************************************/

/************************************************/
/* Set a new FileType. The caller is responsible*/
/* for freeing the last one.                    */
void gfilelist_set_filetype(GFileList *pGFileList,FileType *pFileType)
{ gint index_row;
  FileData *vFileData;
  file_type *fich_type;
  gchar *text,*text_copy;
  GdkPixmap *pixmap;
  GdkBitmap *mask;
  guint8 spacing;
  
  pGFileList->Preference.file_type_list = pFileType;
  
  /* Freeze the clist to avoid flicker */
  gtk_clist_freeze(GTK_CLIST(pGFileList->Clist));

  for(index_row=0;index_row<((GtkCList *)(pGFileList->Clist))->rows;index_row++)
  {
    vFileData=(FileData *)gtk_clist_get_row_data(
      GTK_CLIST(pGFileList->Clist),index_row);
    fich_type = file_data_find_type(pGFileList->Preference.file_type_list,vFileData);
    if(fich_type == NULL){gfile_error(FILE_TYPE_ERROR); exit(FILE_TYPE_ERROR);}

    /*g_print("change type for : %s\n",vFileData->name);*/
    gtk_clist_get_pixtext(GTK_CLIST(pGFileList->Clist),index_row,0,&text,
      			  &spacing,&pixmap,&mask);

    /*g_print("row 0 text : %s\n",text);*/
    text_copy = g_malloc(strlen(text)+1);
    if(text_copy == NULL){gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}
    strcpy(text_copy,text);
    gtk_clist_set_pixtext(GTK_CLIST(pGFileList->Clist),index_row, 0, text_copy,
                          5,vFileData->type->pixmap,
			  vFileData->type->mask);
    g_free(text_copy);
  }

  /* Unfreeze the clist */
  gtk_clist_thaw(GTK_CLIST(pGFileList->Clist));
}
/************************************************/

/*******************************************************/
/* create a list of all the selected file in the clist */
/* when we are called.				       */
/* return : NULL if impossible.			       */
FileDataList *gfilelist_list_selected(GFileList *pGFileList)
{ GList *select_row;
  FileData *vFileData;
  FileDataList *vFileDataList;
  
  vFileDataList = file_data_list_new();
  if(vFileDataList == NULL) return(NULL);

  /* place select_row on the selected line list */
  select_row = (GList *)(GTK_CLIST(pGFileList->Clist)->selection);
  
  while(select_row)
  { /* get the data attach to the selected line */
    vFileData=(FileData *)gtk_clist_get_row_data(
      GTK_CLIST(pGFileList->Clist),(gint)select_row->data);
    /* add the selected data to our list */
    file_data_list_append(vFileDataList, vFileData);
    /* go to the next selected line */
    select_row = g_list_next(select_row);
  }
  return(vFileDataList);  
}
/*******************************************************/

/*******************************************************/
/* Asked if a row is seleted or not                    */
/* return : TRUE if row is selected, FALSE else.       */
gboolean gfilelist_list_is_row_selected(GFileList *pGFileList, gint row)
{ GList *select_row;
  
  /* place select_row on the selected line list */
  select_row = (GList *)(GTK_CLIST(pGFileList->Clist)->selection);
  
  while((select_row) && ((gint)select_row->data != row))
  {
    /* go to the next selected line */
    select_row = g_list_next(select_row);
  }
  return(select_row != NULL);  
}
/*******************************************************/

/************************************************************/
/* use this if you want to unselect every thing in the list */
void gfilelist_unselect_all(GFileList *pGFileList)
{
  gtk_signal_emit_by_name(GTK_OBJECT(pGFileList->Clist),"unselect_all");
  /* reset the last selected row */
  pGFileList->Clist_last_selected_row = -1;
}
/************************************************************/

/***************************************************************/
/* Return : TRUE if row_nb is in selection, else Return FALSE. */
gboolean gfilelist_list_row_is_selected(GFileList *pGFileList, gint row_nb)
{ GList *select_row;

  /* place select_row on the selected line list */
  select_row = (GList *)(GTK_CLIST(pGFileList->Clist)->selection);
  
  while((select_row != NULL) && (row_nb != (gint)select_row->data))
  {
    /* go to the next selected line */
    select_row = g_list_next(select_row);
  }
  if(select_row != NULL) return(TRUE); else return(FALSE);
}
/***************************************************************/

static void prop_file_changed_cb(GtkWidget *Widget,FileData *SrcFile,
  FileData *DstFile,GFileList *pGFileList)
{
  gtk_signal_emit(GTK_OBJECT(pGFileList),
    gfilelist_signals[FILE_MODE_CHANGED_SIGNAL],SrcFile,DstFile);
}  

static void prop_menu_properties_cb(GtkWidget *Widget,GFileList *pGFileList)
{ FileDataList *vFileDataList;
  GtkWidget *propreties;

  vFileDataList = gfilelist_list_selected(pGFileList);
  
  propreties = file_properties_new(vFileDataList,pGFileList->Preference.file_type_list,
    pGFileList->Preference.owner_list);
  gtk_widget_show(propreties);

  gtk_signal_connect(GTK_OBJECT(propreties),"file_mode_changed",
    GTK_SIGNAL_FUNC(prop_file_changed_cb),pGFileList);

  file_data_list_free(vFileDataList);
}

/****************************************/
/* for the popup menu                   */
/* ________ UNDER DEVELOPEMENT _________*/
static gint gfilelist_prop_menu_cb(GtkCList *clist, GdkEvent *event,GFileList
                                   *pGFileList)
{ gint row,column;
    
  if(event->type == GDK_BUTTON_PRESS) {
    GdkEventButton *bevent = (GdkEventButton *) event; 
    /* if it is the third button */
    if(bevent->button == 3)
    { /*g_print("properties called.\n");*/

      row = bevent->x; column = bevent->y;		      
      gtk_clist_get_selection_info(GTK_CLIST(pGFileList->Clist),
        row,column,&row,&column);
      /*g_print("row : %d, col : %d\n",row,column);*/

      /* do something only if the user right click on a selected file */
      if(gfilelist_list_row_is_selected(pGFileList,row) == TRUE)
      {

        gtk_menu_popup(GTK_MENU(pGFileList->PopUpMenu), NULL, NULL, NULL, NULL,
                       bevent->button, bevent->time);
      } 
      /* Tell calling code that we have handled this event; the buck
      * stops here. */	      
      return TRUE;      
    }  	      
    /* Tell calling code that we have not handled this event; pass it on. */
    return FALSE;
  }
  /* Tell calling code that we have not handled this event; pass it on. */
  return FALSE;
}
/****************************************/
