/****************************************/
/*  GFileList.c headers         	*/
/*					*/
/*  Description : Manage the file list  */
/*		  widget		*/
/*					*/
/*  Written by : Daniel Lacroix	        */
/*  GFile is under the GPL2 licence     */
/****************************************/

#ifndef __GFILELIST_H__
#define __GFILELIST_H__

/* to define a new widget */
#define GFILELIST(obj)          GTK_CHECK_CAST (obj, gfilelist_get_type(),GFileList)
#define GFILELIST_CLASS(klass)  GTK_CHECK_CLASS_CAST (klass, gfilelist_get_type(), GFileListClass)
#define IS_GFILELIST(obj)       GTK_CHECK_TYPE (obj, gfilelist_get_type ())

typedef struct _GFileList       GFileList;
typedef struct _GFileListClass  GFileListClass;
typedef struct _GFileListPref   GFileListPref;

/* The structure which define the configuration of the list */
#define SORT_NONE           0
#define SORT_BY_NAME        1
#define SORT_BY_OWNER       2  
#define SORT_BY_GROUP       3
#define SORT_BY_RIGHTS      4
#define SORT_BY_SIZE        5
#define SORT_BY_ACCESS_DATE 6
#define SORT_BY_MODIF_DATE  7
#define SORT_BY_CHANGE_DATE 8
#define SORT_INVERT_ORDER   1<<6 
#define SORT_DIR_FIRST      1<<7
struct _GFileListPref 
{
  gboolean hide_title;		/* TRUE = hide the clist title   */
  gchar    *name_title;
  guint8   name_pos;		/* pos = 255 if we don't want it */
  gint     name_width;
  gchar    *owner_title;
  guint8   owner_pos;
  gint     owner_width;
  gchar    *group_title;
  guint8   group_pos;
  gint     group_width;
  gchar    *rights_title;
  guint8   rights_pos;
  gint     rights_width;
  gchar    *size_title;
  guint8   size_pos;
  gint     size_width;
  gchar    *last_access_date_title; /* creation date of the file */
  guint8   last_access_date_pos;
  gint     last_access_date_width;
  gchar    *last_modif_date_title; /* date of the last modification of the file */
  guint8   last_modif_date_pos;
  gint     last_modif_date_width;
  gchar    *last_status_change_date_title;  /* date of the last access to the file */
  guint8   last_status_change_date_pos;
  gint     last_status_change_date_width;
  guint8   ligne_height;	 /* 18 is a good choice */
  gint     min_width;
  gint     min_height;
  gboolean statusbar;	/* true = we want it. false = don't want.        */

  guint8   sort_mode;	/* 0 = none. 1 = by name. 2 = owner. 3 = group.  */
  			/* 4 = rights. 5 = by size. 6 = create date.     */
			/* 7 = modif date. 8 = last date. +128 for having*/
			/* directory first. +64 for inverting the order. */
  gboolean pop_up_prop_menu; /* TRUE = allow the menu where we can found */
  			     /* the file properties ( right click ) */

  struct _Owner *owner_list;
  struct _Group *group_list;
  /* the GFileList use the file type list given NOT A COPY of it so
     you can't modify the file type list you have given. */
  FileType  *file_type_list;
};

/* the structure of a GFileList */
struct _GFileList
{
  GtkVBox 	Main;
  GtkWidget	*Clist;
  GtkWidget	*PopUpMenu;
  gint		Clist_last_selected_row;
  GtkWidget	*Statusbar;
  /* the directory we are one ended by a '/' */
  gchar		*current_path;

  GFileListPref Preference;
};

struct _GFileListClass
{
   GtkVBoxClass parent_class;

   /* Add here all the signal we want */
   void (* up_dir)   (GFileList *pGFileList);
   void (* down_dir) (GFileList *pGFileList,gchar *directory);
   
  /* signal called when the mode of a file as been changed */
  void (* file_mode_changed) (GFileList *pGFileList,FileData *SrcFile,
    FileData *DstFile);   
};

/* The default GFileList preference */
extern GFileListPref DefaultGFileListPref;

/* copy the content of  pGFileListPref2 in pGFileListPref1 */
void gfilelistpref_copy(GFileListPref *pGFileListPref1,
  GFileListPref *pGFileListPref2);
  
guint gfilelist_get_type ();
/* set the sort mode */
void gfilelist_set_sort(GFileList *pGFileList, guint8 sort_mode);
/* get the sort mode used (usefull because users can changed is
   by clicking on the titles) */
guint8 gfilelist_get_sort(GFileList *pGFileList);
/* Create a new GFileList widget */
GtkWidget* gfilelist_new (GFileListPref pGFileListPref);
/* Free a GFileList */
void gfilelist_free(GFileList *pGFileList);
/* add an element to the list */
void gfilelist_add_file(GFileList *pGFileList,FileData *pFileData);
/* add an element to the list 				   */
/* add it only if the element is in the current directory  */
/* a copy of pFileData will be used so we are free to do   */
/* what you want with pFileData after this called          */
void gfilelist_add_file_with_check(GFileList *pGFileList,FileData *pFileData);
/* Add a file to the list by giving its name */
/* the file MUST be in the current viewed    */
/* directory. ex name = file1		     */
void gfilelist_add_file_by_name(GFileList *pGFileList,gchar *name);
/* Add a file to the list by giving its full path name */
/* ex name = /dir1/subdir1/file1		       */
void gfilelist_add_file_by_full_name(GFileList *pGFileList,gchar *full_name);
/* Del a file to the list by giving its full path name */
/* ex name = /dir1/subdir1/file1		       */
void gfilelist_remove_file_by_full_name(GFileList *pGFileList,gchar *full_name);
/* remove a file in the list using the FileData of the file */
/* The diff with gfilelist_remove_file is that we check if  */
/* the file really belong to the list using content of      */
/* FileData, not place where FileData pointed.              */
void gfilelist_remove_file_with_check(GFileList *pGFileList,FileData *pFileData);
/* remove an element to the list 			      */
/* THIS MUST BE THE ONLY WAY TO REMOVE AN ELEMENT TO THE LIST */
/* except the total destruction of the list                   */
void gfilelist_remove_file(GFileList *pGFileList,FileData *pFileData);
/* free all the file display in the clist */
void gfilelist_remove_all(GFileList *pGFileList);
/* Fill the statusbar of GFileList */
void gfilelist_fill_status(GFileList *pGFileList);
/* Fill a GFileList with the content of the directory 'name' */
/* 'name' is the full path of the directory. DON'T ENDED by  */
/* a '/' (except for '/' directory)                          */
/* Return : TRUE = ok, FALSE = the directory doesn't exit    */
gboolean gfilelist_open_dir(GFileList *pGFileList,gchar *name);
/* Set a new FileType. The caller is responsible*/
/* for freeing the last one.                    */
void gfilelist_set_filetype(GFileList *pGFileList,FileType *pFileType);
/* create a list of all the selected file in the clist */
/* when we are called.				       */
/* return : NULL if impossible.			       */
FileDataList *gfilelist_list_selected(GFileList *pGFileList);
/* use this if you want to unselect every thing in the list */
void gfilelist_unselect_all(GFileList *pGFileList);
/* Asked if a row is seleted or not                    */
/* return : TRUE if row is selected, FALSE else.       */
gboolean gfilelist_list_is_row_selected(GFileList *pGFileList, gint row);
/* modify an entry in the list. The file corresponding to SrcFile */
/* is replaced by DstFile. SrcFile is found using informations    */
/* test (not pinter). A copy of DstFile is keeped.                */
void gfilelist_modify_file(GFileList *pGFileList,FileData *SrcFile,
  FileData *DstFile);
/* return the row corresponding to pFileData. pFileData is */
/* find by comparing info (not pointer). If not found,     */
/* return -1.                                              */
gint gfilelist_find_row(GFileList *pGFileList,FileData *pFileData);  

#endif /* __GFILELIST_H__ */
