/****************************************/
/*  GFileTree.c body    	       	*/
/*					*/
/*  Description : Manage the tree list  */
/*		  widget		*/
/*					*/
/*  Written by : Daniel Lacroix	        */
/*  GFile is under the GPL2 licence     */
/****************************************/

#include "GFile.h"

guint gfiletree_get_type ();
GtkWidget* gfiletree_new (GFileTreePref pGFileTreePref);
void gfiletree_free(GFileTree *pGFileTree);

void gfiletree_up(GFileTree *pGFileTree, gchar *directory);
void gfiletree_down(GFileTree *pGFileTree);

void gfiletree_update_filetype(GFileTree *pGFileTree,
  FileType *new_file_type_list);
gboolean gfiletree_add_dir_by_name(GFileTree *pGFileTree,gchar *directory);
void gfiletree_remove_dir_by_name(GFileTree *pGFileTree,gchar *directory);
void gfiletreepref_copy(GFileTreePref *pGFileTreePref1,
  GFileTreePref *pGFileTreePref2);

/* internal use only */
static void gfiletree_select_cb(GtkCTree *ctree_cb,GtkCTreeNode *row_cb,
                         gint column,GFileTree *pGFileTree);
static void gfiletree_expand_cb(GtkCTree *ctree_cb, GtkCTreeNode *node,
                         GFileTree *pGFileTree);
static gboolean gfiletree_load_path(GFileTree *pGFileTree,gchar *path,
                                  GtkCTreeNode *parent,guint8 nb_level);
static void string_marshal(GtkObject *object, GtkSignalFunc func, 
  gpointer func_data, GtkArg *args);
static void gfiletree_load_root_tree(GFileTree *pGFileTree);
static void gfiletree_select_before_cb(GtkCTree *ctree_cb,GtkCTreeNode *row_cb,
                         gint column,GFileTree *pGFileTree);
static void gfiletree_expand_before_cb(GtkCTree *ctree_cb, GtkCTreeNode *node,
                         GFileTree *pGFileTree);
static void gfiletree_update_filetype_part(GFileTree *pGFileTree,GtkCTreeNode *node);
static GtkCTreeNode *gfiletree_find_dir_at_node(GFileTree *pGFileTree,
                        GtkCTreeNode *parent,gchar *directory);
static void gfiletree_class_init(GFileTreeClass *class);
static void gfiletree_init (GFileTree *pGFileTree);

/* if set to false, the change signal is never emit */
gboolean allow_change_signal = TRUE;

/* ______________________ CORRECTED OVER HERE _________________ */

/* The default Preference */
GFileTreePref DefaultGFileTreePref =
{
  FALSE,        /* don't hide the title */
  "Directory",
  SORT_BY_NAME,	/* sort mode */
  NULL          /* file_type_list can't have a default value */
}; 

/*************************************************************/
guint gfiletree_get_type ()
{
  static guint gfiletree_type = 0;

  if (!gfiletree_type)
  {
    GtkTypeInfo gfiletree_info =
    {
       "GFileTree",
       sizeof (GFileTree),
       sizeof (GFileTreeClass),
       (GtkClassInitFunc)  gfiletree_class_init,
       (GtkObjectInitFunc) gfiletree_init,
       (GtkArgSetFunc) NULL,
       (GtkArgGetFunc) NULL
     };
     gfiletree_type = gtk_type_unique(gtk_vbox_get_type(), &gfiletree_info);
  }
  return gfiletree_type;
}
/*************************************************************/

/*************************************************************/
enum { CHANGE_DIR_SIGNAL, LAST_SIGNAL};
static gint gfiletree_signals[LAST_SIGNAL] = { 0 };

static void string_marshal(GtkObject *object, GtkSignalFunc func, gpointer func_data, GtkArg *args) {
    typedef void (*string_marshal_func)(GFileTree *, gchar *, gpointer);
    string_marshal_func rfunc = (string_marshal_func) func;
    (*rfunc) (GFILETREE(object),
	      GTK_VALUE_POINTER(args[0]), func_data);
}

static void gfiletree_class_init(GFileTreeClass *class)
{
/* here we create the new signals if we want */
  GtkObjectClass *object_class;
  object_class = (GtkObjectClass*) class;

  gfiletree_signals[CHANGE_DIR_SIGNAL] = gtk_signal_new ("change_dir",
            GTK_RUN_FIRST,object_class->type,
            GTK_SIGNAL_OFFSET (GFileTreeClass,change_dir),
            string_marshal, GTK_TYPE_NONE, 1,GTK_TYPE_POINTER);
	    
   gtk_object_class_add_signals (object_class, gfiletree_signals, LAST_SIGNAL);
   class->change_dir = NULL;
}
/*************************************************************/

/* copy the content of  pGFileTreePref2 in pGFileTreePref1 */
void gfiletreepref_copy(GFileTreePref *pGFileTreePref1,
  GFileTreePref *pGFileTreePref2)
{
  pGFileTreePref1->hide_title = pGFileTreePref2->hide_title;
  
  pGFileTreePref1->title = g_malloc(strlen(pGFileTreePref2->title)+1);
  if(pGFileTreePref1->title == NULL){gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);} 
  strcpy(pGFileTreePref1->title,pGFileTreePref2->title);
  
  pGFileTreePref1->sort_mode = pGFileTreePref2->sort_mode;
  pGFileTreePref1->file_type_list = pGFileTreePref2->file_type_list;
}

/*********************/
/* set the sort mode */
static void gfiletree_set_sort_internal(GFileTree *pGFileTree, guint8 sort_mode)
{
  switch(sort_mode)
  {
    case SORT_NONE           :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileTree->Ctree),
  	sort_none_func); break;    
    case SORT_BY_NAME        :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileTree->Ctree),
  	sort_by_name_func); break;    
    case SORT_BY_OWNER       :  
      gtk_clist_set_compare_func(GTK_CLIST(pGFileTree->Ctree),
  	sort_by_owner_func); break;    
    case SORT_BY_GROUP       :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileTree->Ctree),
  	sort_by_group_func); break;    
    case SORT_BY_RIGHTS      :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileTree->Ctree),
  	sort_by_rights_func); break;
    case SORT_BY_SIZE        :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileTree->Ctree),
  	sort_by_size_func); break;    
    case SORT_BY_ACCESS_DATE :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileTree->Ctree),
  	sort_by_access_date_func); break;        
    case SORT_BY_MODIF_DATE  :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileTree->Ctree),
  	sort_by_modif_date_func); break;    
    case SORT_BY_CHANGE_DATE   :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileTree->Ctree),
  	sort_by_change_date_func); break;    

    case SORT_NONE | SORT_INVERT_ORDER           :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileTree->Ctree),
  	sort_none_inv_func); break;    
    case SORT_BY_NAME | SORT_INVERT_ORDER        :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileTree->Ctree),
  	sort_by_name_inv_func); break;    
    case SORT_BY_OWNER | SORT_INVERT_ORDER       :  
      gtk_clist_set_compare_func(GTK_CLIST(pGFileTree->Ctree),
  	sort_by_owner_inv_func); break;    
    case SORT_BY_GROUP | SORT_INVERT_ORDER       :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileTree->Ctree),
  	sort_by_group_inv_func); break;    
    case SORT_BY_RIGHTS | SORT_INVERT_ORDER      :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileTree->Ctree),
  	sort_by_rights_inv_func); break;
    case SORT_BY_SIZE | SORT_INVERT_ORDER        :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileTree->Ctree),
  	sort_by_size_inv_func); break;    
    case SORT_BY_ACCESS_DATE | SORT_INVERT_ORDER :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileTree->Ctree),
  	sort_by_access_date_inv_func); break;        
    case SORT_BY_MODIF_DATE | SORT_INVERT_ORDER  :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileTree->Ctree),
  	sort_by_modif_date_inv_func); break;    
    case SORT_BY_CHANGE_DATE | SORT_INVERT_ORDER   :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileTree->Ctree),
  	sort_by_change_date_inv_func); break;

    case SORT_NONE | SORT_DIR_FIRST          :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileTree->Ctree),
  	sort_none_dir_first_func); break;    
    case SORT_BY_NAME | SORT_DIR_FIRST        :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileTree->Ctree),
  	sort_by_name_dir_first_func); break;    
    case SORT_BY_OWNER | SORT_DIR_FIRST       :  
      gtk_clist_set_compare_func(GTK_CLIST(pGFileTree->Ctree),
  	sort_by_owner_dir_first_func); break;    
    case SORT_BY_GROUP | SORT_DIR_FIRST       :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileTree->Ctree),
  	sort_by_group_dir_first_func); break;    
    case SORT_BY_RIGHTS | SORT_DIR_FIRST      :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileTree->Ctree),
  	sort_by_rights_dir_first_func); break;
    case SORT_BY_SIZE | SORT_DIR_FIRST        :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileTree->Ctree),
  	sort_by_size_dir_first_func); break;    
    case SORT_BY_ACCESS_DATE | SORT_DIR_FIRST :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileTree->Ctree),
  	sort_by_access_date_dir_first_func); break;        
    case SORT_BY_MODIF_DATE | SORT_DIR_FIRST  :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileTree->Ctree),
  	sort_by_modif_date_dir_first_func); break;    
    case SORT_BY_CHANGE_DATE | SORT_DIR_FIRST   :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileTree->Ctree),
  	sort_by_change_date_dir_first_func); break;    

    case SORT_NONE | SORT_INVERT_ORDER | SORT_DIR_FIRST           :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileTree->Ctree),
  	sort_none_inv_dir_first_func); break;    
    case SORT_BY_NAME | SORT_INVERT_ORDER | SORT_DIR_FIRST        :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileTree->Ctree),
  	sort_by_name_inv_dir_first_func); break;    
    case SORT_BY_OWNER | SORT_INVERT_ORDER | SORT_DIR_FIRST       :  
      gtk_clist_set_compare_func(GTK_CLIST(pGFileTree->Ctree),
  	sort_by_owner_inv_dir_first_func); break;    
    case SORT_BY_GROUP | SORT_INVERT_ORDER | SORT_DIR_FIRST       :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileTree->Ctree),
  	sort_by_group_inv_dir_first_func); break;    
    case SORT_BY_RIGHTS | SORT_INVERT_ORDER | SORT_DIR_FIRST      :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileTree->Ctree),
  	sort_by_rights_inv_dir_first_func); break;
    case SORT_BY_SIZE | SORT_INVERT_ORDER | SORT_DIR_FIRST        :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileTree->Ctree),
  	sort_by_size_inv_dir_first_func); break;    
    case SORT_BY_ACCESS_DATE | SORT_INVERT_ORDER | SORT_DIR_FIRST :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileTree->Ctree),
  	sort_by_access_date_inv_dir_first_func); break;        
    case SORT_BY_MODIF_DATE | SORT_INVERT_ORDER | SORT_DIR_FIRST  :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileTree->Ctree),
  	sort_by_modif_date_inv_dir_first_func); break;    
    case SORT_BY_CHANGE_DATE | SORT_INVERT_ORDER | SORT_DIR_FIRST   :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileTree->Ctree),
  	sort_by_change_date_inv_dir_first_func); break;	    
  }
}

void gfiletree_set_sort(GFileTree *pGFileTree, guint8 sort_mode)
{
  gfiletree_set_sort_internal(pGFileTree, sort_mode);
  
  pGFileTree->Preference.sort_mode = sort_mode;
  gtk_ctree_sort_recursive(GTK_CTREE(pGFileTree->Ctree),pGFileTree->root_node);
}
/*********************/

/* use as a parameter given by gfiletree_new to gfiletree_init */
GFileTreePref current_GFileTreePref;

/*************************************************************/
static void gfiletree_init (GFileTree *pGFileTree)
{
  GtkWidget *scrolled_widget;
  gchar     *title[1];
  
  /* take parameters in account */
  gfiletreepref_copy((GFileTreePref *)(&(pGFileTree->Preference)),
    &current_GFileTreePref);
  g_free(current_GFileTreePref.title);  

  title[0] = pGFileTree->Preference.title;
  pGFileTree->Ctree = GTK_CTREE(gtk_ctree_new_with_titles(1,0,title));

  if(pGFileTree->Preference.hide_title == TRUE) 
    gtk_clist_column_titles_hide(GTK_CLIST(pGFileTree->Ctree));
    
  gtk_ctree_set_line_style (pGFileTree->Ctree, GTK_CTREE_LINES_DOTTED);
/*  gtk_ctree_set_reorderable (pGFileTree->Ctree, TRUE);*/
  gtk_widget_show(GTK_WIDGET(pGFileTree->Ctree));

//  gtk_clist_set_selection_mode(GTK_CLIST(pGFileTree->Ctree),/*GTK_SELECTION_SINGLE);*/
//  GTK_SELECTION_BROWSE);

  /* when the widget is destroy, called gfiletree_free */
  gtk_signal_connect(GTK_OBJECT(pGFileTree),"destroy",
		     GTK_SIGNAL_FUNC(gfiletree_free),pGFileTree);

/*  gtk_signal_connect(GTK_OBJECT(pGFileTree->Ctree),"tree_select_row",
		     GTK_SIGNAL_FUNC(gfiletree_select_before_cb),pGFileTree);

  gtk_signal_connect(GTK_OBJECT(pGFileTree->Ctree),"tree_expand",
		     GTK_SIGNAL_FUNC (gfiletree_expand_before_cb),pGFileTree);
*/
  gtk_signal_connect_after(GTK_OBJECT(pGFileTree->Ctree),"tree_select_row",
			   GTK_SIGNAL_FUNC(gfiletree_select_cb),pGFileTree);

  gtk_signal_connect_after(GTK_OBJECT(pGFileTree->Ctree),"tree_expand",
			   GTK_SIGNAL_FUNC (gfiletree_expand_cb),pGFileTree);

  /* define the scroll properties */
  scrolled_widget = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(scrolled_widget),
    GTK_POLICY_AUTOMATIC,GTK_POLICY_ALWAYS);
  gtk_widget_show(scrolled_widget);
  gtk_container_add(GTK_CONTAINER(scrolled_widget),GTK_WIDGET(pGFileTree->Ctree));

  gtk_box_pack_start(GTK_BOX(pGFileTree),GTK_WIDGET(scrolled_widget),
                     TRUE,TRUE,0);

  gfiletree_set_sort_internal(pGFileTree, pGFileTree->Preference.sort_mode);
  /* load the root content */		     
  pGFileTree->current_path = NULL;
  gfiletree_load_root_tree(pGFileTree);
}
/*************************************************************/

/*************************************************************/
GtkWidget* gfiletree_new (GFileTreePref pGFileTreePref)
{
  gfiletreepref_copy(&current_GFileTreePref,&pGFileTreePref);
  return GTK_WIDGET ( gtk_type_new( gfiletree_get_type() ) );
}
/*************************************************************/

/*************************************************/
/* Free a GFileTree				 */
static void free_node_data(GtkCTree *ctree,GtkCTreeNode *node,gpointer data)
{ FileData *vFileData;
 
  vFileData = gtk_ctree_node_get_row_data(ctree,node);
  /*g_print("directory : >%s%s<\n",vFileData->path,vFileData->name);*/
  g_free(vFileData);
}

void gfiletree_free(GFileTree *pGFileTree)
{
  g_print("A GFileTree will be destroy\n");
  /* free the title */
  g_free(pGFileTree->Preference.title);
  /* free all the ctree */
  gtk_ctree_post_recursive(GTK_CTREE(pGFileTree->Ctree), 
    GTK_CTREE_NODE(pGFileTree->root_node),GTK_CTREE_FUNC(free_node_data),NULL);
  gtk_clist_clear (GTK_CLIST(pGFileTree->Ctree));
  gtk_widget_destroy(GTK_WIDGET(pGFileTree));
}
/*************************************************/

/***************************************************/
/* Attention : DON'T ENDED by path by a '/'        */
/*               (except for '/' directory)        */
/* Return : FALSE if this directory doesn't exist  */
/*          or parent are bad. Else : TRUE         */
static gboolean gfiletree_load_path(GFileTree *pGFileTree,gchar *path,
                             GtkCTreeNode *parent,guint8 nb_level)
{
  struct dirent **namelist;
  gint   nb_entry;
  gint   entry_pos = 0;

  struct stat *danstat2;
  gchar *currentfich;
  gchar *currentpath;
  file_type *tmptype, *tmptype2;
  gchar *text[1];
  GtkCTreeNode *sibling;

  FileData *vFileData;

  /* a protection */
  if((path == NULL) || (parent == NULL) || (pGFileTree==NULL)) return(FALSE);

 /* open the content of a dir */  
  nb_entry = scandir(path, &namelist, 0, 0);
  if (nb_entry < 0) return(FALSE); /* THE DIRECTORY DOESN'T EXIST */
  /* Like we exit, the GFileTree stay as we recieved it */

  /* alloc some mem for detail on file  */
  danstat2 = (struct stat *)g_malloc(sizeof(struct stat));
  if(danstat2 == NULL) {gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}

  /* in the tree, no difference are made between link to dir or dir */
  tmptype = file_type_find_type(pGFileTree->Preference.file_type_list,
    NORMAL_DIRECTORY,FALSE);
  if(tmptype == NULL) {gfile_error(FILE_TYPE_ERROR); exit(FILE_TYPE_ERROR);}
  tmptype2 = file_type_find_type(pGFileTree->Preference.file_type_list,
    NORMAL_OPEN_DIRECTORY,FALSE);
  if(tmptype2 == NULL){gfile_error(FILE_TYPE_ERROR); exit(FILE_TYPE_ERROR);}

  currentpath = g_malloc(strlen(path)+2);
  if(currentpath == NULL) {gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}
  strcpy(currentpath,path);
  /* end by a '/' if path different than "/" */
  if (!((path[0]=='/') && (path[1]==0))) strcat(currentpath,"/");

  while(entry_pos < nb_entry)
  {
    currentfich = g_malloc(strlen(path)+strlen(namelist[entry_pos]->d_name)+3);
    if(currentfich == NULL){gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}
    strcpy(currentfich,path);
    /* if we are not at the root add a '/' */
    if(!(path[0]=='/' && path[1]==0))
      strcat(currentfich,"/");
    strcat(currentfich,namelist[entry_pos]->d_name);

    if(!stat(currentfich,danstat2)) /* if not a broken file */
      if(( (danstat2->st_mode >> 12) == 4) && 
          (strcmp(namelist[entry_pos]->d_name,"..") &&
	   strcmp(namelist[entry_pos]->d_name,".")))
      { /* it is a directory and not . or .. */

        vFileData = file_data_new(currentpath,
	  namelist[entry_pos]->d_name,
          (guint16)danstat2->st_mode, (guint32)danstat2->st_uid,
          (guint32)danstat2->st_gid,  (guint64)danstat2->st_size,
          (guint64)danstat2->st_atime,(guint64)danstat2->st_mtime,
          (guint64)danstat2->st_ctime,(guint16)FILE_DATA_IS_DIRECTORY,
	  tmptype,FALSE);
      
        text[0] = namelist[entry_pos]->d_name;
        sibling = gtk_ctree_insert_node(pGFileTree->Ctree,parent,NULL,text,5,
			tmptype->pixmap,tmptype->mask,tmptype2->pixmap,
			tmptype2->mask, FALSE,FALSE);
        gtk_ctree_node_set_row_data(pGFileTree->Ctree,sibling,vFileData);
	
	if(nb_level) /* scan nb_level under level in the tree */
	  gfiletree_load_path(pGFileTree,currentfich,sibling,nb_level-1);
      }
    g_free(currentfich);
    entry_pos++;
  }
  g_free(currentpath);
  g_free(namelist);
  g_free(danstat2);
  
  /* sort the tree DOESN'T WORK (WHY ?)*/
  gtk_ctree_sort_recursive(GTK_CTREE(pGFileTree->Ctree),parent);
}
/***************************************************/

/***************************************************/
/* Load the root tree. This must be done only when */
/* we create a new GFileTree.			   */
static void gfiletree_load_root_tree(GFileTree *pGFileTree)
{
  file_type *tmptype, *tmptype2;

  gchar *message[1]={"/"};
  gchar *text[1];
  GtkCTreeNode *sibling;
  FileData *vFileData;
  
  /* needed to forced root_node to be in current mode */
  gchar *tmp_text;
  guint8 spacing;
  GdkPixmap *pixmap_cb;
  GdkBitmap *mask_cb;
  gboolean is_leaf;
  gboolean expanded;
  GdkPixmap *pixmap_info;
  GdkBitmap *mask_info;
  gchar *tmp_string;

  gtk_clist_freeze(GTK_CLIST(pGFileTree->Ctree));
  /* clear to be sure there is nothing */
  gtk_clist_clear(GTK_CLIST(pGFileTree->Ctree));
  
  /* find directory types */
  tmptype = file_type_find_type(pGFileTree->Preference.file_type_list,
    PAST_DIRECTORY,FALSE);
  if(tmptype == NULL) {gfile_error(FILE_TYPE_ERROR); exit(FILE_TYPE_ERROR);}
  tmptype2 = file_type_find_type(pGFileTree->Preference.file_type_list,
    PAST_OPEN_DIRECTORY,FALSE);
  if(tmptype2 == NULL){gfile_error(FILE_TYPE_ERROR); exit(FILE_TYPE_ERROR);}
  
  /* place the root_node */
  pGFileTree->root_node=gtk_ctree_insert_node(pGFileTree->Ctree,
  		NULL, NULL, message, 5, tmptype->pixmap,
		tmptype->mask, tmptype2->pixmap, tmptype2->mask,
		FALSE, TRUE);

  /* special data */
  vFileData = file_data_new("", "/",
    (guint16)0/*st_mode*/, (guint32)0/*st_uid*/,
    (guint32)0/*st_gid*/,  (guint64)0/*st_size*/,
    (guint64)0/*st_atime*/,(guint64)0/*st_mtime*/,
    (guint64)0/*st_ctime*/,(guint16)FILE_DATA_IS_DIRECTORY|FILE_DATA_IS_DIR_SCAN,
    tmptype,FALSE);

  gtk_ctree_node_set_row_data(pGFileTree->Ctree,pGFileTree->root_node,
                              vFileData);
  /* scan at 2 sub levels */
  gfiletree_load_path(pGFileTree,"/",pGFileTree->root_node,1);
  gtk_clist_thaw(GTK_CLIST(pGFileTree->Ctree));

  /* fill the path */
  pGFileTree->last_node = pGFileTree->root_node;
  if(pGFileTree->current_path != NULL) g_free(pGFileTree->current_path);
  pGFileTree->current_path = g_malloc(2);
  pGFileTree->current_path[0]='/'; pGFileTree->current_path[1]=0;

  /* select the root_node */
  gtk_signal_emit_by_name(GTK_OBJECT(pGFileTree->Ctree),
             "tree_select_row",pGFileTree->root_node,0);

  /* we must forced the root to be in "current mode" */
  gtk_ctree_get_node_info(pGFileTree->Ctree,pGFileTree->root_node,&tmp_text,
    &spacing,&pixmap_info,&mask_info,&pixmap_cb,&mask_cb,&is_leaf,&expanded);
  tmptype = file_type_find_type(pGFileTree->Preference.file_type_list,
    CURRENT_DIRECTORY,FALSE);
  if(tmptype == NULL) {gfile_error(FILE_TYPE_ERROR); exit(FILE_TYPE_ERROR);}
  tmptype2 = file_type_find_type(pGFileTree->Preference.file_type_list,
    CURRENT_OPEN_DIRECTORY,FALSE);    
  if(tmptype2 == NULL) {gfile_error(FILE_TYPE_ERROR); exit(FILE_TYPE_ERROR);}
 
  tmp_string = g_malloc(strlen(tmp_text)+1);
  if(tmp_string == NULL){gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}
  strcpy(tmp_string,tmp_text);
  /* change to have the style of the current file type directory */
  gtk_ctree_set_node_info(pGFileTree->Ctree,pGFileTree->root_node,tmp_string,
    spacing,tmptype->pixmap,tmptype->mask, tmptype2->pixmap,tmptype2->mask,
    is_leaf,expanded);
  g_free(tmp_string);
}
/***************************************************/

/*******************************************/
/* called each time a tree row is selected */
static void gfiletree_select_cb(GtkCTree *ctree_cb, GtkCTreeNode *row_cb, 
		    gint column,GFileTree *pGFileTree)
{ gchar *text;
  guint8 spacing;
  GdkPixmap *pixmap_cb;
  GdkBitmap *mask_cb;
  GtkCTreeNode *last_node_cb;
  gboolean is_leaf;
  gboolean expanded;
  GdkPixmap *pixmap_info;
  GdkBitmap *mask_info;
  file_type *tmptype, *tmptype2;

  FileData  *vFileData;
  FileData  *vFileData2;
  gchar *tmp_string;

  /* freeze the tree to avoid fliker */
  gtk_clist_freeze(GTK_CLIST(pGFileTree->Ctree));

  /* do if the guy click on something new */
  if(row_cb != pGFileTree->last_node)
  {
    gtk_ctree_node_get_pixtext(ctree_cb,row_cb,column,&text,
                            &spacing,&pixmap_cb,&mask_cb);
    vFileData = gtk_ctree_node_get_row_data(ctree_cb,row_cb);
/*    g_print("ctree row selected signal : %s%s\n",vFileData->path,vFileData->name);
*/
    /* change the current path */
    g_free(pGFileTree->current_path);
    /* put the new current_path */
    if(!strcmp(vFileData->name,"/"))
    {
      pGFileTree->current_path = g_malloc(2);
      if(pGFileTree->current_path == NULL)
      {gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}
      pGFileTree->current_path[0] = '/';
      pGFileTree->current_path[1] = 0;      
    } else {
      pGFileTree->current_path = g_malloc(strlen(vFileData->path)+
        strlen(vFileData->name)+2);
      if(pGFileTree->current_path == NULL)
      {gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}
      sprintf(pGFileTree->current_path,"%s%s/",vFileData->path,vFileData->name);
    }

    /* define last current node as a past directory */        
    gtk_ctree_get_node_info(pGFileTree->Ctree,pGFileTree->last_node,&text,&spacing,
                            &pixmap_info,&mask_info,&pixmap_cb,&mask_cb,
                            &is_leaf,&expanded);
    tmptype = file_type_find_type(pGFileTree->Preference.file_type_list,
      PAST_DIRECTORY,FALSE);
    if(tmptype == NULL) {gfile_error(FILE_TYPE_ERROR); exit(FILE_TYPE_ERROR);}
    tmptype2 = file_type_find_type(pGFileTree->Preference.file_type_list,
      PAST_OPEN_DIRECTORY,FALSE);    
    if(tmptype2 == NULL) {gfile_error(FILE_TYPE_ERROR); exit(FILE_TYPE_ERROR);}

    tmp_string = g_malloc(strlen(text)+1);
    if(tmp_string == NULL){gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}
    strcpy(tmp_string,text);

    /* change to previous current file type directory into a past one */
    gtk_ctree_set_node_info(pGFileTree->Ctree,pGFileTree->last_node,tmp_string,
      spacing,tmptype->pixmap,tmptype->mask,
      tmptype2->pixmap,tmptype2->mask, is_leaf, expanded);

    g_free(tmp_string);

    /* define the current selected row as the current directory */
    gtk_ctree_get_node_info(ctree_cb,row_cb,&text,&spacing,
                            &pixmap_info,&mask_info,&pixmap_cb,&mask_cb,
                            &is_leaf,&expanded);			    
    tmptype = file_type_find_type(pGFileTree->Preference.file_type_list,
      CURRENT_DIRECTORY,FALSE);
    if(tmptype == NULL) {gfile_error(FILE_TYPE_ERROR); exit(FILE_TYPE_ERROR);}
    tmptype2 = file_type_find_type(pGFileTree->Preference.file_type_list,
      CURRENT_OPEN_DIRECTORY,FALSE);    
    if(tmptype2 == NULL) {gfile_error(FILE_TYPE_ERROR); exit(FILE_TYPE_ERROR);}

    tmp_string = g_malloc(strlen(text)+1);
    if(tmp_string == NULL){gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}
    strcpy(tmp_string,text);
    /* change to have the style of the current file type directory */
    gtk_ctree_set_node_info(ctree_cb,row_cb,tmp_string,spacing,
      tmptype->pixmap,tmptype->mask, tmptype2->pixmap,tmptype2->mask,
      is_leaf,expanded);
    g_free(tmp_string);

    pGFileTree->last_node = row_cb;

    /* emit the corresponding signal */
    if(allow_change_signal == TRUE)
    {
      tmp_string = (gchar *)g_malloc(strlen(pGFileTree->current_path)+1);
      if(tmp_string == NULL){gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}
      strcpy(tmp_string,pGFileTree->current_path);
      /* if current path is not only "/", suppress the last '/' */
      if(tmp_string[1] != 0) tmp_string[strlen(pGFileTree->current_path)-1] = 0;
      gtk_signal_emit(GTK_OBJECT(pGFileTree),
        gfiletree_signals[CHANGE_DIR_SIGNAL],tmp_string);
      g_free(tmp_string);
    }  

    /* scan the under tree if not known */
    if(!(vFileData->mode & FILE_DATA_IS_DIR_SCAN))
    {
      /*g_print("never scan\n");*/
      last_node_cb = GTK_CTREE_ROW(row_cb)->children;

      while((last_node_cb != NULL) && (GTK_CTREE_ROW(last_node_cb )->parent == row_cb))
      {
         vFileData2 = gtk_ctree_node_get_row_data(ctree_cb,last_node_cb);
	 tmp_string = g_malloc(strlen(vFileData2->path)+
	   strlen(vFileData2->name)+1);
         if(tmp_string == NULL)
	 {gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}
         sprintf(tmp_string,"%s%s",vFileData2->path,vFileData2->name);
         /*g_print("sous rep : %s\n",tmp_string);*/
	 gfiletree_load_path(pGFileTree,tmp_string,
                             last_node_cb,0);
	 g_free(tmp_string);
         last_node_cb = GTK_CTREE_NODE_NEXT(last_node_cb);
      }
      /* current directory is now scanned */
      vFileData->mode |= FILE_DATA_IS_DIR_SCAN;
    } /* end of if(!(vFileDa... */ /*else {g_print("already scan\n");}*/
  }

  /* Job finish. Unfreeze the tree. */
  gtk_clist_thaw (GTK_CLIST(pGFileTree->Ctree));
}
/*******************************************/

/************************************************************/
/* use this if you want to unselect every thing in the tree */
void gfiletree_unselect_all(GFileTree *pGFileTree)
{
  gtk_signal_emit_by_name(GTK_OBJECT(pGFileTree->Ctree),"tree_unselect_row",
    pGFileTree->last_node,0);
}
/************************************************************/

/* ____________________ CORRECTION ENDED HERE ________________ */

static void gfiletree_select_before_cb(GtkCTree *ctree_cb,GtkCTreeNode *row_cb,
                         gint column,GFileTree *pGFileTree)
{
/*  gtk_clist_freeze(GTK_CLIST(pGFileTree->Ctree));
*/}
			 
static void gfiletree_expand_before_cb(GtkCTree *ctree_cb, GtkCTreeNode *node,
                         GFileTree *pGFileTree)
{
/*  gtk_clist_freeze(GTK_CLIST(pGFileTree->Ctree));
*/}			 

/*************************************************/
/* called each time a part of the tree is expand */
static void gfiletree_expand_cb(GtkCTree *ctree_cb, GtkCTreeNode *node,GFileTree *pGFileTree)
{
  GtkCTreeNode *last_node_cb;

  FileData  *vFileData;
  FileData  *vFileData2;
  gchar *tmp_string;

  /* freeze the tree to avoid fliker */
  gtk_clist_freeze(GTK_CLIST(pGFileTree->Ctree));

  vFileData = gtk_ctree_node_get_row_data(ctree_cb,node);
 
  /* scan the under tree if not known */
  if(!(vFileData->mode & FILE_DATA_IS_DIR_SCAN))
  {
    last_node_cb = GTK_CTREE_ROW(node)->children;

    while((last_node_cb != NULL) && (GTK_CTREE_ROW(last_node_cb)->parent == node))
    {
       vFileData2 = gtk_ctree_node_get_row_data(ctree_cb,last_node_cb);
       tmp_string = g_malloc(strlen(vFileData2->path)+strlen(vFileData2->name)+1);
       if(tmp_string == NULL)
       {gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}
       sprintf(tmp_string,"%s%s",vFileData2->path,vFileData2->name);
       /*g_print("sous rep : %s\n",tmp_string);*/
       gfiletree_load_path(pGFileTree,tmp_string,last_node_cb,0);
       g_free(tmp_string);
       last_node_cb = GTK_CTREE_NODE_NEXT(last_node_cb);
    }
    /* current directory is now scanned */
    vFileData->mode |= FILE_DATA_IS_DIR_SCAN;
  } /* end of if(!(vFileDa... */ /*else {g_print("already scan\n");}*/

  /* Job finish. Unfreeze the tree. */
  gtk_clist_thaw (GTK_CLIST(pGFileTree->Ctree));
  
/*  gfiletree_select_cb(ctree_cb,node,0,pGFileTree);  
*/
}
/*************************************************/

/****************************************/
/* move up to directory in current path */
void gfiletree_up(GFileTree *pGFileTree, gchar *directory)
{ gchar *text;
  guint8 spacing;
  GdkPixmap *pixmap_cb;
  GdkBitmap *mask_cb;
  GtkCTreeNode *try_node;

   /*g_print("string to found : %s\n",directory);*/
   /* expand the tree at the current node */
   gtk_ctree_expand(pGFileTree->Ctree,pGFileTree->last_node);
   try_node = GTK_CTREE_ROW(pGFileTree->last_node)->children;
   /* try to found directory */
   while((try_node)&&(GTK_CTREE_ROW(try_node)->parent==pGFileTree->last_node))
   {
      gtk_ctree_node_get_pixtext(pGFileTree->Ctree,try_node,0,&text,
                            &spacing,&pixmap_cb,&mask_cb);  
      /*g_print("text choice : %s\n",text);*/
      
      if(!strcmp(directory,text))
      { 
        /*g_print("list expand found : %s\n",text);*/
	/* block the up_dir signal because the user already know */
	allow_change_signal = FALSE;
        gfiletree_select_cb(GTK_CTREE(pGFileTree->Ctree), try_node, 0, pGFileTree);
	allow_change_signal = TRUE;
        pGFileTree->last_node=try_node;
        return;
      }
      try_node = GTK_CTREE_NODE_NEXT(try_node);
   }
   /* Si on arrive jusque ici, c'est que l'arbre n'est plus bon
      par rapport a la realite. Ce cas n'est pas encore fait. */
   g_print("Erreur list : expand ctree\n");
}
/****************************************/

/****************************/
/* down to parent directory */
void gfiletree_down(GFileTree *pGFileTree)
{ GtkCTreeNode *new_node;

  if(pGFileTree->last_node != pGFileTree->root_node )
  {
    new_node = GTK_CTREE_ROW(pGFileTree->last_node)->parent;
    /* if we are not at the root */
    if(new_node != pGFileTree->root_node)
      gtk_ctree_collapse(pGFileTree->Ctree,new_node);
    /* block the up_dir signal because the user already know */
    allow_change_signal = FALSE;
    gfiletree_select_cb(GTK_CTREE(pGFileTree->Ctree), new_node, 0, pGFileTree);
    allow_change_signal = TRUE;
  }
}
/****************************/

/***************************************************/
/* This must be called when file type have changed */

/* The file type must have been change in pGFileTree->Preference.file_type_list */
static void gfiletree_update_filetype_part(GFileTree *pGFileTree,GtkCTreeNode *node)
{ GtkCTreeNode *vnode;
  file_type *type_close, *type_open;
  gchar *text,*text_copy;
  guint8 spacing;
  GdkPixmap   *pixmap_closed;
  GdkBitmap   *mask_closed;
  GdkPixmap   *pixmap_opened;
  GdkBitmap   *mask_opened;
  gboolean    is_leaf;
  gboolean    expanded;
  
  FileData    *vFileData;

  vnode = node;
  while(vnode != NULL)
  {
    if(GTK_CTREE_ROW(vnode)->children != NULL)
    {
      gfiletree_update_filetype_part(pGFileTree,GTK_CTREE_ROW(vnode)->children);
    }
    vFileData = gtk_ctree_node_get_row_data(pGFileTree->Ctree,vnode);
    if(vnode == pGFileTree->last_node)
    {
      type_close = file_type_find_type(pGFileTree->Preference.file_type_list,
        CURRENT_DIRECTORY,FALSE);
      if(type_close == NULL){gfile_error(FILE_TYPE_ERROR); exit(FILE_TYPE_ERROR);}
      type_open = file_type_find_type(pGFileTree->Preference.file_type_list,
        CURRENT_OPEN_DIRECTORY,FALSE);
      if(type_open == NULL){gfile_error(FILE_TYPE_ERROR); exit(FILE_TYPE_ERROR);}
    } else
    if(vFileData->mode & FILE_DATA_IS_DIR_SCAN)
    { 
      type_close = file_type_find_type(pGFileTree->Preference.file_type_list,
        PAST_DIRECTORY,FALSE);
      if(type_close == NULL){gfile_error(FILE_TYPE_ERROR); exit(FILE_TYPE_ERROR);}
      type_open = file_type_find_type(pGFileTree->Preference.file_type_list,
        PAST_OPEN_DIRECTORY,FALSE);
      if(type_open == NULL){gfile_error(FILE_TYPE_ERROR); exit(FILE_TYPE_ERROR);}
    } else {
      type_close = file_type_find_type(pGFileTree->Preference.file_type_list,
        NORMAL_DIRECTORY,FALSE);
      if(type_close == NULL){gfile_error(FILE_TYPE_ERROR); exit(FILE_TYPE_ERROR);}
      type_open = file_type_find_type(pGFileTree->Preference.file_type_list,
        NORMAL_OPEN_DIRECTORY,FALSE);
      if(type_open == NULL){gfile_error(FILE_TYPE_ERROR); exit(FILE_TYPE_ERROR);}
    }
    gtk_ctree_get_node_info(pGFileTree->Ctree,vnode,&text,&spacing,
 			    &pixmap_closed,&mask_closed,&pixmap_opened,
			    &mask_opened,&is_leaf,&expanded);
    text_copy = g_malloc(strlen(text)+1);
    if(text_copy == NULL){gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}
    strcpy(text_copy,text);
    gtk_ctree_set_node_info(pGFileTree->Ctree,vnode,text_copy,spacing,
      type_close->pixmap,type_close->mask,type_open->pixmap,
      type_open->mask,is_leaf,expanded);
      
    g_free(text_copy);
    vnode = GTK_CTREE_NODE_NEXT(vnode);
  }
}

/* The caller is responsible for freeing the       */
/* previous file type list                         */
void gfiletree_set_filetype(GFileTree *pGFileTree,FileType *new_file_type_list)
{
  /* Freeze the ctree to avoid flicker */
  gtk_clist_freeze(GTK_CLIST(pGFileTree->Ctree));
  pGFileTree->Preference.file_type_list = new_file_type_list;
  gfiletree_update_filetype_part(pGFileTree,pGFileTree->root_node);
  /* Unfreeze the ctree */
  gtk_clist_thaw(GTK_CLIST(pGFileTree->Ctree));  
}
/***************************************************/

/******************************************/
/* Find the node of directory (ex : usr)  */
/* in the list of child node of node      */
/* Return NULL if not found               */
static GtkCTreeNode *gfiletree_find_dir_at_node(GFileTree *pGFileTree,
                        GtkCTreeNode *parent,gchar *directory)
{ gchar *text;
  guint8 spacing;
  GdkPixmap *pixmap_cb;
  GdkBitmap *mask_cb;
  GtkCTreeNode *try_node;

  try_node = GTK_CTREE_ROW(parent)->children;
  while((try_node)&&(GTK_CTREE_ROW(try_node)->parent==parent))
  {
    gtk_ctree_node_get_pixtext(pGFileTree->Ctree,try_node,0,&text,
                            &spacing,&pixmap_cb,&mask_cb);
    /* if equal then we have found the node */
    if(!strcmp(directory,text)) return(try_node);

    try_node = GTK_CTREE_NODE_NEXT(try_node);
  }
  return(NULL);
}
/******************************************/

/*****************************************/
/* Add directory "directory" to the tree */
/* ex : directory=/usr/bin/newdir to add */
/* Do not end by a '/'                   */
/* return : FALSE if this directory      */
/*          doesn't exist or parent are  */
/*	    bad. Else:TRUE               */
/*__________ UNDER DEVELOPEMENT _________*/
extern void abs_2_rel(gchar *full_name,gchar **path,gchar **name);
gboolean gfiletree_add_dir_by_name(GFileTree *pGFileTree,gchar *directory)
{
  struct stat *danstat2;
  file_type *tmptype, *tmptype2;
  gchar *text[1];
  GtkCTreeNode *sibling,*parent,*parent_tmp;
  guint current_pos,copy_pos;
  gchar *tmp_string;
  FileData *vFileData;
  
  gchar *name,*path;

  /* a protection */
  if((directory == NULL) || (directory[0]!='/')) return(FALSE);

  /* alloc some mem for detail on file  */
  danstat2 = (struct stat *)malloc(sizeof(struct stat));
  if(danstat2 == NULL) {g_print("NOT enought memory"); exit(1);}

  if( stat(directory,danstat2) ||
      ( (danstat2->st_mode >> 12) != 4) ||
      !strcmp(&directory[strlen(directory)-3],"/..") ||
      !strcmp(&directory[strlen(directory)-3],"/.") )
  /* if the file does not exist or it is not a directory or it is .. or . */
  {g_free(danstat2); return(FALSE);}

/*  g_print("we must add a directory.\n");
*/
  /* find the node we must attache the new directory */
  /* alloc mem for finding directory parent (cut using / as separator) */
  tmp_string = (gchar *)g_malloc(strlen(directory)+2);
  if(tmp_string == NULL){g_print("Can't allocate memory.\n"); exit(1);}
  current_pos = 1;
  parent = pGFileTree->root_node;
  
  while(1)
  {
    copy_pos = 0;
    while(directory[current_pos] !='/')
    { 
      tmp_string[copy_pos++] = directory[current_pos];
      /* if we have just copy the 0, we are at the end of the string  */
      /* and what we have in tmp_string is the name of the dir to add */
      if(directory[current_pos++] == 0) goto end_sub_dir;
    }
    current_pos++;
    tmp_string[copy_pos] = 0;
    g_print("add sub dir %s\n",tmp_string);
    /* find the corresponding node */
    parent_tmp = gfiletree_find_dir_at_node(pGFileTree,parent,tmp_string);
    if(parent_tmp == NULL) /* dir not listed, had it */
    { g_print("tree don't know dest directory\n");
      if(gfiletree_load_path(pGFileTree,tmp_string,parent,0)==FALSE)
      {g_print("Unexpected error.*\n"); exit(2);}
    }
    parent = parent_tmp;
  }  
  

end_sub_dir:
  g_print("to copy dir %s\n",tmp_string);
  /* if not already present */
  if( (sibling=gfiletree_find_dir_at_node(pGFileTree,parent,tmp_string))==NULL)
  {
    g_print("not already present.\n");
    /* add the entry */
    /* find the corresponding type to a directory */
    tmptype = file_type_find_type(pGFileTree->Preference.file_type_list,NORMAL_DIRECTORY,FALSE);
    if(tmptype == NULL) { g_print("NORMAL_DIRECTORY not found ?\n"); exit(1); }
    tmptype2 = file_type_find_type(pGFileTree->Preference.file_type_list,NORMAL_OPEN_DIRECTORY,FALSE);
    if(tmptype2 == NULL) { g_print("NORMAL_OPEN_DIRECTORY not found ?\n"); exit(1); }

    text[0] = tmp_string;
    sibling = gtk_ctree_insert_node(pGFileTree->Ctree,parent,NULL,text,5,
	      tmptype->pixmap,tmptype->mask,tmptype2->pixmap,
	      tmptype2->mask, FALSE,FALSE);

    abs_2_rel(directory,&path,&name);
	      
    vFileData = file_data_new(path,name,
      (guint16)danstat2->st_mode, (guint32)danstat2->st_uid,
      (guint32)danstat2->st_gid,  (guint64)danstat2->st_size,
      (guint64)danstat2->st_atime,(guint64)danstat2->st_mtime,
      (guint64)danstat2->st_ctime,(guint16)FILE_DATA_IS_DIRECTORY,/*|FILE_DATA_IS_DIR_SCAN*/
      tmptype,FALSE);

    g_free(name); g_free(path);

    gtk_ctree_node_set_row_data(pGFileTree->Ctree,sibling,vFileData);
  }
/*
  if(gtk_ctree_node_get_row_data(pGFileTree->Ctree,sibling) == (gpointer)CTREE_NEVER_SEE_DIR)
  {
    strcpy(tmp_string,directory);
    strcat(tmp_string,"/");
*/    /* under scan */
/*    if(gfiletree_load_path(pGFileTree,tmp_string,sibling,0) == FALSE)
    {g_free(tmp_string); g_print("Unexpected error.\n"); exit(1);}
  }*/
  g_free(danstat2);
  g_free(tmp_string);
  return(TRUE);
}
/*****************************************/

/************************************************************/
/* Remove the directory "directory" of the tree if it exist */
/* ex : directory=/usr/bin/newdir to del 		    */
/* Do not end by a '/'                   		    */	
/*_____________________ UNDER DEVELOPEMENT _________________*/
void gfiletree_remove_dir_by_name(GFileTree *pGFileTree,gchar *directory)
{
  GtkCTreeNode *sibling,*parent,*parent_tmp;
  guint current_pos,copy_pos;
  gchar *tmp_string;

  /* a protection */
  if((directory == NULL) || (directory[0]!='/')) return;

  g_print("we must del a directory.\n");

  /* find the node we must attache the new directory */
  /* alloc mem for finding directory parent (cut using / as separator) */
  tmp_string = (gchar *)g_malloc(strlen(directory)+2);
  if(tmp_string == NULL){g_print("Can't allocate memory.\n"); exit(1);}
  current_pos = 1;
  parent = pGFileTree->root_node;
  
  while(1)
  {
    copy_pos = 0;
    while(directory[current_pos] !='/')
    { 
      tmp_string[copy_pos++] = directory[current_pos];
      /* if we have just copy the 0, we are at the end of the string  */
      /* and what we have in tmp_string is the name of the dir to add */
      if(directory[current_pos++] == 0) goto del_end_sub_dir;
    }
    current_pos++;
    tmp_string[copy_pos] = 0;
    g_print("del sub dir %s\n",tmp_string);
    /* find the corresponding node */
    parent_tmp = gfiletree_find_dir_at_node(pGFileTree,parent,tmp_string);
    if(parent_tmp == NULL) /* dir not listed, had it */
    {
      if(gfiletree_load_path(pGFileTree,tmp_string,parent,0)==FALSE)
      {g_print("Unexpected error.\n"); exit(2);}
    }
    parent = parent_tmp;
  }  
  

del_end_sub_dir:
  g_print("to remove dir %s\n",tmp_string);
  /* if the directory is present we remove it */
  if( (sibling=gfiletree_find_dir_at_node(pGFileTree,parent,tmp_string))!=NULL)
  {
    g_print("removing it.\n");
    gtk_ctree_remove_node(pGFileTree->Ctree,sibling);
  }
  g_free(tmp_string);
  return;
}
/************************************************************/

/************************************************************/
/* Modify the directory "directory" of the tree if it exist */
/* ex : directory=/usr/bin/newdir to modify 		    */
/* Do not end by a '/'                   		    */	
/*_____________________ UNDER DEVELOPEMENT _________________*/
void gfiletree_modify_dir_by_name(GFileTree *pGFileTree,gchar *directory,gchar *new_dir)
{
  GtkCTreeNode *sibling,*parent,*parent_tmp;
  guint current_pos,copy_pos;
  gchar *tmp_string;
  
  FileData *vFileData;
  FileData *lastFileData;
  gchar *name,*path;
  struct stat *danstat2;

  gchar       *text;
  guint8      spacing;
  GdkPixmap   *pixmap;
  GdkBitmap   *mask;


  /* a protection */
  if((directory == NULL) || (directory[0]!='/')) return;

  g_print("we must modify a directory.\n");

  /* alloc some mem for detail on file  */
  danstat2 = (struct stat *)malloc(sizeof(struct stat));
  if(danstat2 == NULL) {g_print("NOT enought memory"); exit(1);}

  if( stat(new_dir,danstat2) ||
      ( (danstat2->st_mode >> 12) != 4) ||
      !strcmp(&new_dir[strlen(directory)-3],"/..") ||
      !strcmp(&new_dir[strlen(directory)-3],"/.") )
  /* if the file does not exist or it is not a directory or it is .. or . */
  {g_free(danstat2); return;}

  /* find the node we must attache the new directory */
  /* alloc mem for finding directory parent (cut using / as separator) */
  tmp_string = (gchar *)g_malloc(strlen(directory)+2);
  if(tmp_string == NULL){g_print("Can't allocate memory.\n"); exit(1);}
  current_pos = 1;
  parent = pGFileTree->root_node;
  
  while(1)
  {
    copy_pos = 0;
    while(directory[current_pos] !='/')
    { 
      tmp_string[copy_pos++] = directory[current_pos];
      /* if we have just copy the 0, we are at the end of the string  */
      /* and what we have in tmp_string is the name of the dir to add */
      if(directory[current_pos++] == 0) goto modify_end_sub_dir;
    }
    current_pos++;
    tmp_string[copy_pos] = 0;
    g_print("del sub dir %s\n",tmp_string);
    /* find the corresponding node */
    parent_tmp = gfiletree_find_dir_at_node(pGFileTree,parent,tmp_string);
    if(parent_tmp == NULL) /* dir not listed, nothing to do */
    {
      g_free(tmp_string);      
      return;
    }
    parent = parent_tmp;
  }  
  

modify_end_sub_dir:
  g_print("to modify dir %s\n",tmp_string);
  /* if the directory is present we modify it */
  if( (sibling=gfiletree_find_dir_at_node(pGFileTree,parent,tmp_string))!=NULL)
  {
    g_print("modify it.\n");

    abs_2_rel(new_dir,&path,&name);
    
    lastFileData = gtk_ctree_node_get_row_data(pGFileTree->Ctree,sibling);

    if(lastFileData->mode & FILE_DATA_IS_DIR_SCAN)
      vFileData = file_data_new(path,name,
        (guint16)danstat2->st_mode, (guint32)danstat2->st_uid,
        (guint32)danstat2->st_gid,  (guint64)danstat2->st_size,
        (guint64)danstat2->st_atime,(guint64)danstat2->st_mtime,
        (guint64)danstat2->st_ctime,(guint16)FILE_DATA_IS_DIRECTORY|FILE_DATA_IS_DIR_SCAN,
        lastFileData->type,FALSE);
    else
      vFileData = file_data_new(path,name,
        (guint16)danstat2->st_mode, (guint32)danstat2->st_uid,
        (guint32)danstat2->st_gid,  (guint64)danstat2->st_size,
        (guint64)danstat2->st_atime,(guint64)danstat2->st_mtime,
        (guint64)danstat2->st_ctime,(guint16)FILE_DATA_IS_DIRECTORY,
        lastFileData->type,FALSE);


    gtk_ctree_node_get_pixtext(GTK_CTREE(pGFileTree->Ctree),sibling,0,&text,
      &spacing,&pixmap,&mask);
    
    gtk_ctree_node_set_pixtext(GTK_CTREE(pGFileTree->Ctree),sibling,0,name,
      spacing,pixmap,mask);
						  
    gtk_ctree_node_set_row_data(pGFileTree->Ctree,sibling,vFileData);

    g_free(name); g_free(path);
    file_data_free(lastFileData);
    
/*    gtk_ctree_remove_node(pGFileTree->Ctree,sibling);*/
  }
  g_free(danstat2);  
  g_free(tmp_string);
  return;
}
/************************************************************/
