/****************************************/
/*  GFileTree.c headers         	*/
/*					*/
/*  Description : Manage the tree list  */
/*		  widget		*/
/*					*/
/*  Written by : Daniel Lacroix	        */
/*  GFile is under the GPL2 licence     */
/****************************************/

#ifndef __GFILETREE_H__
#define __GFILETREE_H__

/* to define a new widget */
#define GFILETREE(obj)          GTK_CHECK_CAST (obj, gfiletree_get_type(),GFileTree)
#define GFILETREE_CLASS(klass)  GTK_CHECK_CLASS_CAST (klass, gfiletree_get_type(), GFileTreeClass)
#define IS_GFILETREE(obj)       GTK_CHECK_TYPE (obj, gfiletree_get_type ())

typedef struct _GFileTreePref   GFileTreePref;
typedef struct _GFileTree       GFileTree;
typedef struct _GFileTreeClass  GFileTreeClass;

struct _GFileTreePref
{
  gboolean hide_title;		/* TRUE = hide the ctree title   */  
  gchar    *title;              /* title that appear if hide_title = TRUE*/
  /* same as in GFileList, use 
    SORT_NONE
    SORT_BY_NAME        
    SORT_BY_OWNER       
    SORT_BY_GROUP       
    SORT_BY_RIGHTS      
    SORT_BY_SIZE        
    SORT_BY_ACCESS_DATE 
    SORT_BY_MODIF_DATE  
    SORT_BY_CHANGE_DATE   */
    
  guint8   sort_mode;	/* 0 = none. 1 = by name. 2 = owner. 3 = group.  */
  			/* 4 = rights. 5 = by size. 6 = create date.     */
			/* 7 = modif date. 8 = last date.                */
  FileType  *file_type_list;
};

/* the structure of a GFileTree */
struct _GFileTree
{
  GtkVBox 	Main;
  GtkCTree      *Ctree;
  GtkCTreeNode  *last_node; /* the node we are on in the ctree */
  GtkCTreeNode  *root_node; /* root node */
  gchar         *current_path; /* current_path is always ended by '/' */

  GFileTreePref Preference;
};

struct _GFileTreeClass
{
   GtkVBoxClass parent_class;
   /* Add here all the signal we want */
   void (* change_dir) (GFileTree *pGFileTree,gchar *directory);
};

/* The default GFileTree preference */
extern GFileTreePref DefaultGFileTreePref;

/* copy the content of  pGFileTreePref2 in pGFileTreePref1 */
void gfiletreepref_copy(GFileTreePref *pGFileTreePref1,
  GFileTreePref *pGFileTreePref2);

guint     gfiletree_get_type(void);
/* create gfiletree widget */
GtkWidget *gfiletree_new(GFileTreePref pGFileTreePref);
/* free a gfiletree widget */
void 	  gfiletree_free(GFileTree *pGFileTree);
/* move to up directory "directory" */
void gfiletree_up(GFileTree *pGFileTree, gchar *directory);
/* move to parent directory */
void gfiletree_down(GFileTree *pGFileTree);
/* must be called when directory type have changed */
/* The caller is responsible for freeing the       */
/* previous file type list                         */
void gfiletree_set_filetype(GFileTree *pGFileTree,
  FileType *new_file_type_list);
/* called to add a directory with the complete path */
gboolean gfiletree_add_dir_by_name(GFileTree *pGFileTree,gchar *directory);
/* called to remove a directory (and its subdirectory) with the complete path*/
void gfiletree_remove_dir_by_name(GFileTree *pGFileTree,gchar *directory);
/* use this if you want to unselect every thing in the tree */
void gfiletree_unselect_all(GFileTree *pGFileTree);

/* Modify the directory "directory" of the tree if it exist */
/* ex : directory=/usr/bin/newdir to modify 		    */
/* Do not end by a '/'                   		    */	
void gfiletree_modify_dir_by_name(GFileTree *pGFileTree,gchar *directory,gchar *new_dir);

#endif /* __GFILETREE_H__ */
