/****************************************/
/*  IconList.c body                     */
/*					*/
/*  Description : The list of all icons */
/*		  for addedit dialog	*/
/*  This is a button and when you click */
/*  on it, the list of choice is display*/
/*					*/
/*  Written by : Daniel Lacroix	        */
/*  GFile is under the GPL2 licence     */
/****************************************/

#include "GFile.h"

static void icon_list_class_init(IconListClass *pIconListClass);
static void icon_list_init(IconList *pIconList);

/*************************************************************/
guint icon_list_get_type ()
{
  static guint icon_list_type = 0;

  if (!icon_list_type)
  {
    GtkTypeInfo icon_list_info =
    {
       "IconList",
       sizeof (IconList),
       sizeof (IconListClass),
       (GtkClassInitFunc)  icon_list_class_init,
       (GtkObjectInitFunc) icon_list_init,
       (GtkArgSetFunc) NULL,
       (GtkArgGetFunc) NULL
     };
     icon_list_type = gtk_type_unique(gtk_vbox_get_type(), &icon_list_info);
  }
  return icon_list_type;
}
/*************************************************************/

/*************************************************************/
enum { ICON_CHANGE_SIGNAL, LAST_SIGNAL};
static gint icon_list_signals[LAST_SIGNAL] = { 0 };

static void string_marshal(GtkObject *object, GtkSignalFunc func, gpointer func_data, GtkArg *args) {
    typedef void (*string_marshal_func)(IconList *, gchar *,gpointer);
    string_marshal_func rfunc = (string_marshal_func) func;
    (*rfunc) (ICON_LIST(object),
	      GTK_VALUE_POINTER(args[0]), func_data);
}

static void icon_list_class_init(IconListClass *class)
{
  /* here we create the new signals if we want */
  GtkObjectClass *object_class;
  object_class = (GtkObjectClass*) class;
       
  icon_list_signals[ICON_CHANGE_SIGNAL] = gtk_signal_new ("icon_change",
    GTK_RUN_FIRST,object_class->type,
    GTK_SIGNAL_OFFSET (IconListClass,icon_change),
    string_marshal, GTK_TYPE_NONE, 1,GTK_TYPE_POINTER);

  gtk_object_class_add_signals (object_class, icon_list_signals, LAST_SIGNAL);
  class->icon_change = NULL;
}  
/*************************************************************/

static void dialog_destroy_cb(GtkWidget *Widget, IconList *pIconList)
{
  pIconList->is_dialog_open = FALSE;
}
static void dialog_cancel_cb(GtkWidget *Widget, IconList *pIconList)
{
  gnome_dialog_close(GNOME_DIALOG(pIconList->icon_dialog));
}

static void dialog_ok_cb(GtkWidget *Widget, IconList *pIconList)
{ const gchar *file_name;

  file_name = gnome_icon_selection_get_icon(
    GNOME_ICON_SELECTION(pIconList->icon_selection),TRUE);
    
  gtk_container_remove(GTK_CONTAINER(pIconList->button),
    pIconList->button_content);

  /* free the icon file name if not null */    
  if(pIconList->icon_file != NULL)
    g_free(pIconList->icon_file);

  if(file_name != NULL)
  {
    pIconList->icon_file = g_malloc(strlen(file_name)+1);
    strcpy(pIconList->icon_file,file_name);
  } else {
    pIconList->icon_file = NULL;
  }    

  /* put the content of the button (an icon or the label "no icon") */
  if(pIconList->icon_file != NULL)
    pIconList->button_content = gnome_pixmap_new_from_file(pIconList->icon_file);
  else
    pIconList->button_content = gtk_label_new("no\nicon");
  gtk_widget_show(pIconList->button_content);    
    
  gtk_container_add(GTK_CONTAINER(pIconList->button),pIconList->button_content);
					  
  gnome_dialog_close(GNOME_DIALOG(pIconList->icon_dialog));
  
  /* alert the user the icon have been changed */
  gtk_signal_emit(GTK_OBJECT(pIconList),icon_list_signals[ICON_CHANGE_SIGNAL],
    pIconList->icon_file);
}

/*************************************************************/
/* called when the button is pressed                         */
static void pressed_cb(GtkWidget *Widget, IconList *pIconList)
{ gchar *tmp_icon_dir;

  /* one icon dialog open at a time */
  if(pIconList->is_dialog_open == TRUE) return;
  pIconList->is_dialog_open = TRUE;

  pIconList->icon_selection = gnome_icon_selection_new();
  gtk_widget_show(pIconList->icon_selection);
  
  tmp_icon_dir = g_strdup(pIconList->icon_path);
  /* suppress the last '/' */
  if(strlen(tmp_icon_dir)>1) tmp_icon_dir[strlen(tmp_icon_dir)-1] = 0;
  gnome_icon_selection_add_directory(GNOME_ICON_SELECTION(pIconList->icon_selection),
    tmp_icon_dir);
  g_free(tmp_icon_dir);  
    
  gnome_icon_selection_show_icons(GNOME_ICON_SELECTION(pIconList->icon_selection));

  pIconList->icon_dialog = gnome_dialog_new("Choose an icon",
    GNOME_STOCK_BUTTON_OK,GNOME_STOCK_BUTTON_CANCEL,NULL);

  gtk_signal_connect(GTK_OBJECT(pIconList->icon_dialog),"destroy",
    GTK_SIGNAL_FUNC(dialog_destroy_cb),pIconList);

  /* the cancel button */
  gnome_dialog_button_connect(GNOME_DIALOG(pIconList->icon_dialog),1,
    GTK_SIGNAL_FUNC(dialog_cancel_cb),pIconList);
  /* the ok button */
  gnome_dialog_button_connect(GNOME_DIALOG(pIconList->icon_dialog),0,
    GTK_SIGNAL_FUNC(dialog_ok_cb),pIconList);

  gtk_box_pack_start_defaults(GTK_BOX(GNOME_DIALOG(pIconList->icon_dialog)->vbox),
    pIconList->icon_selection);

  gtk_widget_set_usize(pIconList->icon_dialog,300,300);  
    
  gtk_widget_show(pIconList->icon_dialog);
}
/*************************************************************/

/* use as a parameter given by icon_list_new for icon_list_init */
static gchar *current_icon_file;
static gchar *current_icon_path;

/*************************************************************/
static void icon_list_init(IconList *pIconList)
{
  /* take parameter in account */
  pIconList->icon_file = current_icon_file;
  pIconList->icon_path = current_icon_path;

  /* no icon_dialog open at startup */
  pIconList->is_dialog_open = FALSE;

  pIconList->button = gtk_button_new();
  gtk_widget_show(pIconList->button);

  /* put the content of the button (an icon or the label "no icon") */
  if(pIconList->icon_file != NULL)
    pIconList->button_content = gnome_pixmap_new_from_file(pIconList->icon_file);
  else
    pIconList->button_content = gtk_label_new("no\nicon");
  gtk_widget_show(pIconList->button_content);    
    
  gtk_container_add(GTK_CONTAINER(pIconList->button),pIconList->button_content);

  gtk_signal_connect(GTK_OBJECT(pIconList->button),"pressed",GTK_SIGNAL_FUNC(pressed_cb),pIconList);

  gtk_box_pack_start_defaults(GTK_BOX(pIconList),pIconList->button);  
}
/*************************************************************/

/***************************************************/
/* call when the icon list is destroy (the button) */
static void icon_list_destroy_cb(GtkWidget *Widget,IconList *pIconList)
{
  /* if the icon list dialog is open, close it */
  if(pIconList->is_dialog_open)
    gnome_dialog_close(GNOME_DIALOG(pIconList->icon_dialog));
  /* free the icon file name if not null */    
  if(pIconList->icon_file != NULL)
    g_free(pIconList->icon_file);

  g_free(pIconList->icon_path);
}
/***************************************************/

/*********************************************************************/
/* Create a new icon list widget, icon_file is the icon file name of */
/* the init icon. If icon_file is NULL, no icon are taken by default */
/* Return : the widget   	                                     */
GtkWidget *icon_list_new(gchar *icon_file, gchar *icon_path)
{ GtkWidget *vIconList;

  if(icon_file != NULL)
  {
    current_icon_file = g_malloc(strlen(icon_file)+1);
    strcpy(current_icon_file,icon_file);
  } else {
    current_icon_file = NULL;
  }    
  current_icon_path = g_strdup(icon_path);
  
  vIconList = GTK_WIDGET(gtk_type_new( icon_list_get_type() ));
 
  gtk_signal_connect(GTK_OBJECT(vIconList),"destroy",
    GTK_SIGNAL_FUNC(icon_list_destroy_cb),vIconList);

  return( vIconList );
}
/*********************************************************************/
