/****************************************/
/*  MoveDialog.c body           	*/
/*					*/
/*  Description : Manage Move (mv of    */
/*                files) widget		*/
/*					*/
/*  Written by : Daniel Lacroix	        */
/*  GFile is under the GPL2 licence     */
/****************************************/

#include "GFile.h"

static void move_dialog_class_init(MoveDialogClass *pMoveDialogClass);
static void move_dialog_init(MoveDialog *pMoveDialog);
static void destroy_cb(GtkWidget *Widget, MoveDialog *pMoveDialog);
static void cancel_cb(GtkWidget *Widget, MoveDialog *pMoveDialog);

/*************************************************************/
guint move_dialog_get_type ()
{
  static guint move_dialog_type = 0;

  if (!move_dialog_type)
  {
    GtkTypeInfo move_dialog_info =
    {
       "MoveDialog",
       sizeof (MoveDialog),
       sizeof (MoveDialogClass),
       (GtkClassInitFunc)  move_dialog_class_init,
       (GtkObjectInitFunc) move_dialog_init,
       (GtkArgSetFunc) NULL,
       (GtkArgGetFunc) NULL
     };
     move_dialog_type = gtk_type_unique(gnome_dialog_get_type(), &move_dialog_info);
  }
  return move_dialog_type;
}
/*************************************************************/

/*************************************************************/
static void move_dialog_class_init(MoveDialogClass *class)
{
  /* here we create the new signals if we want */
}
/*************************************************************/


static void list_all_we_need_to_move(MoveDialog *pMoveDialog)
{

  file_data_list_init(pMoveDialog->MoveList);
  while(!file_data_list_is_end(pMoveDialog->MoveList))
  { 
    /* do something only if it is not . or .. */
    if(strcmp(file_data_list_current(pMoveDialog->MoveList)->name,".") &&
       strcmp(file_data_list_current(pMoveDialog->MoveList)->name,".."))
    {
      pMoveDialog->current_all_file_total_nb_cpt++;
    }  
    file_data_list_next(pMoveDialog->MoveList);
  }
}


void move_current_file(MoveDialog *pMoveDialog)
{ FileData *vFileData;
  gchar *LastFile;
  gchar *NewFile;
  gfloat percent;

  vFileData = file_data_list_current(pMoveDialog->MoveList);
  if(!strcmp(vFileData->name,"..") || !strcmp(vFileData->name,".")) return;
  LastFile = g_strconcat(vFileData->path,vFileData->name,NULL);
  NewFile = g_strconcat(pMoveDialog->DestPath,vFileData->name,NULL);
  
  rename(LastFile, NewFile);

  pMoveDialog->file_move_func(LastFile,NewFile);
   
  g_free(LastFile);
  g_free(NewFile);

  /* update the total progress bar */
  pMoveDialog->current_all_file_nb_cpt++;
  percent = ((gfloat)pMoveDialog->current_all_file_nb_cpt) / 
      ((gfloat)pMoveDialog->current_all_file_total_nb_cpt);

  gtk_progress_bar_update(GTK_PROGRESS_BAR(pMoveDialog->all_file_progress_bar),percent);
  while(gtk_events_pending()) gtk_main_iteration();

}

/*****************************************/
/* called when everything has been moved */
static gint move_dialog_exit_timer_cb(MoveDialog *pMoveDialog)
{ gtk_timeout_remove (pMoveDialog->exit_timer);
  gnome_dialog_close (GNOME_DIALOG(pMoveDialog)); 
}
/*****************************************/

/* given by move_dialog_new as a parameter for move_dialog_init */
static FileDataList *current_MoveList;
static gchar *current_DestPath;
static void (* current_file_move_func) (gchar *LastFile, gchar *NewFile);

/*************************************************************/
static void move_dialog_init(MoveDialog *pMoveDialog)
{ GtkWidget *label;
  GtkWidget *table;
    
  /* take parameters in account */
  pMoveDialog->MoveList = current_MoveList;
  pMoveDialog->file_move_func  = current_file_move_func;
  pMoveDialog->DestPath = current_DestPath;
  
  gtk_window_set_title(GTK_WINDOW(pMoveDialog),"Move");
  /* add the CANCEL button */
  gnome_dialog_append_buttons(GNOME_DIALOG(pMoveDialog),
    GNOME_STOCK_BUTTON_CANCEL,NULL);

  gtk_signal_connect(GTK_OBJECT(pMoveDialog), "destroy",
    GTK_SIGNAL_FUNC(destroy_cb),pMoveDialog);    
    
  /* boutton Cancel */
  gnome_dialog_button_connect(GNOME_DIALOG(pMoveDialog),0,
    GTK_SIGNAL_FUNC(cancel_cb),pMoveDialog);
    
  gtk_widget_realize (GTK_WIDGET(pMoveDialog));

  gtk_box_set_homogeneous(GTK_BOX(GNOME_DIALOG(pMoveDialog)->vbox),FALSE);

  /* table for packing all things */
  table = gtk_table_new(3,2,FALSE);
  gtk_widget_show(table);  
  
  gtk_table_set_row_spacings(GTK_TABLE(table),10);
  gtk_table_set_col_spacings(GTK_TABLE(table),10);  
  
  gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(pMoveDialog)->vbox),
    table,TRUE, TRUE, 2);

  /* move of */
  label = gtk_label_new("Move of :");
  gtk_label_set_justify (GTK_LABEL(label),GTK_JUSTIFY_LEFT);  
  gtk_widget_show(label);  
  gtk_table_attach_defaults(GTK_TABLE(table),label, 0, 1, 0, 1);
  pMoveDialog->move_of = gtk_label_new("");
  gtk_label_set_justify (GTK_LABEL(pMoveDialog->move_of),GTK_JUSTIFY_LEFT);  
  gtk_widget_show(pMoveDialog->move_of);  
  gtk_table_attach_defaults(GTK_TABLE(table),pMoveDialog->move_of,
    1, 2, 0, 1);

  /* move to */
  label = gtk_label_new("Move to :");
  gtk_label_set_justify (GTK_LABEL(label),GTK_JUSTIFY_LEFT);  
  gtk_widget_show(label);  
  gtk_table_attach_defaults(GTK_TABLE(table),label, 0, 1, 1, 2);
  pMoveDialog->move_to = gtk_label_new("");
  gtk_label_set_justify (GTK_LABEL(pMoveDialog->move_to),GTK_JUSTIFY_LEFT);
  gtk_widget_show(pMoveDialog->move_to);  
  gtk_table_attach_defaults(GTK_TABLE(table),pMoveDialog->move_to,
    1, 2, 1, 2);

  /* total file progress bar */
  label = gtk_label_new("total");
  gtk_label_set_justify (GTK_LABEL(label),GTK_JUSTIFY_LEFT);
  gtk_widget_show(label);
  gtk_table_attach_defaults(GTK_TABLE(table),label, 0, 1, 2, 3);
  pMoveDialog->all_file_progress_bar = gtk_progress_bar_new();
    /* show the percentage */
  gtk_progress_set_show_text(GTK_PROGRESS(pMoveDialog->all_file_progress_bar),
    TRUE);
  gtk_widget_show(pMoveDialog->all_file_progress_bar);
  gtk_table_attach_defaults(GTK_TABLE(table),pMoveDialog->all_file_progress_bar,
    1, 2, 2, 3);
  
  gtk_widget_realize(GTK_WIDGET(pMoveDialog));
  /* we must do this here if we want to see something */
  gtk_widget_show(GTK_WIDGET(pMoveDialog));
  
  while(gtk_events_pending()) gtk_main_iteration();

  /* find all we have really to move */
  list_all_we_need_to_move(pMoveDialog);

  while(gtk_events_pending()) gtk_main_iteration();

  file_data_list_init(pMoveDialog->MoveList);
  while(!file_data_list_is_end(pMoveDialog->MoveList))
  { 
    move_current_file(pMoveDialog);
    file_data_list_next(pMoveDialog->MoveList);
  }

  /* when del is finished, close the dialog */
  pMoveDialog->exit_timer = gtk_timeout_add(0,
    (GtkFunction)(&move_dialog_exit_timer_cb), pMoveDialog);
}
/*************************************************************/

/***************************************************/
/* called when the user click on the cancel button */
static void cancel_cb(GtkWidget *Widget, MoveDialog *pMoveDialog)
{ gnome_dialog_close (GNOME_DIALOG(pMoveDialog)); }
/***************************************************/

/***************************************/
/* close when the dialog box is closed */
static void destroy_cb(GtkWidget *Widget, MoveDialog *pMoveDialog)
{
  file_data_list_free(pMoveDialog->MoveList);
}
/***************************************/

/***************************************/
/* Create a new Move widget            */
/* MoveDialog keep a copy of MoveList. */
void move_dialog_new (FileDataList *MoveList,gchar *DestPath,
  void (* file_move_func) (gchar *LastFile, gchar *NewFile))
{ GtkWidget *Widget;

  file_data_list_init(MoveList);
  /* if nothing to move, exit*/
  if(file_data_list_is_end(MoveList)) return;

  current_MoveList = file_data_list_new_copy(MoveList);
  current_DestPath = g_strdup(DestPath);
  current_file_move_func = file_move_func;
  Widget = GTK_WIDGET(gtk_type_new( move_dialog_get_type() ));
}
/**************************************************/
