/*********************************************/
/*  OwnerGroup.c body                        */
/*					     */
/*  Description : Create a user and a group  */
/*		  list with /etc/passwd and  */
/*		  /etc/group. It is for      */
/*		  making the relation the    */
/*		  numbers and the names	     */
/*					     */
/*  Written by : Daniel Lacroix	             */
/*  GFile is under the GPL2 licence          */
/*********************************************/

#include "GFile.h"

/********************************************/
/* Create a list with all the known groups  */
/* Return : NULL = error else the structure.*/
Group *group_load(void)
{ FILE  *fich_handle;
  gchar tempbuf[200]; /* one read line is limited to 200 characters */
  gchar *group_name;
  gint  compt,compt2;
  grp	*grp_struct;

  Group *vGroup;
  
  vGroup = g_malloc(sizeof(Group));
  if(vGroup == NULL){gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}
  vGroup->current = NULL;
  vGroup->end     = NULL;
  vGroup->list    = g_list_alloc();
  if(vGroup->list == NULL){gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}
    
  if((fich_handle = fopen("/etc/group","r")) == NULL)
  {
    g_list_free(vGroup->list); 
    g_free(vGroup);
    gfile_error(OPEN_FILE_GROUP); exit(OPEN_FILE_GROUP);
  }
  
  while(fgets(tempbuf,200,fich_handle)!=NULL)
  {
    for(compt=0;(compt<200)&&(tempbuf[compt]==' ');compt++);
    if(tempbuf[compt]!='#')
    {
      group_name = &tempbuf[compt];
      for(;(compt<200)&&(tempbuf[compt]!=':');compt++);
      tempbuf[compt] = 0;
      for(;(compt<200)&&(tempbuf[compt]!=':');compt++);
      for(compt2=compt+1;(compt2<200)&&(tempbuf[compt2]!=':');compt2++);
      tempbuf[compt2] = 0;
      grp_struct = (grp *)g_malloc(sizeof(grp));
      if(grp_struct == NULL){gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}
      grp_struct->name = (gchar *)g_malloc(strlen(group_name)+1);
      if(grp_struct->name == NULL){gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}
      strcpy(grp_struct->name,group_name);
      grp_struct->nb = atoi(&tempbuf[compt+1]);
      g_list_append(vGroup->list,grp_struct);
    }  
  }
  fclose(fich_handle);
  return(vGroup);
}
/*******************************************/

/***************************************/
/* Free the group list                 */
/* Return : TRUE = done. FALSE = error */
gboolean group_free(Group *pGroup)
{
  group_init(pGroup);
  while(group_is_end(pGroup) == FALSE)
  {  
    g_free(group_current(pGroup)->name);
    g_free(group_current(pGroup));
    g_list_remove(pGroup->list,group_current(pGroup));
    group_init(pGroup);
  }
  g_list_free(pGroup->list); 
  g_free(pGroup);
  return(TRUE);
}
/********************************/

/*****************************************************************/
/* Try to find the group with the GUID nb                        */
/* Return : NULL = not found else a pointer to the grp structure */
grp *group_find_by_nb(Group *pGroup, gint nb)
{
  group_init(pGroup);
  while(group_is_end(pGroup) == FALSE)
  {
    if(group_current(pGroup)->nb == nb) return(group_current(pGroup));
    group_next(pGroup);
  }
  return(NULL);
}
/*****************************************************************/

/*************************************************/
/* After a call to this procedure, group_current */
/* return the first grp                          */
void group_init(Group *pGroup)
{
  pGroup->current = g_list_next(pGroup->list);
  pGroup->end = NULL;
}
/*************************************************/

/****************************/
/* Return the grp we are on */
grp *group_current(Group *pGroup)
{
  return((grp *)pGroup->current->data);
}
/****************************/

/**********************/
/* Go to the next grp */
void group_next(Group *pGroup)
{
  pGroup->current = g_list_next(pGroup->current);
}
/**********************/

/*******************************/
/* say if there is no more grp */
gboolean group_is_end(Group *pGroup)
{
  return(pGroup->current == pGroup->end);
}
/*******************************/

/****************************************************************/
/* Load the known owner (user) list.                            */
/* owner_load need the list of group to affect owner to a group */
/* Return : NULL = error, else the structure.			*/
Owner *owner_load(Group *pGroup)
{ FILE  *fich_handle;
  gchar tempbuf[200];
  gchar *own_name;
  gint  compt,compt2;
  own	*own_struct;
  gint  grp_nb;

  Owner *vOwner;
  
  vOwner = g_malloc(sizeof(Owner));
  if(vOwner == NULL){ gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION); }
  vOwner->current = NULL;
  vOwner->end = NULL;
  vOwner->to_use_group = pGroup;
  vOwner->list = g_list_alloc();
  if(vOwner->list == NULL){ gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION); }

  if((fich_handle = fopen("/etc/passwd","r")) == NULL)
  { 
    g_list_free(vOwner->list); 
    g_free(vOwner);
    gfile_error(OPEN_FILE_PASSWD);
    exit(OPEN_FILE_PASSWD);
  }
  
  while(fgets(tempbuf,200,fich_handle)!=NULL)
  {
    for(compt=0;(compt<200)&&(tempbuf[compt]==' ');compt++);
    if(tempbuf[compt]!='#')
    {
      own_name = &tempbuf[compt];
      for(;(compt<200)&&(tempbuf[compt]!=':');compt++);
      tempbuf[compt] = 0;
      for(;(compt<200)&&(tempbuf[compt]!=':');compt++);
      for(compt2=compt+1;(compt2<200)&&(tempbuf[compt2]!=':');compt2++);
      tempbuf[compt2] = 0;
      own_struct = (own *)g_malloc(sizeof(own));
      if(own_struct == NULL){ gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION); }
      own_struct->name = (gchar *)g_malloc(strlen(own_name));
      if(own_struct->name == NULL){ gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION); }
      strcpy(own_struct->name,own_name);
      own_struct->nb = atoi(&tempbuf[compt+1]);
      for(compt=compt2+1;(compt<200)&&(tempbuf[compt]!=':');compt++);
      tempbuf[compt] = 0;
      grp_nb = atoi(&tempbuf[compt2+1]);
      /* don't forget grp can be null if not found */
      own_struct->grp = group_find_by_nb(vOwner->to_use_group,grp_nb);
      g_list_append(vOwner->list,own_struct);
    }  
  }
  fclose(fich_handle);
  
  /* try to found the user of GFile in this list */
  vOwner->current_user = owner_find_by_name(vOwner, g_get_user_name());
  
  return(vOwner);
}
/*******************************************************************/

/********************************************/
/* Free the own list. 		            */
/* Return : TRUE = job done. FALSE = error. */
gboolean owner_free(Owner *pOwner)
{

  owner_init(pOwner);
  while(owner_is_end(pOwner) == FALSE)
  {
    g_free(owner_current(pOwner)->name);
    g_free(owner_current(pOwner));
    g_list_remove(pOwner->list,owner_current(pOwner));
    owner_init(pOwner);
  }
  g_list_free(pOwner->list);
  g_free(pOwner);
  return(TRUE);
}
/*************************************/

/******************************************************************/
/* Try to found the UID nb in the known owner list                */
/* Return : NULL if not found else a pointer to the own structure */
own *owner_find_by_nb(Owner *pOwner, gint nb)
{
  owner_init(pOwner);
  while(owner_is_end(pOwner) == FALSE)
  {
    if(owner_current(pOwner)->nb == nb) return(owner_current(pOwner));
    owner_next(pOwner);
  }
  return(NULL);
}
/******************************************************************/

/******************************************************************/
/* Try to found the name in the known owner list                  */
/* Return : NULL if not found else a pointer to the own structure */
own *owner_find_by_name(Owner *pOwner, gchar *name)
{
  owner_init(pOwner);
  while(owner_is_end(pOwner) == FALSE)
  {
    if(!strcmp(owner_current(pOwner)->name,name)) return(owner_current(pOwner));
    owner_next(pOwner);
  }
  return(NULL);
}
/******************************************************************/

/******************************************************************/
/* Try to found the current user of GFile                         */
/* Return : NULL if not found else a pointer to the own structure */
own *owner_current_user(Owner *pOwner)
{
  return(pOwner->current_user);
}
/******************************************************************/

/*************************************************/
/* After a call to this procedure, owner_current */
/* return the first own                          */
void owner_init(Owner *pOwner)
{
  pOwner->current = g_list_next(pOwner->list);
  pOwner->end = NULL;
}
/*************************************************/

/****************************/
/* Return the own we are on */
own *owner_current(Owner *pOwner)
{
  return((own *)pOwner->current->data);
}
/****************************/

/**********************/
/* Go to the next own */
void owner_next(Owner *pOwner)
{
  pOwner->current = g_list_next(pOwner->current);
}
/**********************/

/*******************************/
/* say if there is no more own */
gboolean owner_is_end(Owner *pOwner)
{
  return(pOwner->current == pOwner->end);
}
/*******************************/
