/*********************************************/
/*  OwnerGroup.c headers                     */
/*					     */
/*  Description : Create a user and a group  */
/*		  list with /etc/passwd and  */
/*		  /etc/group. It is for      */
/*		  making the relation the    */
/*		  numbers and the names	     */
/*					     */
/*  Written by : Daniel Lacroix	             */
/*  GFile is under the GPL2 licence          */
/*********************************************/

#ifndef __OWNER_GROUP_H__
#define __OWNER_GROUP_H__

typedef struct _grp   grp;
typedef struct _own   own;
typedef struct _Group Group;
typedef struct _Owner Owner;

/* The group structure */
struct _grp
{
  gchar *name;
  gint  nb;
};

/* The user structure. */
struct _own
{
  gchar *name;
  gint  nb;
  grp   *grp;
};

struct _Group
{
  /* the list of all the groups */
  GList *list;
  /* the two above are for going over the list */
  /* current group */
  GList *current;
  /* after the last group (stop condition) */
  GList *end;  
};

struct _Owner
{
  /* the list of all the owners */
  GList *list;
  /* List of owners groups */
  Group *to_use_group;
  /* current user of GFile */
  own   *current_user;
  /* the two above are for going over the list */
  /* current owner */
  GList *current;
  /* after the last owner (stop condition) */
  GList *end;
};

/* Create a list with all the known groups  */
/* Return : NULL = error else the structure.*/
Group *group_load(void);

/* Free the group list                 */
/* Return : TRUE = done. FALSE = error */
gboolean group_free(Group *pGroup);

/* Try to find the group with the GUID nb                        */
/* Return : NULL = not found else a pointer to the grp structure */
grp *group_find_by_nb(Group *pGroup, gint nb);

/*********************************************************/
/************** FOR GOING OVER ALL THE GROUPS ************/

/* After a call to this procedure, group_current */
/* return the first grp                          */
void group_init(Group *pGroup);

/* Return the grp we are on */
grp *group_current(Group *pGroup);

/* Go to the next grp */
void group_next(Group *pGroup);

/* say if there is no more grp */
gboolean group_is_end(Group *pGroup);
/*********************************************************/

/* Load the known owner (user) list.                            */
/* owner_load need the list of group to affect owner to a group */
/* Return : NULL = error, else the structure.			*/
Owner *owner_load(Group *pGroup);

/* Free the own list. 		            */
/* Return : TRUE = job done. FALSE = error. */
gboolean owner_free(Owner *pOwner);

/* Try to found the UID nb in the known owner list                */
/* Return : NULL if not found else a pointer to the own structure */
own *owner_find_by_nb(Owner *pOwner, gint nb);

/* Try to found the name in the known owner list                  */
/* Return : NULL if not found else a pointer to the own structure */
own *owner_find_by_name(Owner *pOwner, gchar *name);

/* Try to found the current user of GFile                         */
/* Return : NULL if not found else a pointer to the own structure */
own *owner_current_user(Owner *pOwner);

/*********************************************************/
/************** FOR GOING OVER ALL THE GROUPS ************/

/* After a call to this procedure, owner_current */
/* return the first own                          */
void owner_init(Owner *pOwner);

/* Return the own we are on */
own *owner_current(Owner *pOwner);

/* Go to the next own */
void owner_next(Owner *pOwner);

/* say if there is no more own */
gboolean owner_is_end(Owner *pOwner);
/*********************************************************/

#endif /* __OWNER_GROUP_H__ */
