/****************************************/
/*  PreferenceDialog.c body             */
/*					*/
/*  Description : Manage the Preference */
/*		  dialog		*/
/*					*/
/*  Written by : Daniel Lacroix	        */
/*  GFile is under the GPL2 licence     */
/****************************************/

#include "GFile.h"

static void preference_dialog_class_init(PreferenceDialogClass *class);
static void preference_dialog_init(PreferenceDialog *pPreferenceDialog);

/*************************************************************/
guint preference_dialog_get_type ()
{
  static guint preference_dialog_type = 0;

  if (!preference_dialog_type)
  {
    GtkTypeInfo preference_dialog_info =
    {
       "PreferenceDialog",
       sizeof (PreferenceDialog),
       sizeof (PreferenceDialogClass),
       (GtkClassInitFunc)  preference_dialog_class_init,
       (GtkObjectInitFunc) preference_dialog_init,
       (GtkArgSetFunc) NULL,
       (GtkArgGetFunc) NULL
     };
     preference_dialog_type = gtk_type_unique(gnome_dialog_get_type(), &preference_dialog_info);
  }
  return preference_dialog_type;
}
/*************************************************************/

/*************************************************************/
enum { APPLY_SIGNAL, LAST_SIGNAL};
static gint preference_dialog_signals[LAST_SIGNAL] = { 0 };

static void PreferenceDialogPref_marshal(GtkObject *object, GtkSignalFunc func, gpointer func_data, GtkArg *args) {
    typedef void (*PreferenceDialogPref_marshal_func)(PreferenceDialog *,
      PreferenceDialogPref *, gpointer);
    PreferenceDialogPref_marshal_func rfunc = (PreferenceDialogPref_marshal_func) func;
    (*rfunc) (PREFERENCE_DIALOG(object),
	      GTK_VALUE_POINTER(args[0]), func_data);
}

static void preference_dialog_class_init(PreferenceDialogClass *class)
{
  /* here we create the new signals if we want */
  GtkObjectClass *object_class;
  object_class = (GtkObjectClass*) class;

  preference_dialog_signals[APPLY_SIGNAL] = gtk_signal_new
            ("apply",
            GTK_RUN_FIRST,object_class->type,
            GTK_SIGNAL_OFFSET (PreferenceDialogClass,apply),
            PreferenceDialogPref_marshal, GTK_TYPE_NONE, 1,GTK_TYPE_POINTER);    
	    
   gtk_object_class_add_signals(object_class,
     preference_dialog_signals, LAST_SIGNAL);
   class->apply = NULL;    
}
/*************************************************************/

static void destroy_cb(GtkWidget *widget,PreferenceDialog *pPreferenceDialog)
{
  /* free the allocated icon_path */
  g_free(pPreferenceDialog->Preference.icon_path);
  /* free the allocated file_type_save_file */
  g_free(pPreferenceDialog->Preference.file_type_save_file);  
  /* free the file type list we were working on */
  file_type_free(pPreferenceDialog->Preference.file_type_list);
  
  g_print("PreferenceDialog is destroy\n");  
}

/* exit without modifications */
static void cancel_cb(GtkWidget *widget,PreferenceDialog *pPreferenceDialog)
{  
  gnome_dialog_close(GNOME_DIALOG(pPreferenceDialog));
}

static void apply_cb(GtkWidget *widget,PreferenceDialog *pPreferenceDialog)
{
  /* update preferences */
  pPreferenceDialog->Preference.general_mode = 
    preference_general_mode(PREFERENCE_GENERAL(pPreferenceDialog->general_content));
  pPreferenceDialog->Preference.left_list_sort_mode =
    preference_list_sort_mode(PREFERENCE_LIST(pPreferenceDialog->left_list_content));
  pPreferenceDialog->Preference.right_list_sort_mode =
    preference_list_sort_mode(PREFERENCE_LIST(pPreferenceDialog->right_list_content));
  /* file_type_save_file and icon_path don't change and file_type_list
     is automaticaly update it is why we work on a copy of the one use
     by gfile */
     
  /* save file type list on the hard drive */
  if(file_type_save_to_file(pPreferenceDialog->Preference.file_type_list,
    pPreferenceDialog->Preference.file_type_save_file) == FALSE)
  {gfile_error(SAVE_FILE_TYPE); exit(SAVE_FILE_TYPE);}
    
  gtk_signal_emit(GTK_OBJECT(pPreferenceDialog),
    preference_dialog_signals[APPLY_SIGNAL],
    &pPreferenceDialog->Preference);
    
  /*g_print("Apply called.\n");*/
}

static void ok_cb(GtkWidget *widget,PreferenceDialog *pPreferenceDialog)
{
  apply_cb(NULL,pPreferenceDialog);
  cancel_cb(NULL,pPreferenceDialog);
}

static void mode_changed_cb(PreferenceGeneral *pPreferenceGeneral,guint8 new_mode,
  PreferenceDialog *pPreferenceDialog)
{ GtkWidget *label;

  switch(new_mode)
  {
    case MODE_TREE_LIST : 
      gtk_widget_show(pPreferenceDialog->tree_content);      
      gtk_widget_hide(pPreferenceDialog->left_list_content);
      gtk_widget_show(pPreferenceDialog->right_list_content);      
      /*g_print("mode has changed, new is TREE LIST\n");*/
      break;
    case MODE_LIST_LIST : 
      gtk_widget_hide(pPreferenceDialog->tree_content);      
      gtk_widget_show(pPreferenceDialog->left_list_content);
      gtk_widget_show(pPreferenceDialog->right_list_content);      
      /*g_print("mode has changed, new is LIST LIST\n");*/
      break;
    case MODE_LIST : 
      gtk_widget_hide(pPreferenceDialog->tree_content);      
      gtk_widget_hide(pPreferenceDialog->left_list_content);
      gtk_widget_show(pPreferenceDialog->right_list_content);      
      /*g_print("mode has changed, new is LIST\n");*/
      break;
  }
}

/*use as a parameter given by preference_dialog_new for preference_dialog_init*/
PreferenceDialogPref current_PreferenceDialogPref;

static void preference_dialog_init (PreferenceDialog *pPreferenceDialog)
{
  GtkWidget *label;
/*  GtkWidget *notebook;*/
  GtkWidget *general_title;
  GtkWidget *style_title;
  GtkWidget *left_list_title;
  GtkWidget *right_list_title;
  GtkWidget *tree_title;
  GtkWidget *file_type_title;

  /* take parameters in account */
  pPreferenceDialog->Preference.file_type_save_file = current_PreferenceDialogPref.file_type_save_file;
  pPreferenceDialog->Preference.icon_path = current_PreferenceDialogPref.icon_path;
  pPreferenceDialog->Preference.general_mode = current_PreferenceDialogPref.general_mode;
  pPreferenceDialog->Preference.left_list_sort_mode = current_PreferenceDialogPref.left_list_sort_mode;
  pPreferenceDialog->Preference.right_list_sort_mode = current_PreferenceDialogPref.right_list_sort_mode;
  pPreferenceDialog->Preference.owner_list = current_PreferenceDialogPref.owner_list;
  pPreferenceDialog->Preference.group_list = current_PreferenceDialogPref.group_list;    
  pPreferenceDialog->Preference.file_type_list = current_PreferenceDialogPref.file_type_list;    

/*  if (pref_dialog) {
    gtk_widget_destroy(GTK_WIDGET(pref_dialog));
    pref_dialog = NULL;
  }*/
			  
  gtk_window_set_title(GTK_WINDOW(pPreferenceDialog),"Preferences");
  /* add the OK, APPLY and CANCEL buttons */
  gnome_dialog_append_buttons(GNOME_DIALOG(pPreferenceDialog),
    GNOME_STOCK_BUTTON_OK,GNOME_STOCK_BUTTON_APPLY,GNOME_STOCK_BUTTON_CANCEL,NULL);
        
  gtk_signal_connect(GTK_OBJECT(pPreferenceDialog), "destroy",
    GTK_SIGNAL_FUNC(destroy_cb),pPreferenceDialog);
    
  /* boutton Ok */
  gnome_dialog_button_connect(GNOME_DIALOG(pPreferenceDialog),0,
    GTK_SIGNAL_FUNC(ok_cb),pPreferenceDialog);
  /* boutton Apply */
  gnome_dialog_button_connect(GNOME_DIALOG(pPreferenceDialog),1,
    GTK_SIGNAL_FUNC(apply_cb),pPreferenceDialog);
  /* boutton Cancel */
  gnome_dialog_button_connect(GNOME_DIALOG(pPreferenceDialog),2,
    GTK_SIGNAL_FUNC(cancel_cb),pPreferenceDialog);
			       
  gtk_widget_realize (GTK_WIDGET(pPreferenceDialog));

  pPreferenceDialog->notebook = gtk_notebook_new ();

  /* General mode panel */
  general_title = gtk_label_new("General");
  gtk_widget_show(general_title);
  pPreferenceDialog->general_content = preference_general_new(
    GTK_WIDGET(pPreferenceDialog),
    owner_current_user(pPreferenceDialog->Preference.owner_list),
    pPreferenceDialog->Preference.general_mode);
  gtk_signal_connect(GTK_OBJECT(pPreferenceDialog->general_content),"mode_changed",
    GTK_SIGNAL_FUNC(mode_changed_cb),pPreferenceDialog);
  gtk_widget_show(pPreferenceDialog->general_content);
  gtk_notebook_append_page(GTK_NOTEBOOK(pPreferenceDialog->notebook),
    pPreferenceDialog->general_content,general_title);

  /* panneau style */
/*  style_title = gtk_label_new("Style");
  gtk_widget_show(style_title);
  label = gtk_label_new("Style HERE");
  gtk_widget_show(label);
  gtk_notebook_append_page (GTK_NOTEBOOK(notebook), label, style_title);
*/

  /* Preference Tree panel */
  tree_title = gtk_label_new("Tree");
  gtk_widget_show(tree_title);
  pPreferenceDialog->tree_content = gtk_label_new("Tree HERE");
  if(pPreferenceDialog->Preference.general_mode == MODE_TREE_LIST)
    gtk_widget_show(pPreferenceDialog->tree_content);
  gtk_notebook_append_page(GTK_NOTEBOOK(pPreferenceDialog->notebook),
    pPreferenceDialog->tree_content, tree_title);

  /* Left Preference List panel */
  left_list_title = gtk_label_new("Left List");
  gtk_widget_show(left_list_title);
  pPreferenceDialog->left_list_content = preference_list_new(
    pPreferenceDialog->Preference.left_list_sort_mode);
  if(pPreferenceDialog->Preference.general_mode == MODE_LIST_LIST)      
    gtk_widget_show(pPreferenceDialog->left_list_content);
  gtk_notebook_append_page (GTK_NOTEBOOK(pPreferenceDialog->notebook),
    pPreferenceDialog->left_list_content, left_list_title);

  /* Right Preference List panel */
  right_list_title = gtk_label_new("Right List");
  gtk_widget_show(right_list_title);
  pPreferenceDialog->right_list_content = preference_list_new(
    pPreferenceDialog->Preference.right_list_sort_mode);
  gtk_widget_show(pPreferenceDialog->right_list_content);
  gtk_notebook_append_page (GTK_NOTEBOOK(pPreferenceDialog->notebook),
    pPreferenceDialog->right_list_content, right_list_title);

  /* File Type panel */
  file_type_title = gtk_label_new("File type");
  gtk_widget_show(file_type_title);
  pPreferenceDialog->file_type_content = preference_file_type_new(
    GTK_WIDGET(pPreferenceDialog),pPreferenceDialog->Preference.file_type_list,
    pPreferenceDialog->Preference.icon_path);
  gtk_widget_show(pPreferenceDialog->file_type_content);
  gtk_notebook_append_page(GTK_NOTEBOOK(pPreferenceDialog->notebook),
    pPreferenceDialog->file_type_content, file_type_title);

  gtk_widget_set_usize (pPreferenceDialog->notebook, 400, 250);

  gtk_widget_show(pPreferenceDialog->notebook);

  gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(pPreferenceDialog)->vbox), 
    pPreferenceDialog->notebook,TRUE, TRUE, GNOME_PAD);
  
}

/******************************************/
/* Create a new preference dialog widget  */
/* Return : the widget			  */
GtkWidget *preference_dialog_new(PreferenceDialogPref pPreferenceDialogPref)
{
  /* make a copy of the icon path */
  current_PreferenceDialogPref.icon_path = g_malloc(strlen(
    pPreferenceDialogPref.icon_path)+1);
  if(current_PreferenceDialogPref.icon_path == NULL)
  {gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}
  strcpy(current_PreferenceDialogPref.icon_path,pPreferenceDialogPref.icon_path);
  /* make a copy of the file_type_save_file */
  current_PreferenceDialogPref.file_type_save_file = g_malloc(strlen(
    pPreferenceDialogPref.file_type_save_file)+1);
  if(current_PreferenceDialogPref.file_type_save_file == NULL)
  {gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}
  strcpy(current_PreferenceDialogPref.file_type_save_file,
    pPreferenceDialogPref.file_type_save_file);

  current_PreferenceDialogPref.general_mode = pPreferenceDialogPref.general_mode;
  current_PreferenceDialogPref.left_list_sort_mode = pPreferenceDialogPref.left_list_sort_mode;
  current_PreferenceDialogPref.right_list_sort_mode = pPreferenceDialogPref.right_list_sort_mode;
  current_PreferenceDialogPref.owner_list = pPreferenceDialogPref.owner_list;
  current_PreferenceDialogPref.group_list = pPreferenceDialogPref.group_list;    
  /* make a copy of the file type list */
  current_PreferenceDialogPref.file_type_list = file_type_new();
  file_type_append_file_type(current_PreferenceDialogPref.file_type_list,
    pPreferenceDialogPref.file_type_list);
  return( GTK_WIDGET(gtk_type_new( preference_dialog_get_type() )) );
}
/******************************************/

