/****************************************/
/*  PreferenceFileType.c body           */
/*					*/
/*  Description : The file type part of */
/*		  the preference dialog	*/
/*					*/
/*  Written by : Daniel Lacroix	        */
/*  GFile is under the GPL2 licence     */
/****************************************/

#include "GFile.h"

/* Case not taking in account :

   - More than one addedit dialog can be open
   - Worst, a type in edition can be remove

   Solution : block this dialog when an addedit dialog is in use
*/

guint preference_file_type_addedit_get_type ();
GtkWidget *preference_file_type_new(GtkWidget *parent_window,FileType *to_use_file_type,gchar *icon_path);

static void addedit_destroy_cb(PreferenceFileTypeAddedit
  *pPreferenceFileTypeAddedit, PreferenceFileType *pPreferenceFileType);
static void addedit_add_type_cb(PreferenceFileTypeAddedit
  *pPreferenceFileTypeAddedit,file_type *beforetype, file_type *newtype,
  PreferenceFileType *pPreferenceFileType);  
static void addedit_remove_type_cb(PreferenceFileTypeAddedit
  *pPreferenceFileTypeAddedit,file_type *removetype,
  PreferenceFileType *pPreferenceFileType);
static void file_type_add_cb(GtkWidget *widget,PreferenceFileType *pPreferenceFileType);
static void file_type_edit_cb(GtkWidget *widget,PreferenceFileType *pPreferenceFileType);
static void file_type_remove_cb(GtkWidget *widget,PreferenceFileType *pPreferenceFileType);
static void file_type_move_up_cb(GtkWidget *widget,PreferenceFileType *pPreferenceFileType);
static void file_type_move_down_cb(GtkWidget *widget,PreferenceFileType *pPreferenceFileType);

static void preference_file_type_class_init(PreferenceFileTypeClass *pPreferenceFileTypeClass);
static void preference_file_type_init(PreferenceFileType *pPreferenceFileType);

/*************************************************************/
guint preference_file_type_get_type ()
{
  static guint preference_file_type_type = 0;

  if (!preference_file_type_type)
  {
    GtkTypeInfo preference_file_type_info =
    {
       "PreferenceFileType",
       sizeof (PreferenceFileType),
       sizeof (PreferenceFileTypeClass),
       (GtkClassInitFunc)  preference_file_type_class_init,
       (GtkObjectInitFunc) preference_file_type_init,
       (GtkArgSetFunc) NULL,
       (GtkArgGetFunc) NULL
     };
     preference_file_type_type = gtk_type_unique(gtk_hbox_get_type(), &preference_file_type_info);
  }
  return preference_file_type_type;
}
/*************************************************************/

/*************************************************************/
static void preference_file_type_class_init(PreferenceFileTypeClass *class)
{
  /* here we create the new signals if we want */
    
}
/*************************************************************/
    
/********************************************************/
/* Remove the selected type from the clist and from the */
/* file type list if it is not a special type.          */
static void file_type_remove_cb(GtkWidget *widget,PreferenceFileType *pPreferenceFileType)
{ file_type *tmp_type;
  gint vselect_row;

  /* if there is something selected */
  if(pPreferenceFileType->list->selection != NULL)
    vselect_row = (gint)((GList *)(pPreferenceFileType->list->selection)->data);
  else return;

  tmp_type = gtk_clist_get_row_data(pPreferenceFileType->list,vselect_row);
  /* if it is not a special type. special type can only be edit. */
  if(file_type_is_a_special_type(tmp_type) == FALSE) 
  {
    gtk_clist_remove(pPreferenceFileType->list,vselect_row);
    /* remove the file type */
    file_type_remove_type(pPreferenceFileType->to_use_file_type,tmp_type);
  }   
}
/********************************************************/

/*********************************/
static void addedit_destroy_cb(PreferenceFileTypeAddedit
  *pPreferenceFileTypeAddedit, PreferenceFileType *pPreferenceFileType)
{ pPreferenceFileType->addedit = NULL; } 
/*********************************/

/*****************************************************/
static void addedit_add_type_cb(PreferenceFileTypeAddedit
  *pPreferenceFileTypeAddedit,file_type *beforetype, file_type *newtype,
  PreferenceFileType *pPreferenceFileType)
{ gchar *entry[2];
  gint vselect_row;

/*  g_print("addedit_add_type_cb called. newtype string : %s, icon : %s\n",
    newtype->string,newtype->icon);*/
  entry[0] = newtype->string;
  entry[1] = newtype->program;
/*  g_print("addedit_add_type_cb before\n");  */
  if(beforetype == NULL) {
/*    g_print("> beforetype null\n");  */
    vselect_row = gtk_clist_append(pPreferenceFileType->list,entry);
  } else { 
/*    g_print("> beforetype NOT null\n");  */
    vselect_row = gtk_clist_find_row_from_data(pPreferenceFileType->list,
      beforetype);
    vselect_row = gtk_clist_insert(pPreferenceFileType->list,
      vselect_row+1,entry);
  }
/*  g_print("addedit_add_type_cb after\n");    */
  
  gtk_clist_set_pixtext(pPreferenceFileType->list, vselect_row,
  		        0, entry[0],5,
			newtype->pixmap,
			newtype->mask);	    			
  /* attach the datas for making a relation between this list and the */
  /* existing file types */
  gtk_clist_set_row_data(pPreferenceFileType->list,vselect_row,newtype);
/*  g_print("addedit_add_type_cb finish\n");  */
}
/*****************************************************/

/******************************************************************/
static void addedit_remove_type_cb(PreferenceFileTypeAddedit
  *pPreferenceFileTypeAddedit,file_type *removetype,
  PreferenceFileType *pPreferenceFileType)
{ gint vselect_row;

  g_print("addedit_remove_type_cb called\n");
  vselect_row = gtk_clist_find_row_from_data(pPreferenceFileType->list,
    removetype);
  gtk_clist_remove(pPreferenceFileType->list,vselect_row);
}
/******************************************************************/

/**************************/
/* add a type to the list */
static void file_type_add_cb(GtkWidget *widget,PreferenceFileType *pPreferenceFileType)
{ file_type *beforetype;
  gint vselect_row;

/* NOT FINISH, we must check if an addedit dialog box is not in use */

  /* if there is something selected, add after that type */
  if(pPreferenceFileType->list->selection != NULL){
    vselect_row = (gint)((GList *)(pPreferenceFileType->list->selection)->data);
    beforetype = gtk_clist_get_row_data(pPreferenceFileType->list,vselect_row);
  /* else add at the end */
  } else beforetype = NULL;

  pPreferenceFileType->addedit = preference_file_type_addedit_new
    (pPreferenceFileType->to_use_file_type,beforetype,NULL,
    pPreferenceFileType->icon_path);

  gtk_signal_connect (GTK_OBJECT(pPreferenceFileType->addedit), "add_type",
                      GTK_SIGNAL_FUNC (addedit_add_type_cb),
                      pPreferenceFileType);  

  gtk_signal_connect (GTK_OBJECT(pPreferenceFileType->addedit), "remove_type",
                      GTK_SIGNAL_FUNC (addedit_remove_type_cb),
                      pPreferenceFileType);  

  gtk_signal_connect (GTK_OBJECT(pPreferenceFileType->addedit), "destroy",
                      GTK_SIGNAL_FUNC (addedit_destroy_cb),
                      pPreferenceFileType);

  gtk_widget_show(pPreferenceFileType->addedit);
}
/**************************/

/***************************/
/* edit a type to the list */
void file_type_edit_cb(GtkWidget *widget,PreferenceFileType *pPreferenceFileType)
{ file_type *lasttype;
  gint vselect_row;

/* NOT FINISH, we must check if an addedit dialog box is not in use */

  /* if there is something selected, add after that type */
  if(pPreferenceFileType->list->selection != NULL){
    vselect_row = (gint)((GList *)(pPreferenceFileType->list->selection)->data);
    lasttype = gtk_clist_get_row_data(pPreferenceFileType->list,vselect_row);
  /* else add at the end */
  } else return;

  pPreferenceFileType->addedit = preference_file_type_addedit_new
    (pPreferenceFileType->to_use_file_type,lasttype,lasttype,
    pPreferenceFileType->icon_path);

  gtk_signal_connect (GTK_OBJECT(pPreferenceFileType->addedit), "add_type",
                      GTK_SIGNAL_FUNC (addedit_add_type_cb),
                      pPreferenceFileType);  

  gtk_signal_connect (GTK_OBJECT(pPreferenceFileType->addedit), "remove_type",
                      GTK_SIGNAL_FUNC (addedit_remove_type_cb),
                      pPreferenceFileType);  

  gtk_signal_connect (GTK_OBJECT(pPreferenceFileType->addedit), "destroy",
                      GTK_SIGNAL_FUNC (addedit_destroy_cb),
                      pPreferenceFileType);

  gtk_widget_show(pPreferenceFileType->addedit);
}
/**************************/

/***************************/
/* move the select type up */
void file_type_move_up_cb(GtkWidget *widget,PreferenceFileType *pPreferenceFileType)
{ gint vselect_row;

  /* if there is something selected */
  if(pPreferenceFileType->list->selection != NULL)
    vselect_row = (gint)((GList *)(pPreferenceFileType->list->selection)->data);
  else return;  
  /*g_print("row to move up : %d.\n");*/
  if(vselect_row > 0)
  {
    gtk_clist_freeze (pPreferenceFileType->list);
    gtk_clist_unselect_row (pPreferenceFileType->list,vselect_row,0);
    gtk_clist_swap_rows(pPreferenceFileType->list,vselect_row-1,vselect_row);
    gtk_clist_select_row (pPreferenceFileType->list,vselect_row-1,0);
    gtk_clist_thaw   (pPreferenceFileType->list);
    /* move up in the file type too */
    /* swap in the file_type list */
    file_type_swap_type(pPreferenceFileType->to_use_file_type,
      gtk_clist_get_row_data(pPreferenceFileType->list,vselect_row),
      gtk_clist_get_row_data(pPreferenceFileType->list,vselect_row-1));
  }  
}
/***************************/

/*****************************/
/* move the select type down */
void file_type_move_down_cb(GtkWidget *widget,PreferenceFileType *pPreferenceFileType)
{ gint vselect_row;

  /* if there is something selected */
  if(pPreferenceFileType->list->selection != NULL)
    vselect_row = (gint)((GList *)(pPreferenceFileType->list->selection)->data);
  else return;  

  if(vselect_row < (pPreferenceFileType->list->rows - 1) )
  {
    gtk_clist_freeze (pPreferenceFileType->list);
    gtk_clist_unselect_row (pPreferenceFileType->list,vselect_row,0);
    gtk_clist_swap_rows(pPreferenceFileType->list,vselect_row,vselect_row+1);
    gtk_clist_select_row (pPreferenceFileType->list,vselect_row+1,0);
    gtk_clist_thaw   (pPreferenceFileType->list);    
    /* move down in the file type too */
    /* swap in the file_type list */
    file_type_swap_type(pPreferenceFileType->to_use_file_type,
      gtk_clist_get_row_data(pPreferenceFileType->list,vselect_row),
      gtk_clist_get_row_data(pPreferenceFileType->list,vselect_row+1));
  }  
}
/*****************************/

/************************************************/
/* Create a widget for the preference file type */
/* Return : the widget			        */

static char * up_xpm[] = {
"14 11 5 1",
" 	c None",
".	c #C6C6C6",
"+	c #969696",
"@	c #000000",
"#	c #FFFFFF",
"      .+      ",
"     .@@+     ",
"    .@@@@+    ",
"   .@@@@@@+   ",
"  .@@@@@@@@+  ",
" .@@@@@@@@@@+ ",
".@@@@@@@@@@@@.",
".@@@@@@@@@@@@#",
".@@@@@@@@@@@@#",
".@@@@@@@@@@@@#",
".#############"};

static char * down_xpm[] = {
"14 11 5 1",
" 	c None",
".	c #C6C6C6",
"+	c #969696",
"@	c #000000",
"#	c #FFFFFF",
".++++++++++++.",
".@@@@@@@@@@@@#",
".@@@@@@@@@@@@#",
".@@@@@@@@@@@@#",
".@@@@@@@@@@@@#",
" .@@@@@@@@@@# ",
"  .@@@@@@@@#  ",
"   .@@@@@@#   ",
"    .@@@@#    ",
"     .@@#     ",
"      .#      "};


/* use as parameters given by preference_file_type_new to 
   preference_file_init */
static FileType  *current_to_use_file_type;
static gchar     *current_icon_path;
static GtkWidget *current_parent_window;

void preference_file_type_init(PreferenceFileType *pPreferenceFileType)
{
  GtkWidget *buttons;
  GtkWidget *edit;
  GtkWidget *add;
  GtkWidget *remove;
  GtkWidget *move_up;
  GtkWidget *move_down;
  GtkWidget *scrolled_widget;
  gchar     *titles[2] = {"String","Program"};
  file_type *tmptype = NULL;
  /* __ FOR THE UP PIXMAP ____*/
  GdkPixmap *updown_pixmap;
  GdkBitmap *updown_mask;
  GtkWidget *updown_pixmap_widget;

  /* take in account parameters */
  pPreferenceFileType->to_use_file_type = current_to_use_file_type;
  pPreferenceFileType->icon_path = current_icon_path;

  gtk_box_set_homogeneous(GTK_BOX(pPreferenceFileType),FALSE);
  gtk_box_set_spacing(GTK_BOX(pPreferenceFileType),0);

  buttons = gtk_vbox_new (FALSE, 0);

  edit   = gtk_button_new_with_label ("Edit");
  gtk_widget_show(edit);
  gtk_box_pack_start (GTK_BOX (buttons), edit, FALSE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (edit), "clicked",
                      GTK_SIGNAL_FUNC (file_type_edit_cb),
                      pPreferenceFileType);

  add    = gtk_button_new_with_label ("Add");
  gtk_widget_show(add);
  gtk_box_pack_start (GTK_BOX (buttons), add, FALSE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (add), "clicked",
                      GTK_SIGNAL_FUNC (file_type_add_cb),
                      pPreferenceFileType);


  remove = gtk_button_new_with_label ("Remove");
  gtk_widget_show(remove);
  gtk_box_pack_start (GTK_BOX (buttons), remove, FALSE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (remove), "clicked",
                      GTK_SIGNAL_FUNC (file_type_remove_cb),
                      pPreferenceFileType);
  /*** create the up button ***/
  updown_pixmap = gdk_pixmap_create_from_xpm_d (
    current_parent_window->window,&updown_mask,NULL,up_xpm);
		    
  if(updown_pixmap == NULL) {gfile_error(INTERNAL_PIXMAP); exit(INTERNAL_PIXMAP);}
  /* create a widget with the pixmap */
  updown_pixmap_widget = gtk_pixmap_new(updown_pixmap,updown_mask);
  /* free the pixmap */
  gdk_pixmap_unref(updown_pixmap); gdk_bitmap_unref(updown_mask);

  move_up = gtk_button_new ();
  gtk_container_add(GTK_CONTAINER(move_up),updown_pixmap_widget);
  gtk_widget_show(updown_pixmap_widget);
  gtk_widget_show(move_up);
  gtk_box_pack_start (GTK_BOX (buttons), move_up, FALSE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (move_up), "clicked",
                      GTK_SIGNAL_FUNC (file_type_move_up_cb),
                      pPreferenceFileType);
		      
  /*** create the down button ***/
  updown_pixmap = gdk_pixmap_create_from_xpm_d (
    current_parent_window->window,&updown_mask,NULL,down_xpm);
		    
  if(updown_pixmap == NULL) {gfile_error(INTERNAL_PIXMAP); exit(INTERNAL_PIXMAP);}
  /* create a widget with the pixmap */
  updown_pixmap_widget = gtk_pixmap_new(updown_pixmap,updown_mask);
  /* free the pixmap */
  gdk_pixmap_unref(updown_pixmap); gdk_bitmap_unref(updown_mask);

  move_down = gtk_button_new();
  gtk_container_add(GTK_CONTAINER(move_down),updown_pixmap_widget);
  gtk_widget_show(updown_pixmap_widget);
  gtk_widget_show(move_down);
  gtk_box_pack_start (GTK_BOX (buttons), move_down, FALSE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (move_down), "clicked",
                      GTK_SIGNAL_FUNC (file_type_move_down_cb),
                      pPreferenceFileType);

  pPreferenceFileType->list = GTK_CLIST(gtk_clist_new_with_titles(2,titles));

  gtk_clist_set_row_height (pPreferenceFileType->list, 18);
  gtk_clist_set_column_width(pPreferenceFileType->list,0,80);

  gtk_widget_show (GTK_WIDGET(pPreferenceFileType->list));

  /* we fill with all the existing file types */
  file_type_init(pPreferenceFileType->to_use_file_type);  
  while(file_type_is_end(pPreferenceFileType->to_use_file_type) == FALSE)
  {
    /* add at the end */
    addedit_add_type_cb(NULL,NULL, file_type_current(
      pPreferenceFileType->to_use_file_type),pPreferenceFileType);

    file_type_next(pPreferenceFileType->to_use_file_type);
  }

  /* define the scrolled policy */
  scrolled_widget = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_widget),
    GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
  gtk_widget_show(scrolled_widget);
  gtk_container_add(GTK_CONTAINER(scrolled_widget), GTK_WIDGET(
    pPreferenceFileType->list));
    
  gtk_box_pack_start (GTK_BOX(pPreferenceFileType),scrolled_widget,TRUE,TRUE,0);
  gtk_box_pack_start (GTK_BOX(pPreferenceFileType), buttons, FALSE, TRUE, 0);

  gtk_container_border_width (GTK_CONTAINER(pPreferenceFileType), 4);
  gtk_container_border_width (GTK_CONTAINER(buttons), 4);

  gtk_widget_show(buttons);

}
/************************************************/

/***************************************/
/* called when the widget is destroyed */
void preference_file_type_destroy_cb(GtkWidget *GtkWidget,PreferenceFileType *pPreferenceFileType)
{
  if(pPreferenceFileType->icon_path != NULL) 
    g_free(pPreferenceFileType->icon_path);
}
/***************************************/

/*****************************************************/
/* Create a new widget for the preference file type  */
/* icon_path = the directory where we must find icons*/
/* icon_path MUST be ended by a '/'.                 */
/* Return : the widget			             */
GtkWidget *preference_file_type_new(GtkWidget *parent_window,FileType *to_use_file_type,gchar *icon_path)
{ PreferenceFileType *vPreferenceFileType;

  current_parent_window = parent_window;
  current_to_use_file_type = to_use_file_type;

  if(icon_path == NULL)
    current_icon_path == NULL;
  else {
    current_icon_path = g_malloc(strlen(icon_path)+1);
    if(current_icon_path == NULL)
    {gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}
    strcpy(current_icon_path,icon_path);
  }

  vPreferenceFileType = PREFERENCE_FILE_TYPE(
    gtk_type_new( preference_file_type_get_type() ));
    
  gtk_signal_connect (GTK_OBJECT(vPreferenceFileType), "destroy",
                      GTK_SIGNAL_FUNC (preference_file_type_destroy_cb),
                      vPreferenceFileType);

  return GTK_WIDGET (vPreferenceFileType);
}
/*****************************************************/
