/****************************************/
/*  PreferenceFileTypeAddEdit.c body    */
/*					*/
/*  Description : Bouton Add and Edit of*/
/*	          the file type part of */
/*		  the preference dialog	*/
/*					*/
/*  Written by : Daniel Lacroix	        */
/*  GFile is under the GPL2 licence     */
/****************************************/

#include "GFile.h"

guint preference_file_type_addedit_get_type ();

void preference_file_type_addedit_ok_cb(GtkWidget *widget,
  PreferenceFileTypeAddedit *pPreferenceFileTypeAddedit);

void preference_file_type_addedit_cancel_cb(GtkWidget *widget,
  PreferenceFileTypeAddedit *pPreferenceFileTypeAddedit);

GtkWidget *preference_file_type_addedit_new(FileType *pFileType,
  file_type *beforetype, file_type *lasttype, gchar *icon_path);  

static void preference_file_type_addedit_class_init(
  PreferenceFileTypeAddeditClass *pPreferenceFileTypeAddeditClass);

static void preference_file_type_addedit_init(
  PreferenceFileTypeAddedit *pPreferenceFileTypeAddedit);

/*************************************************************/
guint preference_file_type_addedit_get_type ()
{
  static guint preference_file_type_addedit_type = 0;

  if (!preference_file_type_addedit_type)
  {
    GtkTypeInfo preference_file_type_addedit_info =
    {
       "PreferenceFileTypeAddedit",
       sizeof (PreferenceFileTypeAddedit),
       sizeof (PreferenceFileTypeAddeditClass),
       (GtkClassInitFunc)  preference_file_type_addedit_class_init,
       (GtkObjectInitFunc) preference_file_type_addedit_init,
       (GtkArgSetFunc) NULL,
       (GtkArgGetFunc) NULL
     };
     preference_file_type_addedit_type = gtk_type_unique(gnome_dialog_get_type(), &preference_file_type_addedit_info);
  }
  return preference_file_type_addedit_type;
}
/*************************************************************/

/*************************************************************/
enum { ADD_TYPE_SIGNAL, REMOVE_TYPE_SIGNAL, LAST_SIGNAL};
static gint preference_file_type_addedit_signals[LAST_SIGNAL] = { 0 };

static void one_file_type_marshal(GtkObject *object, GtkSignalFunc func, gpointer func_data, GtkArg *args) {
    typedef void (*one_file_type_marshal_func)(PreferenceFileTypeAddedit *,
      file_type *, gpointer);
    one_file_type_marshal_func rfunc = (one_file_type_marshal_func) func;
    (*rfunc) (PREFERENCE_FILE_TYPE_ADDEDIT(object),
	      GTK_VALUE_POINTER(args[0]), func_data);
}

static void two_file_type_marshal(GtkObject *object, GtkSignalFunc func, gpointer func_data, GtkArg *args) {
    typedef void (*two_file_type_marshal_func)(PreferenceFileTypeAddedit *,
      file_type *, file_type *, gpointer);
    two_file_type_marshal_func rfunc = (two_file_type_marshal_func) func;
    (*rfunc) (PREFERENCE_FILE_TYPE_ADDEDIT(object),
	      GTK_VALUE_POINTER(args[0]),GTK_VALUE_POINTER(args[1]), func_data );
}

static void preference_file_type_addedit_class_init(PreferenceFileTypeAddeditClass *class)
{
  /* here we create the new signals if we want */
  GtkObjectClass *object_class;
  object_class = (GtkObjectClass*) class;

  preference_file_type_addedit_signals[ADD_TYPE_SIGNAL] = gtk_signal_new
            ("add_type",
            GTK_RUN_FIRST,object_class->type,
            GTK_SIGNAL_OFFSET (PreferenceFileTypeAddeditClass,add_type),
            two_file_type_marshal, GTK_TYPE_NONE, 2,GTK_TYPE_POINTER,
	    GTK_TYPE_POINTER);    

  preference_file_type_addedit_signals[REMOVE_TYPE_SIGNAL] = gtk_signal_new
            ("remove_type",
            GTK_RUN_FIRST,object_class->type,
            GTK_SIGNAL_OFFSET (PreferenceFileTypeAddeditClass,remove_type),
            one_file_type_marshal, GTK_TYPE_NONE, 1,GTK_TYPE_POINTER);    
	    
   gtk_object_class_add_signals(object_class, 
     preference_file_type_addedit_signals, LAST_SIGNAL);
   class->add_type = NULL;  class->remove_type = NULL;	    
}
/*************************************************************/

/* use as parameter given by preference_file_type_addedit_new
  to preference_file_type_addedit_init */
static file_type *current_preference_file_type_addedit_beforetype;
static file_type *current_preference_file_type_addedit_lasttype;
static FileType  *current_preference_file_type_addedit_to_use_file_type;
static gchar     *current_preference_file_type_addedit_icon_path;

static gchar *suppress_start(gchar *start,gchar *string)
{ guint16 pos = 0;
  guint16 end_pos;
  /*g_print("Start : %s, String : %s\n",start,string);*/

  while ((start[pos]) && (string[pos]) && (start[pos] == string[pos])){pos++;}
  /* if we are not at the end of start, that mean that string doesn't
     start like the "start" string. */  
  if(start[pos] != 0) {gfile_error(ICON_NOT_IN_PATH); exit(ICON_NOT_IN_PATH);}
  /* else, we check if there is not '/' in &string[pos] */
  end_pos = pos;
  while(string[end_pos])
  { if(string[end_pos++] == '/') {gfile_error(ICON_NOT_IN_PATH); exit(ICON_NOT_IN_PATH);} 
  }
  return(&string[pos]);
}

/*******************************************************************/
/* called each time the icon file is changed using the icon button */
void icon_change_cb(IconList *pIconList, gchar *icon_file,
  PreferenceFileTypeAddedit *pPreferenceFileTypeAddedit)
{
  if(icon_file != NULL)
    gtk_entry_set_text(GTK_ENTRY(pPreferenceFileTypeAddedit->icon),
      suppress_start(pPreferenceFileTypeAddedit->icon_path,icon_file));
  else /* no icon selected */
    gtk_entry_set_text(GTK_ENTRY(pPreferenceFileTypeAddedit->icon),"");
}
/*******************************************************************/

/****************************************/
/* Create a new dialog for add or edit. */
static void preference_file_type_addedit_init(
  PreferenceFileTypeAddedit *pPreferenceFileTypeAddedit)
{
  GtkWidget *h1;
  GtkWidget *v1;
  GtkWidget *label;
  gboolean protected_type = FALSE;
  /* the button where we can see the icon and change it */
  GtkWidget *icon_button;

  file_type *tmptype;

  /* use parameters */
  pPreferenceFileTypeAddedit->beforetype =
    current_preference_file_type_addedit_beforetype;  
  pPreferenceFileTypeAddedit->lasttype = 
    current_preference_file_type_addedit_lasttype;
  pPreferenceFileTypeAddedit->to_use_file_type = 
    current_preference_file_type_addedit_to_use_file_type;
  pPreferenceFileTypeAddedit->icon_path = 
    current_preference_file_type_addedit_icon_path;

  /* prevent SPECIAL type to be totally change */
  if(pPreferenceFileTypeAddedit->lasttype)
    if(file_type_is_a_special_type(pPreferenceFileTypeAddedit->lasttype)) /* if it is a special type */
      protected_type = TRUE;

  if(pPreferenceFileTypeAddedit->lasttype == NULL)
    gtk_window_set_title(GTK_WINDOW(pPreferenceFileTypeAddedit),"New file type");
  else
    gtk_window_set_title(GTK_WINDOW(pPreferenceFileTypeAddedit),"Edit file type");

  /* add the OK and CANCEL buttons */
  gnome_dialog_append_buttons(GNOME_DIALOG(pPreferenceFileTypeAddedit),
    GNOME_STOCK_BUTTON_OK,GNOME_STOCK_BUTTON_CANCEL,NULL);

  /* Cancel button */
  gnome_dialog_button_connect(GNOME_DIALOG(pPreferenceFileTypeAddedit),1,
    GTK_SIGNAL_FUNC(preference_file_type_addedit_cancel_cb),pPreferenceFileTypeAddedit);
  /* Ok button */
  gnome_dialog_button_connect(GNOME_DIALOG(pPreferenceFileTypeAddedit),0,
    GTK_SIGNAL_FUNC(preference_file_type_addedit_ok_cb),pPreferenceFileTypeAddedit);


  h1 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show(h1);
  gtk_container_border_width (GTK_CONTAINER(h1), 4);

  /* String */
  v1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show(v1);
  label=gtk_label_new("String");
  gtk_widget_show(label);
  gtk_box_pack_start (GTK_BOX (v1), label, TRUE, FALSE, 0);
/*  gtk_widget_set_usize(string,100,20);*/
  pPreferenceFileTypeAddedit->string = gtk_entry_new_with_max_length(FILE_TYPE_STRING_MAX);
  if(protected_type)
  {
    gtk_widget_set_sensitive(GTK_WIDGET(pPreferenceFileTypeAddedit->string),0);
    gtk_widget_set_sensitive(GTK_WIDGET(label),0);
  }
  gtk_widget_show(pPreferenceFileTypeAddedit->string);
  gtk_box_pack_start (GTK_BOX (v1), pPreferenceFileTypeAddedit->string, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (h1), v1, FALSE, FALSE, 0);

  /* Start */
  v1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show(v1);
  label=gtk_label_new("Start");
  gtk_widget_show(label);
  gtk_box_pack_start (GTK_BOX (v1), label, TRUE, TRUE, 0);
  pPreferenceFileTypeAddedit->check_button_1 = gtk_radio_button_new(NULL);
  if(protected_type)
  {
    gtk_widget_set_sensitive(GTK_WIDGET(pPreferenceFileTypeAddedit->check_button_1),0);
    gtk_widget_set_sensitive(GTK_WIDGET(label),0);
  }
  gtk_widget_show(pPreferenceFileTypeAddedit->check_button_1);
  gtk_box_pack_start (GTK_BOX (v1), pPreferenceFileTypeAddedit->check_button_1, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (h1), v1, TRUE, TRUE, 0);

  /* End */
  v1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show(v1);
  label=gtk_label_new("End");
  gtk_widget_show(label);
  gtk_box_pack_start (GTK_BOX (v1), label, TRUE, TRUE, 0);
  pPreferenceFileTypeAddedit->check_button_2 = gtk_radio_button_new(
    gtk_radio_button_group(GTK_RADIO_BUTTON (pPreferenceFileTypeAddedit->check_button_1)));
  if(protected_type)
  {
    gtk_widget_set_sensitive(GTK_WIDGET(pPreferenceFileTypeAddedit->check_button_2),0);
    gtk_widget_set_sensitive(GTK_WIDGET(label),0);
  }
  gtk_widget_show(pPreferenceFileTypeAddedit->check_button_2);
  gtk_box_pack_start (GTK_BOX (v1), pPreferenceFileTypeAddedit->check_button_2, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (h1), v1, TRUE, TRUE, 0);

  /* Exactly */
  v1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show(v1);
  label=gtk_label_new("Exactly");
  gtk_widget_show(label);
  gtk_box_pack_start (GTK_BOX (v1), label, TRUE, TRUE, 0);
  pPreferenceFileTypeAddedit->check_button_3 = gtk_radio_button_new(
    gtk_radio_button_group(GTK_RADIO_BUTTON (pPreferenceFileTypeAddedit->check_button_1)));
  if(protected_type)
  {
    gtk_widget_set_sensitive(GTK_WIDGET(pPreferenceFileTypeAddedit->check_button_3),0);
    gtk_widget_set_sensitive(GTK_WIDGET(label),0);
  }
  gtk_widget_show(pPreferenceFileTypeAddedit->check_button_3);
  gtk_box_pack_start (GTK_BOX (v1), pPreferenceFileTypeAddedit->check_button_3, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (h1), v1, TRUE, TRUE, 0);

  /* Is a link */
  v1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show(v1);
  label=gtk_label_new("Link");
  gtk_widget_show(label);
  gtk_box_pack_start (GTK_BOX (v1), label, TRUE, TRUE, 0);
  pPreferenceFileTypeAddedit->check_button_4 = gtk_check_button_new();
  if(protected_type)
  {
    gtk_widget_set_sensitive(GTK_WIDGET(pPreferenceFileTypeAddedit->check_button_4),0);
    gtk_widget_set_sensitive(GTK_WIDGET(label),0);
  }  
  gtk_widget_show(pPreferenceFileTypeAddedit->check_button_4);
  gtk_box_pack_start (GTK_BOX (v1), pPreferenceFileTypeAddedit->check_button_4, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (h1), v1, TRUE, TRUE, 0);

  gtk_box_pack_start (GTK_BOX (GNOME_DIALOG(pPreferenceFileTypeAddedit)->vbox), h1, FALSE, TRUE, 0);

  label = gtk_hseparator_new();
  gtk_widget_show(label);
  gtk_box_pack_start (GTK_BOX(GNOME_DIALOG(pPreferenceFileTypeAddedit)->vbox), label, FALSE, TRUE, 0);
  
  /* Program */
  h1 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show(h1);
  gtk_container_border_width (GTK_CONTAINER(h1), 4);
  label=gtk_label_new("Program");
  gtk_widget_show(label);
  gtk_box_pack_start (GTK_BOX (h1), label, TRUE, TRUE, 0);
/*  gtk_widget_set_usize(program,150,20);*/
  pPreferenceFileTypeAddedit->program = gtk_entry_new_with_max_length(FILE_TYPE_PROGRAM_MAX);
  gtk_entry_set_editable(GTK_ENTRY(pPreferenceFileTypeAddedit->program), TRUE);
  gtk_widget_show(pPreferenceFileTypeAddedit->program);
  gtk_box_pack_start (GTK_BOX (h1), pPreferenceFileTypeAddedit->program, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX(GNOME_DIALOG(pPreferenceFileTypeAddedit)->vbox), h1, FALSE, TRUE, 0);

  label = gtk_hseparator_new();
  gtk_widget_show(label);
  gtk_box_pack_start (GTK_BOX(GNOME_DIALOG(pPreferenceFileTypeAddedit)->vbox), label, FALSE, TRUE, 0);

  /* Icon */
  h1 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show(h1);
  gtk_container_border_width (GTK_CONTAINER(h1), 4);
  label=gtk_label_new("Icon");
  gtk_widget_show(label);
  gtk_box_pack_start (GTK_BOX (h1), label, TRUE, TRUE, 0);
/*  gtk_widget_set_usize(icon,150,20);*/
  pPreferenceFileTypeAddedit->icon = gtk_entry_new_with_max_length(FILE_TYPE_ICON_MAX);
  gtk_entry_set_editable(GTK_ENTRY(pPreferenceFileTypeAddedit->icon), TRUE);
  gtk_widget_show(pPreferenceFileTypeAddedit->icon);
  gtk_box_pack_start (GTK_BOX (h1), pPreferenceFileTypeAddedit->icon, FALSE, FALSE, 0);

  if((pPreferenceFileTypeAddedit->lasttype == NULL) ||
     (pPreferenceFileTypeAddedit->lasttype->icon[0] == 0))
    icon_button = icon_list_new(NULL,pPreferenceFileTypeAddedit->icon_path);
  else  
    icon_button = icon_list_new(pPreferenceFileTypeAddedit->lasttype->icon,
      pPreferenceFileTypeAddedit->icon_path);
  gtk_signal_connect(GTK_OBJECT (icon_button), "icon_change",
    GTK_SIGNAL_FUNC (icon_change_cb),pPreferenceFileTypeAddedit);
  gtk_widget_show(icon_button);
  gtk_box_pack_start (GTK_BOX (h1), icon_button, FALSE, FALSE, 4);

  gtk_box_pack_start (GTK_BOX(GNOME_DIALOG(pPreferenceFileTypeAddedit)->vbox), h1, FALSE, TRUE, 0);

  /* fill with the old values if we are in edit mode */
  if(pPreferenceFileTypeAddedit->lasttype){
    gtk_entry_set_text(GTK_ENTRY(pPreferenceFileTypeAddedit->string),pPreferenceFileTypeAddedit->lasttype->string);
    gtk_entry_set_text(GTK_ENTRY(pPreferenceFileTypeAddedit->program),pPreferenceFileTypeAddedit->lasttype->program);

    if(pPreferenceFileTypeAddedit->icon_path == NULL)
      gtk_entry_set_text(GTK_ENTRY(pPreferenceFileTypeAddedit->icon),pPreferenceFileTypeAddedit->lasttype->icon);
    else
      /* if it is not an internal type */
      if(pPreferenceFileTypeAddedit->lasttype->icon[0] != 0)
        gtk_entry_set_text(GTK_ENTRY(pPreferenceFileTypeAddedit->icon),
          suppress_start(pPreferenceFileTypeAddedit->icon_path,pPreferenceFileTypeAddedit->lasttype->icon));
      else
        gtk_entry_set_text(GTK_ENTRY(pPreferenceFileTypeAddedit->icon),"");

    if(pPreferenceFileTypeAddedit->lasttype->is_link) gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(pPreferenceFileTypeAddedit->check_button_4),1);
    else gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(pPreferenceFileTypeAddedit->check_button_4),0);
    switch(pPreferenceFileTypeAddedit->lasttype->pos){
    	case 0 : /* exactly */
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(pPreferenceFileTypeAddedit->check_button_1),0);
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(pPreferenceFileTypeAddedit->check_button_2),0);
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(pPreferenceFileTypeAddedit->check_button_3),1);
	break;
	case 1 : /* start */
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(pPreferenceFileTypeAddedit->check_button_1),1);
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(pPreferenceFileTypeAddedit->check_button_2),0);
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(pPreferenceFileTypeAddedit->check_button_3),0);
	break;
	case 2 : /* end */
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(pPreferenceFileTypeAddedit->check_button_1),0);
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(pPreferenceFileTypeAddedit->check_button_2),1);
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(pPreferenceFileTypeAddedit->check_button_3),0);
	break;
    }
  }
}
/****************************************/

/* This is called when the dialog is destroyed */
static void preference_file_type_addedit_destroy_cb(GtkWidget *Widget,
  PreferenceFileTypeAddedit *pPreferenceFileTypeAddedit)
{
  if(pPreferenceFileTypeAddedit->icon_path != NULL) 
    g_free(pPreferenceFileTypeAddedit->icon_path);
}

/***************************************************************/
/* Create a dialog for add or edit. If lasttype = NULL then it */
/* is in add mode else, in edit mode.                          */
/* If icon_path = NULL then icons are given by full path else  */
/* they are given using "icon_path" path. icon_path must be end*/
/* by a /. If beforetype = NULL, insert at the end.            */
GtkWidget *preference_file_type_addedit_new(FileType *pFileType,
  file_type *beforetype, file_type *lasttype, gchar *icon_path)
{ PreferenceFileTypeAddedit * vPreferenceFileTypeAddedit;

  current_preference_file_type_addedit_beforetype = beforetype;
  current_preference_file_type_addedit_lasttype = lasttype;
  current_preference_file_type_addedit_to_use_file_type = pFileType;
  if(icon_path == NULL)
    current_preference_file_type_addedit_icon_path == NULL;
  else {
    current_preference_file_type_addedit_icon_path = g_malloc(strlen(icon_path)+1);
    if(current_preference_file_type_addedit_icon_path == NULL)
    {gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}
    strcpy(current_preference_file_type_addedit_icon_path,icon_path);
  }  
  vPreferenceFileTypeAddedit = PREFERENCE_FILE_TYPE_ADDEDIT(
    gtk_type_new( preference_file_type_addedit_get_type() ));

  gtk_signal_connect (GTK_OBJECT(vPreferenceFileTypeAddedit), "destroy",
                      GTK_SIGNAL_FUNC (preference_file_type_addedit_destroy_cb),
                      vPreferenceFileTypeAddedit);
  
  return GTK_WIDGET (vPreferenceFileTypeAddedit);
}
/***************************************************************/

static void close_dialog_cb(GtkWidget *Widget,GnomeDialog *pGnomeDialog)
{gnome_dialog_close(pGnomeDialog);}

static void preference_file_type_addedit_icon_message(gchar *icon_name)
{ GtkWidget *message_dialog,*label;
  gchar *tmp_name;

  tmp_name = (gchar *)g_malloc(27+strlen(icon_name)+1);
  sprintf(tmp_name,"The icon : %s,\n can't be found",icon_name);

  label = gtk_label_new(tmp_name);
  g_free(tmp_name);
  gtk_widget_show(label);
  message_dialog = gnome_dialog_new("Message",GNOME_STOCK_BUTTON_OK,NULL);

  gnome_dialog_button_connect(GNOME_DIALOG(message_dialog),0,
    GTK_SIGNAL_FUNC(close_dialog_cb),message_dialog);

  gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(message_dialog)->vbox), label, TRUE, TRUE, GNOME_PAD);
  
  gtk_widget_show(message_dialog);
}

/*************************************/
/* Button ok pressed in the Add mode */
void preference_file_type_addedit_ok_cb(GtkWidget *widget,
  PreferenceFileTypeAddedit *pPreferenceFileTypeAddedit)
{
  gchar     *entry[2];
  file_type *new_type;
  gchar     string_pos;
  gboolean  is_link;
  gchar     *icon;
  

  if(GTK_TOGGLE_BUTTON (pPreferenceFileTypeAddedit->check_button_1)->active)
    string_pos=1;    /* start with */
  else if(GTK_TOGGLE_BUTTON (pPreferenceFileTypeAddedit->check_button_2)->active)
    string_pos=2;    /* end with */
  else
    string_pos=0;    /* exactly */
    
  if(GTK_TOGGLE_BUTTON (pPreferenceFileTypeAddedit->check_button_4)->active)
    is_link = 1; 
  else
    is_link = 0;  

  if(pPreferenceFileTypeAddedit->icon_path == NULL)
    icon = gtk_entry_get_text(GTK_ENTRY(pPreferenceFileTypeAddedit->icon));
  else {
    icon = g_malloc(strlen(gtk_entry_get_text(GTK_ENTRY(pPreferenceFileTypeAddedit->icon)))+
                    strlen(pPreferenceFileTypeAddedit->icon_path)+1);
    if(icon == NULL){gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}
    strcpy(icon,pPreferenceFileTypeAddedit->icon_path);
    strcat(icon,gtk_entry_get_text(GTK_ENTRY(pPreferenceFileTypeAddedit->icon)));
  }		    

  /*g_print("ok pressed, icon wanted is : %s\n",icon);*/

  /* we create the new type */
  new_type = file_type_insert_type(pPreferenceFileTypeAddedit->to_use_file_type,
    pPreferenceFileTypeAddedit->beforetype,
    gtk_entry_get_text(GTK_ENTRY(pPreferenceFileTypeAddedit->string)),
    string_pos,is_link,
    gtk_entry_get_text(GTK_ENTRY(pPreferenceFileTypeAddedit->program)),
    icon);

  if(new_type != NULL)
  {
    /* alerte the parent that we have inserted a new type */
    gtk_signal_emit(GTK_OBJECT(pPreferenceFileTypeAddedit),
      preference_file_type_addedit_signals[ADD_TYPE_SIGNAL],
	pPreferenceFileTypeAddedit->beforetype,
	new_type);

    /* if we are in edit mode and the new creation has worked */
    if(pPreferenceFileTypeAddedit->lasttype) { /* we are in edit mode */
      /* alerte the parent that we are going to suppress a type */
      gtk_signal_emit(GTK_OBJECT(pPreferenceFileTypeAddedit),
        preference_file_type_addedit_signals[REMOVE_TYPE_SIGNAL],
	  pPreferenceFileTypeAddedit->lasttype);
      /* remove the type */	  
      file_type_remove_type(pPreferenceFileTypeAddedit->to_use_file_type,
        pPreferenceFileTypeAddedit->lasttype);
    }

    gnome_dialog_close(GNOME_DIALOG(pPreferenceFileTypeAddedit));
  } else { /* we must alerte the user that the icon doesn't exist in */
           /* "icon_path"icons directory */ 
    preference_file_type_addedit_icon_message(icon);
  }

  /* if we have allocated memory for icon : free */
  if(pPreferenceFileTypeAddedit->icon_path != NULL) g_free(icon);
}
/*************************************/

/*************************************************/
/* Button cancel pressed in the Add or Edit mode */
void preference_file_type_addedit_cancel_cb(GtkWidget *widget,
  PreferenceFileTypeAddedit *pPreferenceFileTypeAddedit)
{ gnome_dialog_close (GNOME_DIALOG(pPreferenceFileTypeAddedit)); }
/*************************************************/
