/****************************************/
/*  PreferenceGeneral.c body            */
/*					*/
/*  Description : The General part of   */
/*		  the preference dialog	*/
/*					*/
/*  Written by : Daniel Lacroix	        */
/*  GFile is under the GPL2 licence     */
/****************************************/

#include "GFile.h"

guint preference_general_get_type ();
GtkWidget *preference_general_new(GtkWidget *parent_window,own *in_use_owner,
  guint8 mode);

static void preference_general_class_init(PreferenceGeneralClass *pPreferenceGeneralClass);
static void preference_general_init(PreferenceGeneral *pPreferenceGeneral);

/*************************************************************/
guint preference_general_get_type ()
{
  static guint preference_general_type = 0;

  if (!preference_general_type)
  {
    GtkTypeInfo preference_general_info =
    {
       "PreferenceGeneral",
       sizeof (PreferenceGeneral),
       sizeof (PreferenceGeneralClass),
       (GtkClassInitFunc)  preference_general_class_init,
       (GtkObjectInitFunc) preference_general_init,
       (GtkArgSetFunc) NULL,
       (GtkArgGetFunc) NULL
     };
     preference_general_type = gtk_type_unique(gtk_vbox_get_type(), &preference_general_info);
  }
  return preference_general_type;
}
/*************************************************************/

/*************************************************************/
enum { MODE_CHANGED_SIGNAL, LAST_SIGNAL};
static gint preference_general_signals[LAST_SIGNAL] = { 0 };

static void guint8_marshal(GtkObject *object, GtkSignalFunc func, gpointer func_data, GtkArg *args) {
    typedef void (*guint8_marshal_func)(PreferenceGeneral *, guint8,gpointer);
    guint8_marshal_func rfunc = (guint8_marshal_func) func;
    (*rfunc) (PREFERENCE_GENERAL(object),
	      GTK_VALUE_UCHAR(args[0]),func_data );
}

static void preference_general_class_init(PreferenceGeneralClass *class)
{
  /* here we create the new signals if we want */
  GtkObjectClass *object_class;
  object_class = (GtkObjectClass*) class;

  preference_general_signals[MODE_CHANGED_SIGNAL] = gtk_signal_new
            ("mode_changed",
            GTK_RUN_FIRST,object_class->type,
            GTK_SIGNAL_OFFSET (PreferenceGeneralClass,mode_changed),
            guint8_marshal, GTK_TYPE_NONE, 1,GTK_TYPE_UCHAR);
	    
   gtk_object_class_add_signals(object_class, 
     preference_general_signals, LAST_SIGNAL);
   class->mode_changed = NULL;
    
}
/*************************************************************/

static void unselect_last(PreferenceGeneral *pPreferenceGeneral)
{
  switch(pPreferenceGeneral->mode)
  {
    case MODE_TREE_LIST :     
      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(
        pPreferenceGeneral->button_tree_list),0);
      break;	
    case MODE_LIST_LIST :     
      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(
        pPreferenceGeneral->button_list_list),0);
      break;
    case MODE_LIST :             
      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(
        pPreferenceGeneral->button_list),0);
      break;	
  }  
}

static void preference_general_cb(GtkWidget *Widget,
  PreferenceGeneral *pPreferenceGeneral)
{
  if(pPreferenceGeneral->button_tree_list == Widget) 
  {
    /*g_print("button tree list pressed\n");*/
    unselect_last(pPreferenceGeneral);
    pPreferenceGeneral->mode = MODE_TREE_LIST;
  } else 
  if(pPreferenceGeneral->button_list_list == Widget)
  {
    /*g_print("button list list pressed\n");*/
    unselect_last(pPreferenceGeneral);
    pPreferenceGeneral->mode = MODE_LIST_LIST;
  } else 
  if(pPreferenceGeneral->button_list == Widget)
  {
    /*g_print("button list pressed\n");*/
    unselect_last(pPreferenceGeneral);
    pPreferenceGeneral->mode = MODE_LIST;
  }
  gtk_signal_emit(GTK_OBJECT(pPreferenceGeneral),
    preference_general_signals[MODE_CHANGED_SIGNAL],
    pPreferenceGeneral->mode);
}  

/* use as parameters given by preference_general_new to preference_general_init */
static guint8    current_mode;
static GtkWidget *current_parent_window;
static own       *current_in_use_owner;

#include "gfile_logo.xpm"

static void preference_general_init(PreferenceGeneral *pPreferenceGeneral)
{ gchar     *text_label;
  GtkWidget *label;
  GtkWidget *frame;
  GtkWidget *hbox;
  GdkPixmap *logo_pixmap;
  GdkBitmap *logo_mask;
  GtkWidget *logo_pixmap_widget;
  
  /* take parameters in account */
  pPreferenceGeneral->mode = current_mode;
  pPreferenceGeneral->in_use_owner = current_in_use_owner;

  gtk_box_set_homogeneous(GTK_BOX(pPreferenceGeneral),FALSE);
  gtk_box_set_spacing(GTK_BOX(pPreferenceGeneral),0);  
  
  gtk_container_border_width(GTK_CONTAINER(pPreferenceGeneral), 10);

  hbox = gtk_hbox_new(0,0);
  gtk_container_border_width(GTK_CONTAINER(hbox), 10);  
  gtk_widget_show(hbox);

  gtk_box_pack_start(GTK_BOX(pPreferenceGeneral), GTK_WIDGET(
    hbox), TRUE, TRUE, 0);

  if(pPreferenceGeneral->in_use_owner != NULL)
  {
    text_label = (gchar *)g_malloc(25+
      strlen(pPreferenceGeneral->in_use_owner->name)+
      strlen(pPreferenceGeneral->in_use_owner->grp->name));
    if(text_label == NULL){gfile_error(MEM_ALLOCATION); exit(MEM_ALLOCATION);}
    sprintf(text_label,"You are %s\n in the %s group\n",
      pPreferenceGeneral->in_use_owner->name,
      pPreferenceGeneral->in_use_owner->grp->name);
    label = gtk_label_new(text_label);
    g_free(text_label);
  } else {
    label = gtk_label_new("Can't find who you are.");
  }  
  gtk_label_set_justify(GTK_LABEL(label),GTK_JUSTIFY_LEFT);

  gtk_widget_show(label);

  gtk_box_pack_start(GTK_BOX(hbox), GTK_WIDGET(
    label), TRUE, TRUE, 0);
    
  /*** create the logo ***/
  logo_pixmap = gdk_pixmap_create_from_xpm_d (
    current_parent_window->window,&logo_mask,NULL,gfile_logo);
		    
  if(logo_pixmap == NULL) {gfile_error(INTERNAL_PIXMAP); exit(INTERNAL_PIXMAP);}
  /* create a widget with the pixmap */
  logo_pixmap_widget = gtk_pixmap_new(logo_pixmap,logo_mask);
  /* free the pixmap */
  gdk_pixmap_unref(logo_pixmap); gdk_bitmap_unref(logo_mask);

  gtk_widget_show(logo_pixmap_widget);

  gtk_box_pack_end(GTK_BOX(hbox), GTK_WIDGET(
    logo_pixmap_widget), TRUE, TRUE, 0);

  frame = gtk_frame_new("Choose mode");
  gtk_widget_show(frame);

  gtk_box_pack_end(GTK_BOX(pPreferenceGeneral), GTK_WIDGET(
    frame), TRUE, TRUE, 0);

  hbox = gtk_hbox_new(1,0);
  gtk_container_border_width(GTK_CONTAINER(hbox), 10);  
  gtk_widget_show(hbox);

  gtk_container_add(GTK_CONTAINER(frame), GTK_WIDGET(hbox));
  
  pPreferenceGeneral->button_tree_list = gtk_toggle_button_new_with_label("tree list");
  pPreferenceGeneral->button_list_list = gtk_toggle_button_new_with_label("list list");
  pPreferenceGeneral->button_list      = gtk_toggle_button_new_with_label("list");  
  gtk_widget_show(pPreferenceGeneral->button_tree_list);
  gtk_widget_show(pPreferenceGeneral->button_list_list);
  gtk_widget_show(pPreferenceGeneral->button_list);

  gtk_signal_connect(GTK_OBJECT(pPreferenceGeneral->button_tree_list),"pressed",
    GTK_SIGNAL_FUNC(preference_general_cb),pPreferenceGeneral);
  gtk_signal_connect(GTK_OBJECT(pPreferenceGeneral->button_list_list),"pressed",
    GTK_SIGNAL_FUNC(preference_general_cb),pPreferenceGeneral);
  gtk_signal_connect(GTK_OBJECT(pPreferenceGeneral->button_list),"pressed",
    GTK_SIGNAL_FUNC(preference_general_cb),pPreferenceGeneral);    

  /* init at the choosen mode */
  switch(pPreferenceGeneral->mode)
  {
    case MODE_TREE_LIST :     
      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(
        pPreferenceGeneral->button_tree_list),1);
      break;	
    case MODE_LIST_LIST :     
      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(
        pPreferenceGeneral->button_list_list),1);
      break;
    case MODE_LIST :             
      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(
        pPreferenceGeneral->button_list),1);
      break;	
  }  

  gtk_box_pack_start(GTK_BOX(hbox), GTK_WIDGET(
    pPreferenceGeneral->button_tree_list), TRUE, TRUE, 0);
  gtk_box_pack_start(GTK_BOX(hbox), GTK_WIDGET(
    pPreferenceGeneral->button_list_list), TRUE, TRUE, 0);
  gtk_box_pack_end(GTK_BOX(hbox), GTK_WIDGET(
    pPreferenceGeneral->button_list), TRUE, TRUE, 0);    
}


/******************************************/
/* Create a new preference general widget */
/* Return : the widget			  */
GtkWidget *preference_general_new(GtkWidget *parent_window,own *in_use_owner,
  guint8 mode)
{
  current_mode = mode;
  current_parent_window = parent_window;
  current_in_use_owner = in_use_owner;
  return( GTK_WIDGET(gtk_type_new( preference_general_get_type() )) );
}
/******************************************/

/***********************************/
/* Return the general mode choosen */
guint8 preference_general_mode(PreferenceGeneral *pPreferenceGeneral)
{
  return(pPreferenceGeneral->mode);
}
/***********************************/
