/****************************************/
/*  PreferenceList.c body               */
/*					*/
/*  Description : The List part of      */
/*		  the preference dialog	*/
/*					*/
/*  Written by : Daniel Lacroix	        */
/*  GFile is under the GPL2 licence     */
/****************************************/

#include "GFile.h"

guint preference_list_get_type ();
GtkWidget *preference_list_new(guint8 sort_mode);

static void preference_list_class_init(PreferenceListClass *pPreferenceListClass);
static void preference_list_init(PreferenceList *pPreferenceList);

/*************************************************************/
guint preference_list_get_type ()
{
  static guint preference_list_type = 0;

  if (!preference_list_type)
  {
    GtkTypeInfo preference_list_info =
    {
       "PreferenceList",
       sizeof (PreferenceList),
       sizeof (PreferenceListClass),
       (GtkClassInitFunc)  preference_list_class_init,
       (GtkObjectInitFunc) preference_list_init,
       (GtkArgSetFunc) NULL,
       (GtkArgGetFunc) NULL
     };
     preference_list_type = gtk_type_unique(gtk_vbox_get_type(), &preference_list_info);
  }
  return preference_list_type;
}
/*************************************************************/

/*************************************************************/
static void preference_list_class_init(PreferenceListClass *class)
{
  /* here we create the new signals if we want */    
}
/*************************************************************/

static void sort_mode_change_cb(GtkWidget *Widget,PreferenceList *pPreferenceList)
{ guint8 local_sort_mode;

  if(Widget == pPreferenceList->dir_prior)
  {
    if(pPreferenceList->sort_mode & SORT_DIR_FIRST)
      pPreferenceList->sort_mode -= SORT_DIR_FIRST;
    else
      pPreferenceList->sort_mode += SORT_DIR_FIRST;
    /*g_print("Directory priority changed\n");*/
    return;
  }
  if(Widget == pPreferenceList->invert)
  {
    if(pPreferenceList->sort_mode & SORT_INVERT_ORDER)
      pPreferenceList->sort_mode -= SORT_INVERT_ORDER;
    else
      pPreferenceList->sort_mode += SORT_INVERT_ORDER;
    /*g_print("Invert changed\n");*/
    return;
  }
    
  if(Widget == pPreferenceList->sort_none)
    local_sort_mode = SORT_NONE; else
  if(Widget == pPreferenceList->sort_by_name)
    local_sort_mode = SORT_BY_NAME; else
  if(Widget == pPreferenceList->sort_by_owner)
    local_sort_mode = SORT_BY_OWNER; else
  if(Widget == pPreferenceList->sort_by_group)
    local_sort_mode = SORT_BY_GROUP; else        
  if(Widget == pPreferenceList->sort_by_rights)
    local_sort_mode = SORT_BY_RIGHTS; else
  if(Widget == pPreferenceList->sort_by_size)
    local_sort_mode = SORT_BY_SIZE; else
  if(Widget == pPreferenceList->sort_by_access_date)
    local_sort_mode = SORT_BY_ACCESS_DATE; else
  if(Widget == pPreferenceList->sort_by_modif_date)
    local_sort_mode = SORT_BY_MODIF_DATE; else
  if(Widget == pPreferenceList->sort_by_change_date)
    local_sort_mode = SORT_BY_CHANGE_DATE;

  if(pPreferenceList->sort_mode & SORT_INVERT_ORDER) 
    local_sort_mode += SORT_INVERT_ORDER;
  if(pPreferenceList->sort_mode & SORT_DIR_FIRST) 
    local_sort_mode += SORT_DIR_FIRST;
  pPreferenceList->sort_mode = local_sort_mode;    
  /*g_print("Mode changed. New : %u\n",local_sort_mode);*/
}

/* use as parameters given by preference_list_new to preference_list_init */
static guint8 current_sort_mode;

/*************************************************************/
static void preference_list_init(PreferenceList *pPreferenceList)
{
  GtkWidget *label;
  GtkWidget *frame;
  GtkWidget *hbox;
  GtkWidget *menu;    
  guint8    tmp_sort_mode;
    
  /* take parameters in account */
  pPreferenceList->sort_mode = current_sort_mode;
  
  gtk_box_set_homogeneous(GTK_BOX(pPreferenceList),FALSE);
  gtk_box_set_spacing(GTK_BOX(pPreferenceList),0);  
  
  gtk_container_border_width(GTK_CONTAINER(pPreferenceList), 10);

  frame = gtk_frame_new("Sort mode");
  gtk_widget_show(frame);
  
  gtk_box_pack_start(GTK_BOX(pPreferenceList), GTK_WIDGET(
    frame), TRUE, FALSE, 0);

  hbox = gtk_hbox_new(1,0);
  gtk_container_border_width(GTK_CONTAINER(hbox), 10);  
  gtk_widget_show(hbox);

  gtk_container_add(GTK_CONTAINER(frame), GTK_WIDGET(hbox));
  
  pPreferenceList->dir_prior = gtk_check_button_new_with_label("directory priority");
  pPreferenceList->invert    = gtk_check_button_new_with_label("invert mode");
  pPreferenceList->mode_list = gtk_option_menu_new();
  gtk_widget_show(pPreferenceList->dir_prior);
  gtk_widget_show(pPreferenceList->invert);
  gtk_widget_show(pPreferenceList->mode_list);
  gtk_box_pack_start(GTK_BOX(hbox), GTK_WIDGET(
    pPreferenceList->dir_prior), TRUE, TRUE, 0);
  gtk_box_pack_start(GTK_BOX(hbox), GTK_WIDGET(
    pPreferenceList->invert), TRUE, TRUE, 0);
  gtk_box_pack_start(GTK_BOX(hbox), GTK_WIDGET(
    pPreferenceList->mode_list), TRUE, TRUE, 0);

  menu = gtk_menu_new();
  gtk_widget_show(menu);
  
  gtk_option_menu_set_menu(GTK_OPTION_MENU(pPreferenceList->mode_list),menu);     
  pPreferenceList->sort_none = gtk_menu_item_new_with_label("none");
  gtk_signal_connect(GTK_OBJECT(pPreferenceList->sort_none),"activate",
    GTK_SIGNAL_FUNC(sort_mode_change_cb),pPreferenceList);
  gtk_widget_show(pPreferenceList->sort_none);
  gtk_menu_append(GTK_MENU(menu),pPreferenceList->sort_none);
  
  pPreferenceList->sort_by_name = gtk_menu_item_new_with_label("name");
  gtk_signal_connect(GTK_OBJECT(pPreferenceList->sort_by_name),"activate",
    GTK_SIGNAL_FUNC(sort_mode_change_cb),pPreferenceList);
  gtk_widget_show(pPreferenceList->sort_by_name);
  gtk_menu_append(GTK_MENU(menu),pPreferenceList->sort_by_name);
  
  pPreferenceList->sort_by_owner = gtk_menu_item_new_with_label("owner");
  gtk_signal_connect(GTK_OBJECT(pPreferenceList->sort_by_owner),"activate",
    GTK_SIGNAL_FUNC(sort_mode_change_cb),pPreferenceList);  
  gtk_widget_show(pPreferenceList->sort_by_owner);
  gtk_menu_append(GTK_MENU(menu),pPreferenceList->sort_by_owner);

  pPreferenceList->sort_by_group = gtk_menu_item_new_with_label("group");
  gtk_signal_connect(GTK_OBJECT(pPreferenceList->sort_by_group),"activate",
    GTK_SIGNAL_FUNC(sort_mode_change_cb),pPreferenceList);  
  gtk_widget_show(pPreferenceList->sort_by_group);
  gtk_menu_append(GTK_MENU(menu),pPreferenceList->sort_by_group);
  
  pPreferenceList->sort_by_rights = gtk_menu_item_new_with_label("rights");
  gtk_signal_connect(GTK_OBJECT(pPreferenceList->sort_by_rights),"activate",
    GTK_SIGNAL_FUNC(sort_mode_change_cb),pPreferenceList);  
  gtk_widget_show(pPreferenceList->sort_by_rights);
  gtk_menu_append(GTK_MENU(menu),pPreferenceList->sort_by_rights);
  
  pPreferenceList->sort_by_size = gtk_menu_item_new_with_label("size");
  gtk_signal_connect(GTK_OBJECT(pPreferenceList->sort_by_size),"activate",
    GTK_SIGNAL_FUNC(sort_mode_change_cb),pPreferenceList);  
  gtk_widget_show(pPreferenceList->sort_by_size);
  gtk_menu_append(GTK_MENU(menu),pPreferenceList->sort_by_size);
  
  pPreferenceList->sort_by_access_date = gtk_menu_item_new_with_label("access date");
  gtk_signal_connect(GTK_OBJECT(pPreferenceList->sort_by_access_date),"activate",
    GTK_SIGNAL_FUNC(sort_mode_change_cb),pPreferenceList);  
  gtk_widget_show(pPreferenceList->sort_by_access_date);
  gtk_menu_append(GTK_MENU(menu),pPreferenceList->sort_by_access_date);
  
  pPreferenceList->sort_by_modif_date = gtk_menu_item_new_with_label("modif date");
  gtk_signal_connect(GTK_OBJECT(pPreferenceList->sort_by_modif_date),"activate",
    GTK_SIGNAL_FUNC(sort_mode_change_cb),pPreferenceList);  
  gtk_widget_show(pPreferenceList->sort_by_modif_date);
  gtk_menu_append(GTK_MENU(menu),pPreferenceList->sort_by_modif_date);
  
  pPreferenceList->sort_by_change_date = gtk_menu_item_new_with_label("change date");
  gtk_signal_connect(GTK_OBJECT(pPreferenceList->sort_by_change_date),"activate",
    GTK_SIGNAL_FUNC(sort_mode_change_cb),pPreferenceList);  
  gtk_widget_show(pPreferenceList->sort_by_change_date);
  gtk_menu_append(GTK_MENU(menu),pPreferenceList->sort_by_change_date);
  
  label = gtk_label_new("Test label.");
  gtk_widget_show(label);
  
  gtk_box_pack_end(GTK_BOX(pPreferenceList), GTK_WIDGET(
    label), TRUE, TRUE, 0);

  /* set the last choosen preference */
  tmp_sort_mode = pPreferenceList->sort_mode;
  if(tmp_sort_mode & SORT_INVERT_ORDER) 
  {
    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(pPreferenceList->invert),1);
    tmp_sort_mode -= SORT_INVERT_ORDER;
  }  
  if(tmp_sort_mode & SORT_DIR_FIRST)
  {
    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(pPreferenceList->dir_prior),1);
    tmp_sort_mode -= SORT_DIR_FIRST;
  }  
  gtk_option_menu_set_history(GTK_OPTION_MENU(pPreferenceList->mode_list),
    tmp_sort_mode);    

  gtk_signal_connect(GTK_OBJECT(pPreferenceList->dir_prior),"toggled",
    GTK_SIGNAL_FUNC(sort_mode_change_cb),pPreferenceList);
  gtk_signal_connect(GTK_OBJECT(pPreferenceList->invert),"toggled",
    GTK_SIGNAL_FUNC(sort_mode_change_cb),pPreferenceList); 
}
/*************************************************************/

/***************************************/
/* Create a new preference list widget */
/* Return : the widget                 */
GtkWidget *preference_list_new(guint8 sort_mode)
{ 
  current_sort_mode = sort_mode;
  return( GTK_WIDGET(gtk_type_new( preference_list_get_type() )) );
}
/***************************************/

/****************************************/
/* Return the current choosen sort mode */
guint8 preference_list_sort_mode(PreferenceList *pPreferenceList)
{
  return(pPreferenceList->sort_mode);
}
/****************************************/
